Package: mpmaggregate
Type: Package
Title: Aggregate Matrix Population Models
Version: 0.2.6
Date: 2026-05-23
Author: Richard A. Hinrichsen [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0761-3005>),
  Roberto Salguero-Gomez [aut] (ORCID:
    <https://orcid.org/0000-0002-6085-4433>),
  Hiroyuki Yokomizo [aut] (ORCID:
    <https://orcid.org/0000-0001-9814-8465>)
Maintainer: Richard A. Hinrichsen <r.hinrichsen.ecology@gmail.com>
Authors@R: c(
      person(c("Richard", "A."), "Hinrichsen", , "r.hinrichsen.ecology@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0761-3005")),
      person("Roberto", "Salguero-Gomez", , "rob.salguero@zoo.ox.ac.uk", role = "aut",
           comment = c(ORCID = "0000-0002-6085-4433")),
      person("Hiroyuki", "Yokomizo", ,"hiroyuki.yokomizo@nies.go.jp", role="aut",
           comment = c(ORCID = "0000-0001-9814-8465"))
  )
Description: Aggregates matrix population models (MPMs) in both the lambda (stable 
    growth rate) and R0 (net reproductive rate) frameworks, including standard and 
    elasticity-consistent aggregators. Standard aggregation in the lambda framework 
    maintains consistent lambda and stable stage distribution, while standard aggregation 
    in the R0 framework maintains consistent R0 and cohort stable stage distribution. 
    Elasticity-consistent aggregators maintain these same consistencies with respect
    to the chosen framework and additionally preserve consistent reproductive values in 
    the lambda framework and cohort reproductive values in the R0 framework. Aggregation 
    can take the form of general-to-general MPM (mpm_aggregate) or Leslie-to-Leslie
    MPM (leslie_aggregate).
License: MIT + file LICENSE
Encoding: UTF-8
Imports: expm
Suggests: testthat (>= 3.0.0), roxygen2, knitr, rmarkdown, Rcompadre,
        kableExtra, rphylopic, collidr, png
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 01:43:31 UTC; Rich
Repository: CRAN
Date/Publication: 2026-05-24 09:40:02 UTC
Built: R 4.5.2; ; 2026-05-24 13:52:00 UTC; unix

Package: mpmaggregate
Type: Package
Title: Aggregate Matrix Population Models
Version: 0.2.6
Date: 2026-05-23
Author: Richard A. Hinrichsen [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0761-3005>),
  Roberto Salguero-Gomez [aut] (ORCID:
    <https://orcid.org/0000-0002-6085-4433>),
  Hiroyuki Yokomizo [aut] (ORCID:
    <https://orcid.org/0000-0001-9814-8465>)
Maintainer: Richard A. Hinrichsen <r.hinrichsen.ecology@gmail.com>
Authors@R: c(
      person(c("Richard", "A."), "Hinrichsen", , "r.hinrichsen.ecology@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0761-3005")),
      person("Roberto", "Salguero-Gomez", , "rob.salguero@zoo.ox.ac.uk", role = "aut",
           comment = c(ORCID = "0000-0002-6085-4433")),
      person("Hiroyuki", "Yokomizo", ,"hiroyuki.yokomizo@nies.go.jp", role="aut",
           comment = c(ORCID = "0000-0001-9814-8465"))
  )
Description: Aggregates matrix population models (MPMs) in both the lambda (stable 
    growth rate) and R0 (net reproductive rate) frameworks, including standard and 
    elasticity-consistent aggregators. Standard aggregation in the lambda framework 
    maintains consistent lambda and stable stage distribution, while standard aggregation 
    in the R0 framework maintains consistent R0 and cohort stable stage distribution. 
    Elasticity-consistent aggregators maintain these same consistencies with respect
    to the chosen framework and additionally preserve consistent reproductive values in 
    the lambda framework and cohort reproductive values in the R0 framework. Aggregation 
    can take the form of general-to-general MPM (mpm_aggregate) or Leslie-to-Leslie
    MPM (leslie_aggregate).
License: MIT + file LICENSE
Encoding: UTF-8
Imports: expm
Suggests: testthat (>= 3.0.0), roxygen2, knitr, rmarkdown, Rcompadre,
        kableExtra, rphylopic, collidr, png
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 01:43:31 UTC; Rich
Repository: CRAN
Date/Publication: 2026-05-24 09:40:02 UTC
Built: R 4.6.0; ; 2026-05-25 03:25:35 UTC; unix

# Generated by roxygen2: do not edit by hand

export(dominant_eigen)
export(generation_time)
export(is_leslie)
export(leslie_aggregate)
export(leslie_disaggregate)
export(leslie_dominant_eigen)
export(leslie_reproductive_values)
export(leslie_stable_age)
export(mpm_aggregate)
export(mpm_elasticity)
export(mpm_partition)
export(reproductive_values)
export(spectral_radius)
export(stable_stage)

# mpmaggregate

Tools for aggregating matrix population models while preserving key demographic properties.

Matrix population models (MPMs) are widely used to describe population dynamics,
but models often differ in their projection intervals, stage definitions, and
levels of complexity. These differences can make demographic models difficult
to compare directly across studies. The **mpmaggregate** package provides tools
for aggregating MPMs while preserving key demographic quantities under different
demographic frameworks. This allows finer-scale models to be reduced to simpler
representations that remain comparable across studies.

## Installation

### Install from CRAN (recommended)

```r
install.packages("mpmaggregate")
```

### Install a specific release (includes vignettes)

```r
install.packages(
  "https://github.com/hinrich62/mpmaggregate/releases/download/v0.2.6/mpmaggregate_0.2.6.tar.gz",
  repos = NULL,
  type = "source"
)

library(mpmaggregate)

# Open package vignettes
browseVignettes("mpmaggregate")
```

This installs a pre-built version of the package that includes the vignettes.


## Overview

The package provides tools for aggregating matrix population models (MPMs)
under different demographic frameworks and criteria.

The main functions are:

- `mpm_aggregate()` — aggregate general stage-structured MPMs using user-defined groupings  
- `leslie_aggregate()` — aggregate age-structured Leslie matrices into coarser age classes  
- `mpm_elasticity()` — compute λ- or R₀-based elasticities of matrix elements  

In addition, aggregation functions return effectiveness metrics that quantify
the agreement between the original and aggregated models.

See the package vignettes for a detailed introduction and worked examples.

## Minimal Example

```r
library(mpmaggregate)

# Simple 2x2 projection matrix
A <- matrix(
  c(0.2, 0.1,
    0.3, 0.4),
  nrow = 2,
  byrow = TRUE
)

# Compute elasticities under the lambda framework
out <- mpm_elasticity(matA = A, framework = "lambda")

out$elasticity
```

## Documentation

After installing the package, the vignettes can be viewed with:

```r
browseVignettes("mpmaggregate")
```

## Testing

The package includes automated tests written with **testthat** and namespace
collision checks using **collidr**. These tests help ensure that functions
behave as expected and that exported names do not conflict with other packages.

To run the unit tests from the package root:

```r
devtools::test()
```

To perform a full diagnostic build and check of the package:

```r
devtools::check()
```

These commands run the test suite and verify that the package builds and
passes standard R package checks.

## References

Bienvenu, F., Akçay, E., Legendre, S., and McCandlish, D. M. (2017). The genealogical
decomposition of a matrix population model with applications to the aggregation
of stages. *Theoretical Population Biology*, 115, 69–80. 
https://doi.org/10.1016/j.tpb.2017.04.002

Hinrichsen, R. A., Yokomizo, H., and Salguero-Gómez, R. (2026). From theory to
application: Elasticity-consistent aggregation of Leslie matrix population
models for comparative demography. *bioRxiv*, preprint.
https://doi.org/10.64898/2026.02.04.703802


## Status

This package is available on CRAN and under active development. Feedback, testing, and bug reports are welcome.

## Bug Reports and Feedback

If you encounter a bug or have suggestions for improvement, please open an issue on GitHub:

https://github.com/hinrich62/mpmaggregate/issues

When reporting a bug, please include a minimal reproducible example if possible.

## Acknowledgements

R.A.H. is grateful to Charlie Johnson for suggesting Leslie model 
aggregation as a line of research during his graduate studies at Clemson University 
and to Eric Howe, Yoh Iwasa, and Simon Levin for helpful discussions. R.S.-G. was 
supported by a NERC Pushing the Frontiers grant (NE/X013766/1). H.Y. was supported 
by JSPS KAKENHI Grant Number 25K09778.


## Dedication

R. A. Hinrichsen dedicates this package to Robert Y. Dean, who introduced him to the 
Leslie matrix population model at Central Washington University and sparked a lifelong 
fascination with population dynamics.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----packages-----------------------------------------------------------------
library(mpmaggregate)
library(kableExtra)
library(rphylopic)

## ----fetch-comadre------------------------------------------------------------

# Matrices for Asian elephant populations included in this package were
# originally retrieved from COMADRE using the Rcompadre package.

# Not run: requires Rcompadre and internet access

# library(Rcompadre)
# comadre <- cdb_fetch("comadre",version = "4.23.3.1")
# mpm_elephant1 <- comadre[comadre$MatrixID == 249273, ]
# matA_elephant1 <- matA(mpm_elephant1)[[1]]             # Projection interval is 5 years
# matA_elephant1 <- matA_elephant1[1:12,1:12]            # Confine to females 
# mpm_elephant2 <- comadre[comadre$MatrixID == 249274, ]
# matA_elephant2 <- matA(mpm_elephant2)[[1]]             # Projection interval is 1 year
# matA_elephant2[60, 60] <- 0                            # Remove statis to form a true Leslie Matrix
# usethis::use_data(matA_elephant1, matA_elephant2,      # Write data (development)
# overwrite = TRUE)    

# Load data locally
data(matA_elephant1, matA_elephant2, package="mpmaggregate")
matA_coarse <- matA_elephant1 # 12x12 5-year projection interval
matA_fine <- matA_elephant2   # 60x60 1-year projection interval

## ----leslie-aggregation-------------------------------------------------------

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "lambda",
  criterion = "standard"
)
matB_lambda_stand <- res$matAk_agg
matB_lambda_stand_eff <- res$effectiveness

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "lambda",
  criterion = "elasticity"
)
matB_lambda_elast <- res$matAk_agg
matB_lambda_elast_eff <- res$effectiveness

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "R0",
  criterion = "standard"
)
matB_R0_stand <- res$matAk_agg
matB_R0_stand_eff<- res$effectiveness

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "R0",
  criterion = "elasticity"
)
matB_R0_elast <- res$matAk_agg
matB_R0_elast_eff <- res$effectiveness

## ----leslie-metrics-----------------------------------------------------------

# Decompose a Leslie matrix into its components U and F
leslie_UF <- function(A) {
  F <- matrix(0, nrow(A), ncol(A))
  F[1, ] <- A[1, ]
  U <- A
  U[1, ] <- 0
  list(U = U, F = F)
}

# Return R0, the net reproductive rate
get_R0_leslie <- function(A) {
  uf <- leslie_UF(A)
  U <- uf$U
  F <- uf$F
  
  I <- diag(nrow(U))
  N <- solve(I - U)
  K <- F %*% N
  spectral_radius(K)
}

# Return generation time in the lambda framework
get_generation_time_lambda <- function(A) {
  uf <- leslie_UF(A)
  U <- uf$U
  F <- uf$F
  I <- diag(nrow(U))
  N <- solve(I - U)
  gt = generation_time(F, U, framework = "lambda")$generation_time
  return(gt)
}

# Return cohort generation time
get_generation_time_R0 <- function(A) {
  uf <- leslie_UF(A)
  U <- uf$U
  F <- uf$F
  I <- diag(nrow(U))
  N <- solve(I - U)
  gt = generation_time(F, U, framework = "R0")$generation_time
  return(gt)
}


## ----define-models------------------------------------------------------------
models <- list(
  "Orig. coarse"             = matA_coarse,
  "Agg λ + standard"         = matB_lambda_stand,
  "Agg λ + elasticity"       = matB_lambda_elast,
  "Agg R0 + standard"        = matB_R0_stand,
  "Agg R0 + elasticity"      = matB_R0_elast
)

## ----lambda - r0 - table------------------------------------------------------

#Form the data frame that contains the table information
results <- data.frame(
  Model  = names(models),
  lambda = sapply(models, spectral_radius),
  R0     = sapply(models, get_R0_leslie),
  Ta     = sapply(models, get_generation_time_lambda)*5,  #measure in years
  Tc     = sapply(models, get_generation_time_R0)*5,      #measure in years
  Eff    = c(NA,matB_lambda_stand_eff,matB_lambda_elast_eff,
             matB_R0_stand_eff,matB_R0_elast_eff),
  row.names = NULL,
  stringsAsFactors = FALSE
)

n_rows <- nrow(results)

#Construct a table using function kable
knitr::kable(
  results,
  col.names = c(
    "Model",
    "<em>&lambda;</em>",
    "<em>R</em><sub>0</sub>",
    "<em>T</em><sub>a</sub> (years)",
    "<em>T</em><sub>c</sub> (years)",
    "<em>&rho;</em><sup>2</sup>"
  ),
  digits = 3,
  caption = paste0(
  "<strong>Table 1.</strong> Demographic comparison between two Asian elephant populations, ",
  "represented by an original coarse-stage Leslie model from Nagarhole National Park ",
  "(MatrixID 249273) and aggregated versions of a Leslie model from Periyar Reserve ",
  "(MatrixID 249274). Metrics include population growth rate (<em>&lambda;</em>), ",
  "net reproductive rate (<em>R</em><sub>0</sub>), generation time (years; ",
  "<em>T</em><sub>a</sub>), cohort generation time (years; ",
  "<em>T</em><sub>c</sub>), and aggregation effectiveness (<em>&rho;</em><sup>2</sup>). ",
  "&ldquo;Standard&rdquo; denotes use of a standard aggregator and ",
  "&ldquo;elasticity&rdquo; denotes elasticity-consistent aggregation."
  ),
  format = "html",
  escape = FALSE
) |>
  kable_styling(full_width = TRUE) |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  row_spec(nrow(results), extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> The projection interval associated with the population ",
      "growth rate (<em>&lambda;</em>) is 5 years.</span>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )

## ----vital-rates-barplot, fig.height=8, fig.width=10--------------------------

#Asian elephant
#"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

# Not run: requires internet access

# uuid <- "7c9ab182-175d-4f02-96d0-09c1e5212bff"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 691)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/elephant.png")

#Load the image file that is stored locally 
img_file <- system.file("extdata", "elephant.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/elephant.png"
}

img <- png::readPNG(img_file)
# ---- Helpers to extract Leslie vital rates from A ----
extract_fertility <- function(A) {
  as.numeric(A[1, ])
}

extract_survival <- function(A) {
  n <- nrow(A)
  s_sub  <- A[cbind(2:n, 1:(n - 1))]  # A[i+1, i]
  c(s_sub)
}

model_names <- names(models)

# Choose distinct colors for the 5 models (base R friendly)
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Build rate-by-model matrices so:
# rows = models (5 bars), cols = vital rates (groupings)
F_by_model <- t(sapply(models, extract_fertility))  # 5 x n
S_by_model <- t(sapply(models, extract_survival))   # 5 x n-1

# Name columns (vital rates)
n <- nrow(models[[1]])
colnames(F_by_model) <- paste0("F", 1:n)
colnames(S_by_model) <- c(paste0("S", 1:(n - 1)))

# Keep only vital rates that are nonzero in at least one model
keep_F <- colSums(abs(F_by_model) > 0) > 0
keep_S <- colSums(abs(S_by_model) > 0) > 0

F_plot <- F_by_model[, keep_F, drop = FALSE]
S_plot <- S_by_model[, keep_S, drop = FALSE]

# Leave extra space on the right for a legend that won't overlap bars
op <- par(
  mfrow = c(2, 1),
  mar = c(7, 4, 3, 7),
  oma = c(0, 0, 0, 0)
)

op <- par(
  mfrow = c(2, 1),
  mar = c(5, 4, 1.5, 7),
  oma = c(0, 0, 0, 0)
)

ylim = c(0, max(F_plot))
mydiff = ylim[2] - ylim[1]
ylim[2] = ylim[2] + .3 * mydiff

# Panel (a): Fertility rates
barplot(
  F_plot,
  ylim = ylim,
  beside = TRUE,
  col = cols,
  border = "white",
  las = 2,
  ylab = "Fertility rate",
  main = "Fertility rates"
)
mtext("(a)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .9,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

# One legend for both panels, placed in the right margin of the top panel
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.18, 0.00),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)
ylim <-  c(0, max(S_plot))
mydiff <- ylim[2] - ylim[1]
ylim[2] <- ylim[2] + .3 * mydiff

# Panel (b): Survival probs
barplot(
  S_plot,
  ylim = ylim,
  beside = TRUE,
  col = cols,
  border = "white",
  las = 2,
  ylab = "Survival probability",
  main = "Survival probabilities"
)
mtext("(b)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .9,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

# Legend for bottom panel (same placement logic)
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.18, 0.00),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

par(op)

## ----lambda-elasticities-barplot, fig.height=8, fig.width=10------------------
#Asian elephant
#"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

# Not run: requires internet access

# uuid <- "7c9ab182-175d-4f02-96d0-09c1e5212bff"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 691)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/elephant.png")

#Load a local copy of the image file
img_file <- system.file("extdata", "elephant.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/elephant.png"
}

img <- png::readPNG(img_file)

# Compute elasticity matrices for each model (same dimensions as A)
E_models <- lapply(models, function(A) {
  E <- mpm_elasticity(A, framework = "lambda")$elasticity
  as.matrix(E)
})

# ---- Helpers to extract Leslie elasticities from an elasticity matrix ----
extract_fertility_elast <- function(E) {
  as.numeric(E[1, ])
}

extract_survival_elast <- function(E) {
  n <- nrow(E)
  e_sub  <- E[cbind(2:n, 1:(n - 1))]  # elasticity of A[i+1, i]
  as.numeric(e_sub)
}

model_names <- names(models)

# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Build elasticity-by-model matrices so:
# rows = models (5 bars), cols = vital rates (groupings)
F_el_by_model <- t(sapply(E_models, extract_fertility_elast))  # 5 x n
S_el_by_model <- t(sapply(E_models, extract_survival_elast))   # 5 x n-1

# Name columns (vital rates)
n <- nrow(models[[1]])
colnames(F_el_by_model) <- paste0("F", 1:n)
colnames(S_el_by_model) <- c(paste0("S", 1:(n - 1)))

# Keep only elasticities that are nonzero in at least one model
keep_F <- colSums(abs(F_el_by_model) > 0) > 0
keep_S <- colSums(abs(S_el_by_model) > 0) > 0

F_el_plot <- F_el_by_model[, keep_F, drop = FALSE]
S_el_plot <- S_el_by_model[, keep_S, drop = FALSE]

# Leave extra space on the right for legends that won't overlap bars
op <- par(
  mfrow = c(2, 1),
  mar = c(7, 4, 3, 7),
  oma = c(0, 0, 0, 0)
)

op <- par(
  mfrow = c(2, 1),
  mar = c(5, 4, 1.5, 7),
  oma = c(0, 0, 0, 0)
)

# Panel (a): Elasticities with respect to fertility rates
barplot(
  F_el_plot,
  beside = TRUE,
  ylim = c(0, 0.20),
  col = cols,
  border = "white",
  las = 2,
  ylab = "Elasticity of λ",
  main = "Elasticities with respect to fertility rates"
)
mtext("(a)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

# Panel (b): Elasticities with respect to survival probabilities
barplot(
  S_el_plot,
  beside = TRUE,
  ylim = c(0, .20),
  col = cols,
  border = "white",
  las = 2,
  ylab = "Elasticity of λ",
  main = "Elasticities with respect to survival probabilities"
)
mtext("(b)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

par(op)

## ----R0-elasticities-barplot, fig.height=8, fig.width=10----------------------

#Asian elephant
#"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

# Not run: requires internet access

# uuid <- "7c9ab182-175d-4f02-96d0-09c1e5212bff"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 691)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/elephant.png")


img_file <- system.file("extdata", "elephant.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/elephant.png"
}

img <- png::readPNG(img_file)

# Compute elasticity matrices for each model (same dimensions as A)
# These are elasticities of R0 wrt to matrix entries
# normalized to sum to 1.
E_models <- lapply(models, function(A) {
  n <- nrow(A)
  matU <- A
  matU[1,] <- 0
  matF <- matrix(0,n,n)
  matF[1,] <- A[1,]
  E <- mpm_elasticity(matA=NULL, matF = matF, matU = matU, 
                  framework = "R0", normalize = TRUE)$elasticity
  as.matrix(E)
})

# ---- Helpers to extract Leslie elasticities from an elasticity matrix ----
extract_fertility_elast <- function(E) {
  as.numeric(E[1, ])
}

extract_survival_elast <- function(E) {
  n <- nrow(E)
  e_sub  <- E[cbind(2:n, 1:(n - 1))]  # elasticity of A[i+1, i]
  as.numeric(e_sub)
}

model_names <- names(models)

# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Build elasticity-by-model matrices so:
# rows = models (5 bars), cols = vital rates (groupings)
F_el_by_model <- t(sapply(E_models, extract_fertility_elast))  # 5 x n
S_el_by_model <- t(sapply(E_models, extract_survival_elast))   # 5 x n-1

# Name columns (vital rates)
n <- nrow(models[[1]])
colnames(F_el_by_model) <- paste0("F", 1:n)
colnames(S_el_by_model) <- c(paste0("S", 1:(n - 1)))

# Keep only elasticities that are nonzero in at least one model
keep_F <- colSums(abs(F_el_by_model) > 0) > 0
keep_S <- colSums(abs(S_el_by_model) > 0) > 0

F_el_plot2 <- F_el_by_model[, keep_F, drop = FALSE]
S_el_plot2 <- S_el_by_model[, keep_S, drop = FALSE]

# Leave extra space on the right for legends that won't overlap bars
op <- par(
  mfrow = c(2, 1),
  mar = c(7, 4, 3, 7),
  oma = c(0, 0, 0, 0)
)

op <- par(
  mfrow = c(2, 1),
  mar = c(5, 4, 1.5, 7),
  oma = c(0, 0, 0, 0)
)

# Panel (a): Elasticities with respect to fertility rates
barplot(
  F_el_plot2,
  beside = TRUE,
  ylim = c(0, 0.20),
  col = cols,
  border = "white",
  las = 2,
  ylab = expression("Normalized elasticity of " * R[0] * ""),
  main = "Elasticities with respect to fertility rates"
)
mtext("(a)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

# Panel (b): Elasticities with respect to survival probabilities
barplot(
  S_el_plot2,
  beside = TRUE,
  ylim = c(0, .20),
  col = cols,
  border = "white",
  las = 2,
  ylab = expression("Normalized elasticity of " * R[0] * ""),
  main = "Elasticities with respect to survival probabilities"
)
mtext("(b)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

par(op)

## ----lambda-vs.-R0-elasticities-----------------------------------------------
#compare elasticities with respect to fertility rates between frameworks 
#fertility rates in lambda framework 
  sum(F_el_plot)/5
#fertility rates in R0 framework
  sum(F_el_plot2)/5

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----packages-----------------------------------------------------------------
library(mpmaggregate)
library(kableExtra)
library(rphylopic)
library(expm)

## -----------------------------------------------------------------------------
# Leslie population projection matrix for Marmota flaviventris
# MatrixID = 249455

# Not run: requires Rcompadre and internet access

# Matrices can be retrieved with:
# library(Rcompadre)
# comadre <- cdb_fetch("comadre",version = "4.23.3.1")
# mpm <- comadre[comadre$MatrixID == 249455, ]
# matA <- matA(mpm)[[1]]
# matU <- matU(mpm)[[1]]
# matF <- matF(mpm)[[1]]


# Matrices are defined locally to avoid requiring the use of the internet and Rcompadre

# Leslie population projection matrix
matA <- matrix(c(
  0.000000, 0.258040, 0.627254, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037,
  0.545125, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.498625, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.592875, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000
), nrow = 10, byrow = TRUE)


# Reproduction matrix
matF <- matrix(c(
  0, 0.25804, 0.627254, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0
), nrow = 10, byrow = TRUE)

# Survival/transition matrix
matU <- matrix(c(
  0,       0,       0,       0,      0,      0,      0,      0,      0,      0,
  0.545125,0,       0,       0,      0,      0,      0,      0,      0,      0,
  0,       0.498625,0,       0,      0,      0,      0,      0,      0,      0,
  0,       0,       0.592875,0,      0,      0,      0,      0,      0,      0,
  0,       0,       0,       0.6435, 0,      0,      0,      0,      0,      0,
  0,       0,       0,       0,      0.6435, 0,      0,      0,      0,      0,
  0,       0,       0,       0,      0,      0.6435, 0,      0,      0,      0,
  0,       0,       0,       0,      0,      0,      0.6435, 0,      0,      0,
  0,       0,       0,       0,      0,      0,      0,      0.6435, 0,      0,
  0,       0,       0,       0,      0,      0,      0,      0,      0.6435, 0
), nrow = 10, byrow = TRUE)

# Sanity check: A = U + F (after folding any C into F)
stopifnot(all.equal(unname(matA), unname(matU + matF)))
   

## ----helpers------------------------------------------------------------------
`%||%` <- function(x, y) if (!is.null(x)) x else y

# Return R0, the net reproductive rate
get_R0 <- function(U, F) {
  I <- diag(nrow(U))
  N <- solve(I - U)
  K <- F %*% N
  spectral_radius(K)
}

# Return generation time
get_Ta <- function(U, F) {
  generation_time(F, U, framework = "lambda")$generation_time
}

# Return cohort generation time
get_Tc <- function(U, F) {
  generation_time(F, U, framework = "R0")$generation_time
}

## ----aggregate----------------------------------------------------------------
# Aggregate into 5 x 5 matrices (m = 5)
ngroups <- 5

agg1 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "lambda", criterion = "standard"
)

agg2 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "lambda", criterion = "elasticity"
)

agg3 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "R0", criterion = "standard"
)

agg4 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "R0", criterion = "elasticity"
)

# Split a Leslie matrix into U and F components (Leslie form: fertility in row 1)
split_matA <- function(A) {
  n <- nrow(A)
  U <- A
  U[1, ] <- 0
  F <- matrix(0, n, n)
  F[1, ] <- A[1, ]
  list(U = U, F = F, A = A)
}

m0 <- list(U = matU, F = matF, A = matA)
m1 <- split_matA(agg1$matAk_agg)
m2 <- split_matA(agg2$matAk_agg)
m3 <- split_matA(agg3$matAk_agg)
m4 <- split_matA(agg4$matAk_agg)

eff <- c(
  NA,
  agg1$effectiveness,
  agg2$effectiveness,
  agg3$effectiveness,
  agg4$effectiveness
)

models <- list(
  Original            = m0,
  Agg_lambda_std      = m1,
  Agg_lambda_elast    = m2,
  Agg_R0_std          = m3,
  Agg_R0_elast        = m4
)

model_labels <- c(
  Original         = "Original",
  Agg_lambda_std   = "Agg &lambda; + standard",
  Agg_lambda_elast = "Agg &lambda; + elasticity",
  Agg_R0_std       = "Agg R<sub>0</sub> + standard",
  Agg_R0_elast     = "Agg R<sub>0</sub> + elasticity"
)

# Basic checks
stopifnot(all(sapply(models, function(m) !is.null(m$A))))

# Show aggregated matrices
print("Agg λ + standard");   m1$A
print("Agg λ + elasticity"); m2$A
print("Agg R0 + standard");  m3$A
print("Agg R0 + elasticity");m4$A

## ----results-table alt--------------------------------------------------------
# Adjust lambdas so that they are all per capita growth rate over a year
# Adjust generation times so they all use units of 1 year
adj = c(1,2,2,2,2)

lambda = sapply(models, function(m) spectral_radius(m$A))
lambda = lambda^(1/adj)
R0     = sapply(models, function(m) get_R0(m$U, m$F))
Ta     = sapply(models, function(m) get_Ta(m$U, m$F))*adj
Tc     = sapply(models, function(m) get_Tc(m$U, m$F))*adj

results <- data.frame(
  Model  = unname(model_labels[names(models)]),
  lambda = lambda,
  R0     = R0,
  Ta     = Ta,
  Tc     = Tc,
  Effectiveness = eff,
  row.names = NULL,
  stringsAsFactors = FALSE
)

n_rows <- nrow(results)

knitr::kable(
  results,
  col.names = c(
    "Model",
    "<em>&lambda;</em>",
    "<em>R</em><sub>0</sub>",
    "<em>T</em><sub>a</sub> (years)",
    "<em>T</em><sub>c</sub> (years)",
    "<em>&rho;</em><sup>2</sup>"
  ),
  digits = 3,
  caption = paste0(
    "<strong>Table 1.</strong> Comparison of demographic properties for the ",
    "yellow-bellied marmot <em>Marmota flaviventris</em> (COMADRE MatrixID 249455) ",
    "using the original 10 × 10 Leslie matrix and four aggregated Leslie matrices ",
    "collapsed to 5 age groups. Metrics include population growth rate ",
    "(<em>&lambda;</em>), net reproductive rate (<em>R</em><sub>0</sub>), ",
    "generation time (years; <em>T</em><sub>a</sub>), cohort generation time ",
    "(years; <em>T</em><sub>c</sub>), and aggregation effectiveness ",
    "(<em>&rho;</em><sup>2</sup>). &ldquo;Standard&rdquo; denotes use of a standard aggregator ",
    "and &ldquo;elasticity&rdquo; denotes elasticity-consistent aggregation."
  ),
  format = "html",
  escape = FALSE
) |>
  kable_styling(full_width = TRUE) |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  row_spec(nrow(results), extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> Population growth rates (<em>&lambda;</em>) are all transformed to ",
      "represent growth over 1 year. Generation times have all been transformed to ",
      "represent generation time measured in years.</span>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )

## ----elasticities-lambda^k----------------------------------------------------

# Age groups used for aggregation in example
groups <- list(c(1,2), c(3,4), c(5,6), c(7,8), c(9,10))

# Partitioning matrix induced by the age groups.
# This matrix is constructed internally by leslie_aggregate(),
# but we construct it explicitly here so it can be used to
# map quantities from the original age-class space to the
# aggregated space.
P <- mpm_partition(groups = groups, n = nrow(matA))

# Dimensions of the original and aggregated matrices
m <- ngroups
n <- nrow(matA)
k <- n / m

matAk <- matA %^% k

# Elasticity of lambda^k with respect to entries of A^k
E_A <- mpm_elasticity(matA = matAk, framework = "lambda")$elasticity

E_list <- list(
  "Original"            = P %*% E_A %*% t(P),
  "Agg λ + standard"    = mpm_elasticity(matA = m1$A, framework = "lambda")$elasticity,
  "Agg λ + elasticity"  = mpm_elasticity(matA = m2$A, framework = "lambda")$elasticity,
  "Agg R0 + standard"   = mpm_elasticity(matA = m3$A, framework = "lambda")$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matA = m4$A, framework = "lambda")$elasticity
)

to_long <- function(M, model_name) {
  idx <- which(M != 0, arr.ind = TRUE)
  data.frame(
    model = model_name,
    row = idx[, 1],
    col = idx[, 2],
    entry = paste(idx[, 1], idx[, 2], sep = ","),
    elasticity = M[idx],
    stringsAsFactors = FALSE
  )
}

elast_df <- do.call(rbind, Map(to_long, E_list, names(E_list)))
elast_df <- elast_df[elast_df$elasticity > 0, ]
elast_df <- elast_df[order(elast_df$row, elast_df$col), ]

models_order <- names(E_list)
entries <- unique(elast_df$entry)

elast_mat <- sapply(models_order, function(m) elast_df$elasticity[elast_df$model == m])
elast_mat <- t(elast_mat)
rownames(elast_mat) <- models_order
colnames(elast_mat) <- entries

## ----elasticity-plot, fig.width=12, fig.height=6------------------------------
# Not run: requires internet access

# Marmot 2012 Feb 2 by T. Michael Keesey, CC0 1.0, Marmota monax (Linnaeus 1758)
# uuid <- "eee50efb-40dc-47d0-b2cb-52a14a5e0e51"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 869)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/marmot.png")

# Load image locally
img_file <- system.file("extdata", "marmot.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/marmot.png"
}

img <- png::readPNG(img_file)

rc <- do.call(rbind, strsplit(colnames(elast_mat), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

make_entry_label <- function(r, c) {
  if (r == 1) {
    bquote(F[1, .(c)])
  } else {
    bquote(U[.(r), .(c)])
  }
}
entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)

cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat,
  beside = TRUE,
  ylim = c(0, .35),
  col = cols,
  border = NA,
  ylab = expression("Elasticity of " * lambda^k),
  xaxt = "n",
  space = c(0.2, 1)
)

axis(side = 1, at = colMeans(bp), labels = entry_labels, las = 2, cex.axis = 0.9)
legend("topleft", legend = rownames(elast_mat), fill = cols, bty = "n", inset = 0.02)

lims <- par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x = NULL,
  y = mydiff * .9,
  height = .2 * mydiff * dim(img)[1] / 1024
)

par(op)

## ----elasticities-r0----------------------------------------------------------
# Age groupings used for aggregation (reintroduced here so this section
# can be read independently)
groups <- list(c(1,2), c(3,4), c(5,6), c(7,8), c(9,10))

# Partitioning matrix induced by the age groups.
# Constructed internally by leslie_aggregate(), but reproduced here
# because it is used to map quantities to aggregated space.
P <- mpm_partition(groups = groups, n = nrow(matA))

# Dimensions of the original and aggregated matrices
# Reintroduced here for clarity
m <- ngroups
n <- nrow(matA)
k <- n / m

R0 <- spectral_radius(matF %*% solve(diag(n) - matU))

# Reference operator for aggregation eigenstructure in the R0 framework
# Divide by R0 so that it has Leslie matrix form
A_R0ref <- (matF + R0 * matU) / R0

Uk <- expm::`%^%`(matU, k)
Rk <- (expm::`%^%`(A_R0ref, k) - Uk) * R0
Ak <- Uk + Rk

# Elasticity of R0 with respect to entries of Ak (closest to what aggregated matrices yield)
E_A <- mpm_elasticity(matF = Rk, matU = Uk, framework = "R0", normalize = TRUE)$elasticity

E_list2 <- list(
  "Original"            = P %*% E_A %*% t(P),
  "Agg λ + standard"    = mpm_elasticity(matF = m1$F, matU = m1$U, framework = "R0", normalize = TRUE)$elasticity,
  "Agg λ + elasticity"  = mpm_elasticity(matF = m2$F, matU = m2$U, framework = "R0", normalize = TRUE)$elasticity,
  "Agg R0 + standard"   = mpm_elasticity(matF = m3$F, matU = m3$U, framework = "R0", normalize = TRUE)$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matF = m4$F, matU = m4$U, framework = "R0", normalize = TRUE)$elasticity
)

elast_df2 <- do.call(rbind, Map(to_long, E_list2, names(E_list2)))
elast_df2 <- elast_df2[elast_df2$elasticity > 0, ]
elast_df2 <- elast_df2[order(elast_df2$row, elast_df2$col), ]

models_order2 <- names(E_list2)
entries2 <- unique(elast_df2$entry)

elast_mat2 <- sapply(models_order2, function(m) elast_df2$elasticity[elast_df2$model == m])
elast_mat2 <- t(elast_mat2)
rownames(elast_mat2) <- models_order2
colnames(elast_mat2) <- entries2

## ----elasticity-r0-plot, fig.width=12, fig.height=6---------------------------

# Not run: requires internet access

# Marmot 2012 Feb 2 by T. Michael Keesey, CC0 1.0, Marmota monax (Linnaeus 1758)
# uuid <- "eee50efb-40dc-47d0-b2cb-52a14a5e0e51"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 869)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/marmot.png")

# Load the image file locally
img_file <- system.file("extdata", "marmot.png", package = "mpmaggregate")

#For development
if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/marmot.png"
}

img <- png::readPNG(img_file)

rc <- do.call(rbind, strsplit(colnames(elast_mat2), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)

cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat2,
  beside = TRUE,
  ylim = c(0, .35),
  col = cols,
  border = NA,
  ylab = expression("Normalized elasticity of " * R[0]),
  xaxt = "n",
  space = c(0.2, 1)
)

axis(side = 1, at = colMeans(bp), labels = entry_labels, las = 2, cex.axis = 0.9)
legend("topleft", legend = rownames(elast_mat2), fill = cols, bty = "n", inset = 0.02)

lims <- par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x = NULL,
  y = mydiff * .9,
  height = .2 * mydiff * dim(img)[1] / 1024
)

par(op)

## ----fertility-elasticity-mass------------------------------------------------
# Mass for elasticity of R0 w.r.t. fertilities
sum(elast_mat2[, 1:5]) / 5

# Mass for elasticity of lambda^k w.r.t. fertilities
sum(elast_mat[, 1:5]) / 5

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----packages-----------------------------------------------------------------
library(mpmaggregate)
library(knitr)
library(kableExtra)

## ----fetch-comadre------------------------------------------------------------
# Example matrices used in the general-to-general MPM aggregation examples below
# Population projection matrix for Rourea induta

# Not run: requires Rcompadre and internet access

# MatrixID = 246781
# Matrices can be retrieved with:
# library(Rcompadre)
# compadre <- cdb_fetch("compadre")
# mpm <- compadre[compadre$MatrixID == 246781, ]
# matA <- matA(mpm)[[1]]
# matU <- matU(mpm)[[1]]
# matF <- matF(mpm)[[1]]
# matC <- matC(mpm)[[1]]


# The matrices are defined locally so that Rcompadre and internet access
# are not required

# Population projection matrix
matA <- matrix(c(
  0.000,0.000,0.000,0.000,0.002,0.007,0.021,0.055,0.102,0.139,0.176,0.191,0.203,0.214,
  0.000,0.000,0.000,0.001,0.002,0.004,0.007,0.011,0.016,0.021,0.036,0.052,0.072,0.096,
  0.833,0.043,0.883,0.151,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.391,0.043,0.791,0.079,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.348,0.000,0.047,0.810,0.139,0.024,0.024,0.056,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.087,0.733,0.082,0.071,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.087,0.000,0.000,0.000,0.129,0.765,0.024,0.000,0.013,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.106,0.762,0.028,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.012,0.119,0.778,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.083,0.861,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.127,0.891,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.909,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.918,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.082,0.967
), nrow = 14, byrow = TRUE)

# Survival/transition matrix
matU <- matrix(c(
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.833,0.043,0.883,0.151,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.391,0.043,0.791,0.079,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.348,0.000,0.047,0.810,0.139,0.024,0.024,0.056,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.087,0.733,0.082,0.071,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.087,0.000,0.000,0.000,0.129,0.765,0.024,0.000,0.013,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.106,0.762,0.028,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.012,0.119,0.778,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.083,0.861,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.127,0.891,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.909,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.918,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.082,0.967
), nrow = 14, byrow = TRUE)

# Sexual reproduction matrix
matF <- matrix(c(
  0,0,0,0,0.002,0.007,0.021,0.055,0.102,0.139,0.176,0.191,0.203,0.214,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000
),nrow=14,byrow=TRUE)

#Clonal reproduction matrix
matC <- matrix(c(
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.001,0.002,0.004,0.007,0.011,0.016,0.021,0.036,0.052,0.072,0.096,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000
), nrow = 14, byrow = TRUE)

#redefined matF so it includes sexual + clonal reproduction
matF <- matF + matC

#sanity check
stopifnot(all.equal(unname(matA), unname(matU + matF)))

#1                active          Seedling
#2                active            Sucker
#3                active         1-1 .9 mm
#4                active         2-3 .9 mm
#5                active         4-5 .9 mm
#6                active         6-7 .9 mm
#7                active        8-.9 .9 mm
#8                active       10-14 .9 mm
#9                active        15-19.9 mm
#10               active          20-29 mm
#11               active          30-39 mm
#12               active          40-49 mm
#13               active          50-69 mm
#14               active            70+ mm

# Stage aggregation used in later examples:
# Form the aggregated groups, leaving Seedling and Sucker stages alone
groups <- list(
  c(1),                #Seedling
  c(2),                #Sucker
  c(3, 4, 5, 6),       #1-7.9 mm
  c(7, 8, 9, 10),      #8-29 mm
  c( 11, 12, 13, 14)   #30+ mm
)

## ----helpers------------------------------------------------------------------
`%||%` <- function(x, y) if (!is.null(x)) x else y

get_R0 <- function(U, F) {
  I <- diag(nrow(U))
  N <- solve(I - U)
  K <- F %*% N
  spectral_radius(K)
}

get_Ta <- function(U, F) {
  generation_time(F, U, framework = "lambda")$generation_time
}

get_Tc <- function(U, F) {
  generation_time(F, U, framework = "R0")$generation_time
}

## ----aggregate----------------------------------------------------------------
agg1 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "lambda",
  criterion = "standard"
)

agg2 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "lambda",
  criterion = "elasticity"
)

agg3 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "R0",
  criterion = "standard"
)

agg4 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "R0",
  criterion = "elasticity"
)

# Extract aggregated U and F robustly (in case object field names differ by version)
extract_U <- function(x) x$matUk_agg %||% x$matU_agg %||% x$matUagg %||% x$U
extract_F <- function(x) x$matFk_agg %||% x$matF_agg %||% x$matFagg %||% x$F
extract_A <- function(x) x$matAk_agg %||% x$matA_agg %||% x$matAagg %||% x$A

as_ufA <- function(x) {
  U <- extract_U(x)
  F <- extract_F(x)
  A <- extract_A(x)
  if (is.null(A) && !is.null(U) && !is.null(F)) A <- U + F
  list(U = U, F = F, A = A)
}

m0 <- list(U = matU, F = matF, A = matA)
m1 <- as_ufA(agg1)
m2 <- as_ufA(agg2)
m3 <- as_ufA(agg3)
m4 <- as_ufA(agg4)


eff <- c(NA,agg1$effectiveness,agg2$effectiveness,agg3$effectiveness,agg4$effectiveness)

models <- list(
  "Original"                  = m0,
  "Agg lambda + standard"     = m1,
  "Agg lambda + elasticity"   = m2,
  "Agg R0 + standard"         = m3,
  "Agg R0 + elasticity"       = m4
)

model_labels <- c(
  Original         = "Original",
  Agg_lambda_std   = "Agg &lambda; + standard",
  Agg_lambda_elast = "Agg &lambda; + elasticity",
  Agg_R0_std       = "Agg R<sub>0</sub> + standard",
  Agg_R0_elast     = "Agg R<sub>0</sub> + elasticity"
)

names(models) <- c(
  "Original",
  "Agg_lambda_std",
  "Agg_lambda_elast",
  "Agg_R0_std",
  "Agg_R0_elast"
)

# Basic checks
stopifnot(all(sapply(models, function(m) !is.null(m$A))))
#stopifnot(all(sapply(models, function(m) nrow(m$A) == length(groups))))

#show aggregated matrices
print("Agg λ + standard")
m1$A
print("Agg λ + elasticity")
m2$A
print("Agg R0 + standard")
m3$A
print("Agg R0 + elasticity")
m4$A


## ----results-table------------------------------------------------------------

results <- data.frame(
  Model  = unname(model_labels[names(models)]),
  lambda = sapply(models, function(m) spectral_radius(m$A)),
  R0     = sapply(models, function(m) get_R0(m$U, m$F)),
  Ta     = sapply(models, function(m) get_Ta(m$U, m$F)),
  Tc     = sapply(models, function(m) get_Tc(m$U, m$F)),
  Effectiveness = eff,
  row.names = NULL,
  stringsAsFactors = FALSE
)

n_rows <- nrow(results)

knitr::kable(
  results,
  col.names = c(
    "Model",
    "<em>&lambda;</em>",
    "<em>R</em><sub>0</sub>",
    "<em>T</em><sub>a</sub> (years)",
    "<em>T</em><sub>c</sub> (years)",
    "<em>&rho;</em><sup>2</sup>"
  ),
  digits = 3,
  caption = paste0(
  "<strong>Table 1.</strong> Comparison of demographic properties for the original ",
  "stage-structured matrix population model of <em>Rourea induta</em> ",
  "(COMPADRE MatrixID 246781) and four aggregated versions. The original 14 stages ",
  "are collapsed to 5 while leaving the Seedling and Sucker stages unchanged. ",
  "&ldquo;Standard&rdquo; denotes use of a standard aggregator and ",
  "&ldquo;elasticity&rdquo; denotes elasticity-consistent aggregation. ",
  "<em>T</em><sub>a</sub> is generation time (years) in the <em>&lambda;</em> framework and ",
  "<em>T</em><sub>c</sub> is cohort generation time (years) in the ",
  "<em>R</em><sub>0</sub> framework. ",
  "<em>&rho;</em><sup>2</sup> quantifies aggregation effectiveness, with values closer to 1 ",
  "indicating closer agreement between the aggregated and reference models."
  ),
  format = "html",
  escape = FALSE
) |>
  kable_styling(full_width = TRUE) |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  row_spec(nrow(results), extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> The projection interval associated with the population ",
      "growth rate (<em>&lambda;</em>) is 1 year.</span>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )

## ----elasticities-------------------------------------------------------------

# Partitioning matrix induced by the stage groups
# This matrix is constructed internally by mpm_aggregate(),
# but we reproduce it here because it can be used to map quantities
# from the original stage space to the aggregated space.
P <- mpm_partition(groups=groups, n=nrow(matA))

# Elasticity matrices in aggregated space (k x k) for each model
E_A <- mpm_elasticity(matA=matA,framework="lambda")$elasticity
E_list <- list(
  "Original" = P %*% E_A %*% t(P),
  "Agg lambda + standard" = mpm_elasticity(matA=m1$A,framework="lambda")$elasticity,
  "Agg lambda + elasticity" = mpm_elasticity(matA=m2$A,framework="lambda")$elasticity,
  "Agg R0 + standard" = mpm_elasticity(matA=m3$A,framework="lambda")$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matA=m4$A,framework="lambda")$elasticity
)

# Build a long data.frame for all nonzero entries (row-major order)
to_long <- function(M, model_name) {
  idx <- which(M != 0, arr.ind = TRUE)
  data.frame(
    model = model_name,
    row = idx[, 1],
    col = idx[, 2],
    entry = paste(idx[, 1], idx[, 2], sep = ","),
    elasticity = M[idx],
    stringsAsFactors = FALSE
  )
}

elast_df <- do.call(rbind, Map(to_long, E_list, names(E_list)))

# Keep positive values for log scale
elast_df <- elast_df[elast_df$elasticity > 0, ]

# Row-major ordering: row 1 entries first, row 2 second, rows 3+ third, etc.
elast_df <- elast_df[order(elast_df$row, elast_df$col), ]

models_order <- names(E_list)
entries <- unique(elast_df$entry)

# Create matrix for barplot: rows = models (5), cols = entries
elast_mat <- sapply(models_order, function(m) {
  elast_df$elasticity[elast_df$model == m]
})
elast_mat <- t(elast_mat)
rownames(elast_mat) <- models_order
colnames(elast_mat) <- entries

## ----lambda-elasticity-plot, fig.width=12-------------------------------------
# Build biologically meaningful entry labels
# Build (row, col) vectors in the exact column order of elast_mat
rc <- do.call(rbind, strsplit(colnames(elast_mat), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

make_entry_label <- function(r, c) {
  if (r == 1) {
    bquote(F[1, .(c)])
  } else if (r == 2) {
    bquote(C[2, .(c)])
  } else {
    bquote(U[.(r), .(c)])
  }
}

entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)


# Colors by model
# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Leave room for left legend and long x labels
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat,
  beside = TRUE,
  log = "y",
  col = cols,
  border = NA,
  ylab = expression("Elasticity of " * lambda * " (log scale)"),
  xaxt = "n",
  space = c(0.2, 1)   # ← this is the key line
)

axis(
  side = 1,
  at = colMeans(bp),
  labels = entry_labels,
  las = 2,
  cex.axis = 0.9
)

legend(
  "topleft",
  legend = c(
    "Original",
    expression(paste("Agg ", lambda, " + standard")),
    expression(paste("Agg ", lambda, " + elasticity")),
    expression(paste("Agg ", R[0], " + standard")),
    expression(paste("Agg ", R[0], " + elasticity"))
  ),
  fill = cols,
  bty = "n",
  inset = 0.02
)

par(op)

## ----elasticities of R0-------------------------------------------------------
# Partitioning matrix induced by the stage group
# Reintroduced here so this section can be read independently
P <- mpm_partition(groups=groups, n=nrow(matA))

# Elasticity matrices in aggregated space (k x k) for each model
# The elasticity matrix of the original model is presented in its aggregated
# form using the partitioning matrix P. This is the true aggregated form of the elasticity matrix
# to which elasticities derived from aggregated models are compared.
E_A <- mpm_elasticity(matF=matF,matU=matU,framework="R0", normalize=TRUE)$elasticity
E_list <- list(
  "Original" = P %*% E_A %*% t(P),
  "Agg lambda + standard" = mpm_elasticity(matF=m1$F, matU=m1$U,framework="R0", normalize=TRUE)$elasticity,
  "Agg lambda + elasticity" = mpm_elasticity(matF=m2$F, matU=m2$U,framework="R0", normalize=TRUE)$elasticity,
  "Agg R0 + standard" = mpm_elasticity(matF=m3$F, matU=m3$U,framework="R0", normalize=TRUE)$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matF=m4$F, matU=m4$U,framework="R0", normalize=TRUE)$elasticity
)

# Build a long data.frame for all nonzero entries (row-major order)
to_long <- function(M, model_name) {
  idx <- which(M != 0, arr.ind = TRUE)
  data.frame(
    model = model_name,
    row = idx[, 1],
    col = idx[, 2],
    entry = paste(idx[, 1], idx[, 2], sep = ","),
    elasticity = M[idx],
    stringsAsFactors = FALSE
  )
}

elast_df <- do.call(rbind, Map(to_long, E_list, names(E_list)))

# Keep positive values for log scale
elast_df <- elast_df[elast_df$elasticity > 0, ]

# Row-major ordering: row 1 entries first, row 2 second, rows 3+ third, etc.
elast_df <- elast_df[order(elast_df$row, elast_df$col), ]

models_order <- names(E_list)
entries <- unique(elast_df$entry)

# Create matrix for barplot: rows = models (5), cols = entries
elast_mat2 <- sapply(models_order, function(m) {
  elast_df$elasticity[elast_df$model == m]
})
elast_mat2 <- t(elast_mat2)
rownames(elast_mat2) <- models_order
colnames(elast_mat2) <- entries

## ----elasticity-R0-plot, fig.width=12, fig.height=6---------------------------
# Build biologically meaningful entry labels
# Build (row, col) vectors in the exact column order of elast_mat
rc <- do.call(rbind, strsplit(colnames(elast_mat2), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

make_entry_label <- function(r, c) {
  if (r == 1) {
    bquote(F[1, .(c)])
  } else if (r == 2) {
    bquote(C[2, .(c)])
  } else {
    bquote(U[.(r), .(c)])
  }
}

entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)


# Colors by model
# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Leave room for left legend and long x labels
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat2,
  beside = TRUE,
  log = "y",
  col = cols,
  border = NA,
  ylab = expression("Normalized elasticity of " * R[0] * " (log scale)"),
  xaxt = "n",
  space = c(0.2, 1)   # ← this is the key line
)

axis(
  side = 1,
  at = colMeans(bp),
  labels = entry_labels,
  las = 2,
  cex.axis = 0.9
)

legend(
  "topleft",
  legend = c(
    "Original",
    expression(paste("Agg ", lambda, " + standard")),
    expression(paste("Agg ", lambda, " + elasticity")),
    expression(paste("Agg ", R[0], " + standard")),
    expression(paste("Agg ", R[0], " + elasticity"))
  ),
  fill = cols,
  bty = "n",
  inset = 0.02
)

par(op)

## ----lambda-vs.-R0-elasticities-----------------------------------------------
#compare elasticities of top 3 vital rates
#top three elasticities in lambda framework
sum(elast_mat[,c(9,13,15)])/5
#top three elasticities in R0 framework
sum(elast_mat2[,c(9,13,15)])/5

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----projection matrix, echo=FALSE, out.width="50%", fig.cap="  "-------------
knitr::include_graphics("figures/Slide2.PNG")

## ----lambda-r0-figure, echo=FALSE, out.width="90%", fig.cap="Figure 1. Interpretation of λ and R0 frameworks"----
knitr::include_graphics("figures/Slide1.PNG")

## -----------------------------------------------------------------------------
library(mpmaggregate)
library(expm)
library(knitr)
library(kableExtra)

## ----table-preserved, echo=FALSE, results='asis'------------------------------

preserved_tbl <- data.frame(
  Framework = c("lambda", "lambda", "R0", "R0"),
  Criterion = c("standard", "elasticity-consistent", "standard", "elasticity-consistent"),
  Preserved = c(
    "<em>λ</em> (stable growth rate)<br>
     Stable stage distribution<br>
     Interstage flow matrix",

    "<em>λ</em> (stable growth rate)<br>
     Stable stage distribution<br>
     Reproductive values<br>
     Elasticities of <em>λ</em><br>
     Generation time (<em>T</em><sub>a</sub>)",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable stage distribution<br>
     Cohort interstage flow matrix",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable stage distribution<br>
     Cohort reproductive values<br>
     Elasticities of <em>R</em><sub>0</sub><br>
     Cohort generation time (<em>T</em><sub>c</sub>)"
  ),
  check.names = FALSE
)

knitr::kable(
  preserved_tbl,
  format = "html",
  escape = FALSE,
  align = c("l", "l", "l"),
  col.names = c("Framework", "Criterion", "Consistent Demographic Quantities"),
caption = paste0(
  "<strong>Table 1.</strong> Demographic quantities that remain consistent under ",
  "<code>mpm_aggregate()</code> in the <em>λ</em> and <em>R</em><sub>0</sub> frameworks ",
  "for both the standard and elasticity-consistent aggregation criteria. ",
  "Under the <em>λ</em> framework, aggregation maintains consistency in stable ",
  "growth properties, whereas under the <em>R</em><sub>0</sub> framework it maintains ",
  "consistency in cohort-based reproductive quantities. ",
  "Interstage flow weights the columns of <strong>A</strong> by their respective ",
  "stable stage densities (Yokomizo et al. 2024). ",
  "Here, “cohort” denotes the <em>R</em><sub>0</sub>-framework analogues of the ",
  "corresponding <em>λ</em>-framework demographic quantities. ",
  "Notice the symmetry between the <em>λ</em> and <em>R</em><sub>0</sub> frameworks."
)
) |>
  kable_styling(
    full_width = FALSE,
    bootstrap_options = c("striped", "condensed")
  ) |>
  column_spec(3, width = "7cm") |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<div style='border-top: 1px solid black; padding-top: 6px;'>",
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> <code>mpm_aggregate()</code> retains the projection ",
      "interval of the original matrix population model.",
      "</span></div>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )

## -----------------------------------------------------------------------------
# Example matrices used in the general-to-general MPM aggregation examples below
#population projection matrix for Sclerocarya	birrea
#MatrixID = 247940

# Not run: requires Rcompadre and internet access

#these matrices can be retrieved with:
#library(Rcompadre)
#compadre <- cdb_fetch("compadre")
#mpm <- compadre[compadre$MatrixID == 247940, ]
#A <- matA(mpm)[[1]]
#U <- matU(mpm)[[1]]
#F <- matF(mpm)[[1]]


A <- matrix(
  c(0.0, 0.0000, 12.46,
    0.4, 0.9160,  0.00,
    0.0, 0.0371,  0.99),
  nrow = 3,
  byrow = TRUE
)
#survival/transitions matrix
U <- matrix(
  c(0.0, 0.0000,  0.00,
    0.4, 0.9160,  0.00,
    0.0, 0.0371,  0.99),
  nrow = 3,
  byrow = TRUE
)
#reproduction matrix
F <- matrix(
  c(0.0, 0.0000, 12.46,
    0.0, 0.0000,  0.00,
    0.0, 0.0000,  0.00),
  nrow = 3,
  byrow = TRUE
)

# Stage aggregation used in later examples:
# group vegetative + reproductive stages together,
# leaving the seedling stage separate
groups <- list(
  c(1),
  c(2,3)
)

## -----------------------------------------------------------------------------

# General-to-general MPM aggregation
# in the lambda framework, standard aggregation
res_lambda_std <- mpm_aggregate(
  matA = A,
  groups = groups,
  framework = "lambda",
  criterion = "standard"
)

# Aggregated projection matrix
res_lambda_std$matA_agg

# Effectiveness of aggregation
res_lambda_std$effectiveness

# ---- Check: lambda consistency ----

# Expected
spectral_radius(A)

# Lambda of aggregated model
spectral_radius(res_lambda_std$matA_agg)

# ---- Check: stable stage distribution consistency ----

# P is the partitioning matrix that groups stages
P <- mpm_partition(groups=groups,n=3)

# w is the stable stage distribution
w <- stable_stage(A=A)

# Expected
c(P%*%w)

# stable stage distribution of aggregated model
w_agg <-stable_stage(res_lambda_std$matA_agg)
w_agg

# ---- Check: interstage flows consistency ----

# Expected
P%*%A%*%diag(w)%*%t(P)

# Interstage flows of aggregated model
res_lambda_std$matA_agg%*%diag(w_agg)

## -----------------------------------------------------------------------------

# General-to-general MPM aggregation
# in the lambda framework, elasticity-consistent aggregation
res_lambda_el <- mpm_aggregate(
  matA = A,
  matF = F,
  matU = U,
  groups = groups,
  framework = "lambda",
  criterion = "elasticity"
)

# Aggregated matrix
res_lambda_el$matA_agg

# Effectiveness of aggregation
res_lambda_el$effectiveness

# ---- Check: lambda consistency ----

# Expected
spectral_radius(A)

# Lambda of aggregated model
spectral_radius(res_lambda_el$matA_agg)

# ---- Check: stable stage distribution consistency ----

# P is the partitioning matrix
P <- mpm_partition(groups=groups,n=3)

# w us the stable stage distribution 
w <- stable_stage(A=A)

# Expected
c(P%*%w)

# Stable stage distribution of aggregated model
stable_stage(res_lambda_el$matA_agg)

# ---- Check: reproductive values consistency ----

# v is reproductive values (dominant left eigenvector)
v = reproductive_values(A)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(w*v))

# Reproductive values of aggregated model
reproductive_values(res_lambda_el$matA_agg)

# ---- Check: elasticities consistency ----

# Elasticity matrix
EA <- mpm_elasticity(matA=A,framework="lambda")$elasticity

# Expected
P%*%EA%*%t(P)

# Elasticities of aggregated model
mpm_elasticity(matA=res_lambda_el$matA_agg,framework="lambda")$elasticity

# ---- Check: generation time consistency ----

# Expected
generation_time(matF=F,matU=U,framework="lambda")$generation_time

# Generation time from aggregated model
generation_time(matF=res_lambda_el$matF_agg,
                matU=res_lambda_el$matU_agg,
                framework="lambda")$generation_time

## -----------------------------------------------------------------------------

# General-to-general MPM aggregation
# in the R0 framework, standard aggregation
res_R0_std <- mpm_aggregate(
  matU = U,
  matF = F,
  groups = groups,
  framework = "R0",
  criterion = "standard"
)

# Aggregated projection matrix
res_R0_std$matA_agg

# Effectiveness of aggregation
res_R0_std$effectiveness

# ---- Check: R0 consistency ----

# Expected
spectral_radius(F%*%solve(diag(3)-U))

# R0 from aggregated matrix
spectral_radius(res_R0_std$matF_agg%*%solve(diag(2)-res_R0_std$matU_agg))

# ---- Check: cohort stable stage distribution consistency ----

# w is the cohort stable stage distribution from a next generation matrix
w <- stable_stage(solve(diag(3)-U)%*%F)

# Expected for consistency
c(P%*%stable_stage(solve(diag(3)-U)%*%F))

# Aggregated cohort stable stage distribution
w_agg <- stable_stage(solve(diag(2)-res_R0_std$matU_agg)%*%res_R0_std$matF_agg)
w_agg

# ---- Check: cohort interstage flows  consistency ----

# Expected
P%*%A%*%diag(w)%*%t(P)

# Aggregated interstage flows
res_R0_std$matA_agg%*%diag(w_agg)

## -----------------------------------------------------------------------------

# General-to-general MPM aggregation
# in the R0 framework, elasticity-consistent aggregation
res_R0_el <- mpm_aggregate(
  matU = U,
  matF = F,
  groups = groups,
  framework = "R0",
  criterion = "elasticity"
)

# Aggregated projection matrix
res_R0_el$matA_agg

# Effectiveness of aggregation
res_R0_el$effectiveness

# ---- Check: R0 consistency ----

# Expected R0
R0 <- spectral_radius(F%*%solve(diag(3)-U))
R0

# R0 from the aggregated model
spectral_radius(res_R0_el$matF_agg%*%solve(diag(2)-res_R0_el$matU_agg))

# ---- Check: cohort stable stage distribution consistency ----

# Get partitioning matrix
P <- mpm_partition(groups=groups,n=3)

# Construct cohort projection matrix
Aref = F + U*R0

# Cohort stable stage distribution
w <- stable_stage(A=Aref)

# Expected for consistency
c(P%*%w)

# Aggregated cohort projection matrix
Aref_agg = res_R0_el$matF_agg+ R0*res_R0_el$matU_agg

# Aggregated cohort stable stage distribution
stable_stage(Aref_agg)

# ---- Check: cohort reproductive values consistency ----
v = reproductive_values(Aref)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(w*v))

# Cohort reproductive values for aggregated matrix
reproductive_values(Aref_agg)

# ---- Check: elasticities of R0 consistency ----
# Here elasticities are normalized to sum to 1

# Original elasticities
EA <- mpm_elasticity(matF=F,matU=U,framework="R0")$elasticity

# Expected for consistency
P%*%EA%*%t(P)

# Elasticities of R0 from aggregated matrix
mpm_elasticity(matF=res_R0_el$matF_agg,matU=res_R0_el$matU_agg,framework="R0")$elasticity

# ---- Check: cohort generation time consistency ----

# Expected (from original model)
generation_time(matF=F,matU=U,framework="R0")$generation_time

# Cohort generation time from aggregated model
generation_time(matF = res_R0_el$matF_agg, 
                matU = res_R0_el$matU_agg,
                framework="R0")$generation_time

## ----table-leslie-preserved, echo=FALSE, results='asis'-----------------------
preserved_leslie_tbl <- data.frame(
  Framework = c("lambda", "lambda", "R0", "R0"),
  Criterion = c("standard", "elasticity-consistent", "standard", "elasticity-consistent"),
  Preserved = c(
    "<em>λ</em> (stable growth rate)<br>
     Stable age distribution<br>
     Interstage flow matrix of <strong>A</strong><sup><em>k</em></sup>",

    "<em>λ</em> (stable growth rate)<br>
     Stable age distribution<br>
     Reproductive values<br>
     Elasticities of <em>λ</em><sup><em>k</em></sup> with respect to the entries of <strong>A</strong><sup><em>k</em></sup><br>
     Generation time (<em>T</em><sub>a</sub>) of <strong>A</strong><sup><em>k</em></sup>",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable age distribution<br>
     consistent of <strong>A</strong><sub><em>k</em></sub><sup></sup>",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable age distribution<br>
     Cohort reproductive values<br>
     Elasticities of <em>R</em><sub>0</sub> with respect to <strong>A</strong><sub><em>k</em></sub><sup></sup><br>
     Cohort generation time (<em>T</em><sub>c</sub>) for <strong>A</strong><sub><em>k</em></sub><sup></sup>"
  ),
  check.names = FALSE
)

knitr::kable(
  preserved_leslie_tbl,
  format = "html",
  escape = FALSE,
  align = c("l", "l", "l"),
  col.names = c("Framework", "Criterion", "Consistent Demographic Quantities"),
caption = paste0(
  "<strong>Table 2.</strong> Summary of the demographic properties that remain consistent under ",
  "<code>leslie_aggregate()</code> when reducing a Leslie matrix from dimensionality ",
  "<em>n</em> to <em>m</em>. The quantities the remain consistent depend on both the demographic framework ",
  "(long-term growth under <em>λ</em> or lifetime reproductive output under ",
  "<em>R</em><sub>0</sub>) and the aggregation criterion (standard versus ",
  "elasticity-consistent). The integer <em>k</em> = <em>n</em>/<em>m</em> defines the ",
  "temporal rescaling by aggregating a Leslie model. Quantities in the ",
  "<em>λ</em> framework are defined for <strong>A</strong><sup><em>k</em></sup>, ",
  "while cohort quantities in the <em>R</em><sub>0</sub> framework are defined for ",
  "<strong>A</strong><sub><em>k</em></sub><sup>&dagger;</sup>. Notice the structural symmetry ",
  "between the <em>λ</em> and <em>R</em><sub>0</sub> frameworks."
)
) |>
  kable_styling(
    full_width = FALSE,
    bootstrap_options = c("striped", "condensed")
  ) |>
  column_spec(3, width = "7cm") |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<div style='border-top: 1px solid black; padding-top: 6px;'>",
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> Aggregation with <code>leslie_aggregate()</code> ",
      "changes the projection interval from Δt to <em>k</em>Δt.<br>",
      "<sup>&dagger;</sup> <strong>A</strong><sub><em>k</em></sub><sup></sup> denotes the <em>k</em>-step-ahead projection matrix ",
      "in the <em>R</em><sub>0</sub> framework.</span></div>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )

## -----------------------------------------------------------------------------
# New example used for Leslie-to-Leslie aggregation
# These matrices replace the earlier example matrices (A, U, F)
# Leslie population projection matrix for Pimephales promelas (MatrixID = 248187)

# Not run: requires Rcompadre and internet access

# Matrices can be retrieved with:
# library(Rcompadre)
# comadre <- cdb_fetch("comadre",version = "4.23.3.1")
# mpm <- comadre[comadre$MatrixID == 248187, ]
# A <- matA(mpm)[[1]]
# U <- matU(mpm)[[1]]
# F <- matF(mpm)[[1]]



# Leslie population projection matrix
A <- rbind(
  c(0.00, 0.00,  6.00, 62.50),
  c(0.50, 0.00,  0.00,  0.00),
  c(0.00, 0.18,  0.00,  0.00),
  c(0.00, 0.00,  0.12,  0.00)
)

# Reproduction matrix
F <- rbind(
  c(0.00, 0.00,  6.00, 62.50),
  c(0.00, 0.00,  0.00,  0.00),
  c(0.00, 0.00,  0.00,  0.00),
  c(0.00, 0.00,  0.00,  0.00)
)

# Survival/transition matrix
U <- rbind(
  c(0.00, 0.00,  0.00,  0.00),
  c(0.50, 0.00,  0.00,  0.00),
  c(0.00, 0.18,  0.00,  0.00),
  c(0.00, 0.00,  0.12,  0.00)
)

# Partitioning matrix used for the aggregation examples below
# (first two age classes combined, last two combined)
# In practice this matrix is constructed automatically inside leslie_aggregate()
# It is reproduced here only to verify the aggregation results.
P <- rbind(
  c(1,1,0,0),
  c(0,0,1,1)
)

# Dimensions used in the aggregation examples
n <- 4     # dimensionality of the original matrix A
m <- 2     # dimensionality of the aggregated matrix
k <- n / m #(also constructed automatically)

## -----------------------------------------------------------------------------

# Leslie-to-Leslie MPM aggregation
# in the lambda framework, standard aggregation
res_leslie_lambda_std <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "lambda",
  criterion = "standard"
)


# Aggregated Leslie matrix
res_leslie_lambda_std$matAk_agg

# Effectiveness of aggregation
res_leslie_lambda_std$effectiveness

# ---- Check: lambda consistency ----

# Expected lambda
spectral_radius(A)^k

# Lambda of aggregated model
spectral_radius(res_leslie_lambda_std$matAk_agg)

# ---- Check: stable age distribution consistency ----

# Stable age distribution
w <- leslie_stable_age(A=A)

# Expected 
c(P%*%w)

# Aggregated stable age distribution
w_agg <-leslie_stable_age(res_leslie_lambda_std$matAk_agg)
w_agg

# ---- Check: interstage flow consistency ----

# Expected
P%*%(A%^%k)%*%diag(w)%*%t(P)

# Interstage flows from aggregated model
res_leslie_lambda_std$matAk_agg%*%diag(w_agg)

## -----------------------------------------------------------------------------

# Leslie-to-Leslie MPM aggregation
# in the lambda framework, elasticity-consistent aggregation
res_leslie_lambda_el <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "lambda",
  criterion = "elasticity"
)

# Aggregated Leslie matrix
res_leslie_lambda_el$matAk_agg

# Effectiveness of aggregation
res_leslie_lambda_el$effectiveness

# get aggregated F and U
F_agg <-matrix(0,2,2)
F_agg[1,] <- res_leslie_lambda_el$matAk_agg[1,]
U_agg <- res_leslie_lambda_el$matAk_agg
U_agg[1,] <-0

# ---- Check: lambda consistency ----

# Expected
spectral_radius(A)^k

# Lambda for aggregated model
spectral_radius(res_leslie_lambda_el$matAk_agg)

# ---- Check: stable age distribution consistency ----

# Stable age distribution
w <- leslie_stable_age(A=A)

# Expected
c(P%*%w)

# Stable age distribution of aggregated odel
w_agg <-leslie_stable_age(res_leslie_lambda_el$matAk_agg)
w_agg

# ---- Check: reproductive values consistency ----

# Get reproductive values
v <- leslie_reproductive_values(A=A)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(v*w))

# Reproductive values of aggregated model
leslie_reproductive_values(A=res_leslie_lambda_el$matAk_agg)

# ---- Check: elasticities consistency ----

# Get elasticities of lambda^k
EA <- mpm_elasticity(A%^%k,framework="lambda")$elasticity

# Expected
P%*%EA%*%t(P)

# Elasticities of lambda^k for aggregated model
mpm_elasticity(res_leslie_lambda_el$matAk_agg,framework="lambda")$elasticity

# ---- Check: generation time consistency ----

# Survival/transition matrix
U_k <- U%^%k

# Reproduction matrix
F_k <- A%^%k - U_k

# Expected generation time
generation_time(matF=F_k,matU=U_k,framework="lambda")$generation_time

# Generation time of aggregated model
generation_time(matF=F_agg,matU=U_agg,framework="lambda")$generation_time

## -----------------------------------------------------------------------------

# Leslie-to-Leslie MPM aggregation
# in the R0 framework, standard aggregation
res_leslie_R0_std <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "R0",
  criterion = "standard"
)

# Aggregated Leslie matrix
res_leslie_R0_std$matAk_agg

# Effectiveness of aggregation
res_leslie_R0_std$effectiveness

# get F and U for aggregated matrix
F_agg <- matrix(0,2,2)
F_agg[1,] <- res_leslie_R0_std$matAk_agg[1,]
U_agg <- res_leslie_R0_std$matAk_agg
U_agg[1,] <- 0

# ---- Check: generation time consistency ----

# Expected
R0 <- spectral_radius(F%*%solve(diag(4)-U))
R0 

# R0 for aggregated model
spectral_radius(F_agg%*%solve(diag(2)-U_agg))


# ---- Check: cohort stable age distribution consistency ----

# Standardized cohort projection matrix
A_ref <- (F + U*R0)/R0

# Standardized cohort projection matrix for aggregated model
A_ref_agg <- (F_agg + U_agg*R0)/R0

# w is the stable age distribution
w <- leslie_stable_age(A=A_ref)

# Expected
c(P%*%w)

# Stable age distribution for aggregated model
w_agg <-leslie_stable_age(A_ref_agg)
w_agg

# ---- Check: cohort interstage flows consistency ----

# Survival/transition matrix (k steps ahead)
U_k <- U%^%k

# Reproduction matrix (k steps ahead)
F_k <- A%^%k - U_k

# A projection matrix (k steps ahead)
A_k <- F_k + U_k

# Expected
P%*%A_k%*%diag(w)%*%t(P)

# Interstage flows for aggregated model
(F_agg + U_agg)%*%diag(w_agg)

## -----------------------------------------------------------------------------

# Leslie-to-Leslie MPM aggregation
# in the R0 framework, elasticity-consistent aggregation
res_leslie_R0_el <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "R0",
  criterion = "elasticity"
)

# Aggregated Leslie matrix
res_leslie_R0_el$matAk_agg

# Effectiveness of aggregation
res_leslie_R0_el$effectiveness

# Get F and U for aggregated matrix
F_agg <- matrix(0,2,2)
F_agg[1,] <- res_leslie_R0_el$matAk_agg[1,]
U_agg <- res_leslie_R0_el$matAk_agg
U_agg[1,] <- 0

# ---- Check: R0 consistency ----

# Expected
R0 <- spectral_radius(F%*%solve(diag(4)-U))
R0 

# R0 of aggregated model
spectral_radius(F_agg%*%solve(diag(2)-U_agg))


# ---- Check: cohort stage age distribution consistency ----

# Standardized cohort projection matrix
A_ref <- (F + U*R0)/R0

# Standardized cohort projection matrix for aggregated model
A_ref_agg <- (F_agg + U_agg*R0)/R0

# Cohort stable age distribution
w <- leslie_stable_age(A=A_ref)

# Expected
c(P%*%w)

# Cohort stage age distribution for aggregated model
w_agg <-leslie_stable_age(A_ref_agg)
w_agg

# ---- Check: cohort reproductive values consistency ----

# Cohort reproductive values
v <- leslie_reproductive_values(A_ref)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(v*w))

# Cohort reproductive values of aggregated model
leslie_reproductive_values(A_ref_agg)

# ---- Check: elasticities of R0 consistency ----
# These elasticities are normalized to sum to 1

# Survival/transition matrix (k steps ahead)
U_k <- U%^%k

# Reproduction matrix (k steps ahead)
F_k <- A%^%k - U_k

# Cohort projection matrix (k steps ahead)
A_k <- F_k + U_k

# Elasticity of R0 (normalized to sum to 1)
EA <- mpm_elasticity(matF=F_k,matU=U_k,framework="R0")$elasticity

# Expected
P%*%EA%*%t(P)

# Elasticity of R0 for the aggregated model
mpm_elasticity(matF=F_agg,matU=U_agg,framework="R0")$elasticity

# ---- Check: cohort generation time consistency ----

# Expected
generation_time(matF=F_k, matU = U_k, framework="R0")$generation_time

# Generation time of aggregated model
generation_time(matF=F_agg,matU=U_agg, framework="R0")$generation_time

#' Check irreducibility of a nonnegative matrix using the Horn-Johnson criterion
#'
#' Internal helper to enforce irreducibility assumptions used throughout
#' \code{mpmaggregate}. For a nonnegative \eqn{n \times n} matrix \eqn{A},
#' a standard characterization is:
#' \deqn{A \text{ is irreducible } \iff (I + A)^{n-1} \text{ is strictly positive.}}
#'
#' This implementation avoids numerical overflow by operating on the
#' zero/nonzero pattern only (boolean arithmetic): entries are treated as edges
#' in the directed graph of \eqn{A}. The diagonal is forced to be \code{TRUE} to
#' represent \eqn{I + A}.
#'
#' @param A A square numeric matrix. Must be nonnegative and finite.
#' @param name Character string used to label \code{A} in error messages.
#' @param tol Numeric threshold for treating entries as structural zeros.
#'   An entry is considered present if \code{A[i, j] > tol}. Defaults to \code{0}.
#'
#' @return Invisibly returns \code{TRUE} if \code{A} is irreducible; otherwise
#'   throws an error.
#'
#' @references
#' Horn, R. A., and Johnson, C. R. (2013). \emph{Matrix Analysis} (2nd ed.). Cambridge University Press.
#' (See the characterization of irreducibility via positivity of \eqn{(I + A)^{n-1}}
#' for nonnegative matrices.)
#'
#' @keywords internal
#' @noRd
.check_irreducible_hj <- function(A, name = "A", tol = 0) {
  if (!is.matrix(A))
    stop(sprintf("%s must be a matrix.", name), call. = FALSE)
  if (nrow(A) != ncol(A))
    stop(sprintf("%s must be square.", name), call. = FALSE)
  if (any(!is.finite(A)))
    stop(sprintf("%s contains non-finite values.", name), call. = FALSE)
  if (any(A < 0))
    stop(sprintf("%s must be nonnegative.", name), call. = FALSE)

  n <- nrow(A)
  if (n == 1L) {
    # For our purposes: 1x1 is irreducible only if it has a positive self-loop
    if (A[1, 1] <= tol) {
      stop(sprintf("%s must be irreducible (1x1 with A[1,1] <= tol).", name),
           call. = FALSE)
    }
    return(invisible(TRUE))
  }

  # Work with the nonzero pattern (logical adjacency).
  # B = I + A => edges include self-loops.
  B <- (A > tol)
  diag(B) <- TRUE

  # Compute B^(n-1) positivity in boolean semiring:
  # multiplication is (X %*% Y) > 0, addition is OR.
  X <- B
  for (k in 2:(n - 1L)) {
    X <- (X %*% B) > 0
  }

  if (!all(X)) {
    stop(
      sprintf(
        "%s must be irreducible (Horn-Johnson test failed: (I + A)^(n-1) not positive).",
        name
      ),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' Create a partitioning matrix for MPM aggregation
#'
#' Builds a 0/1 partitioning matrix \eqn{P} that maps an \eqn{n}-stage model to an \eqn{m}-stage
#' aggregated model by summing specified original stages within each aggregated stage.
#'
#' The grouping is provided as a list of integer vectors. For example,
#' \code{groups = list(c(1), c(2:3))} defines \eqn{m = 2} aggregated stages:
#' the first contains original stage 1, and the second contains original stages 2 and 3.
#'
#' Each original stage index \code{1:n} must appear exactly once across \code{groups}.
#'
#' @param groups A list of integer vectors giving stage indices for each aggregated group.
#' @param n Optional integer. Number of original stages. If \code{NULL}, inferred as
#'   \code{max(unlist(groups))}. Supplying \code{n} can help detect errors when
#'   some stage indices are missing from \code{groups}, since all indices
#'   \code{1:n} must appear exactly once.
#'
#' @return A numeric matrix \eqn{P} of dimensionality \code{m x n} with entries 0 or 1.
#'
#' @examples
#' g <- list(c(1), c(2:3))
#' P <- mpm_partition(g, n = 3)
#' P
#'
#' @export
mpm_partition <- function(groups, n = NULL) {
  if (!is.list(groups) || length(groups) == 0) {
    stop("groups must be a non-empty list of integer vectors.", call. = FALSE)
  }

  idx <- unlist(groups, use.names = FALSE)
  if (length(idx) == 0)
    stop("groups must contain at least one index.", call. = FALSE)
  if (any(!is.finite(idx)))
    stop("groups contains non-finite indices.", call. = FALSE)
  if (any(abs(idx - round(idx)) > 0))
    stop("groups indices must be integers.", call. = FALSE)
  idx <- as.integer(idx)

  if (is.null(n))
    n <- max(idx)
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n < 1) {
    stop("n must be a positive integer.", call. = FALSE)
  }
  if (abs(n - round(n)) > 0)
    stop("n must be an integer.", call. = FALSE)
  n <- as.integer(n)

  if (any(idx < 1) ||
      any(idx > n))
    stop("groups contains indices outside 1:n.", call. = FALSE)

  tab <- tabulate(idx, nbins = n)
  if (any(tab == 0)) {
    missing <- which(tab == 0)
    stop(
      sprintf(
        "Each original stage must be included exactly once; missing stages: %s",
        paste(missing, collapse = ", ")
      ),
      call. = FALSE
    )
  }
  if (any(tab > 1)) {
    dup <- which(tab > 1)
    stop(
      sprintf(
        "Each original stage must be included exactly once; duplicated stages: %s",
        paste(dup, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  m <- length(groups)
  P <- matrix(0, nrow = m, ncol = n)

  for (i in seq_len(m)) {
    gi <- groups[[i]]
    if (length(gi) == 0)
      stop("groups may not contain empty elements.", call. = FALSE)
    if (any(!is.finite(gi)))
      stop(sprintf("groups[[%d]] contains non-finite indices.", i),
           call. = FALSE)
    if (any(abs(gi - round(gi)) > 0))
      stop(sprintf("groups[[%d]] indices must be integers.", i), call. = FALSE)
    gi <- as.integer(gi)
    if (any(gi < 1 |
            gi > n))
      stop(sprintf("groups[[%d]] contains indices outside 1:n.", i),
           call. = FALSE)

    P[i, gi] <- 1
  }

  return(P)
}


#' Aggregate a general-to-general matrix population model
#'
#' Aggregates one or more matrix population model (MPM) components from an
#' \eqn{n \times n} model to an \eqn{m \times m} model by combining stages into
#' user-defined groups.
#'
#' Groupings are supplied by \code{groups}, a list of integer vectors specifying
#' how original stages (1,\dots,n) are combined; see \code{\link{mpm_partition}}.
#' Each original stage must appear exactly once.
#'
#' Aggregation can be performed under two frameworks (\code{"lambda"} or \code{"R0"})
#' and two criteria (\code{"standard"} or \code{"elasticity"}), which determine how
#' stages are weighted during aggregation.
#'
#' Clonal reproduction \code{matC} is treated as reproductive output. When both
#' \code{matF} and \code{matC} are supplied, the effective reproductive matrix
#' \eqn{R = F + C} is used internally (e.g., in the \code{"R0"} framework) but is
#' not returned. The aggregated effective reproduction can be recovered as
#' \code{matF_agg + matC_agg}.
#'
#' Irreducibility requirement: The effective projection matrix \eqn{A}
#' must be nonnegative, square, and irreducible. This function enforces
#' irreducibility of \eqn{A} as either \code{matA} (if supplied) or
#' \eqn{A = U + (F + C)} otherwise, and will fail if reducible.
#'
#' @param matA Optional projection matrix \eqn{A} (square, finite, nonnegative).
#'   If \code{NULL}, \eqn{A} is constructed as \eqn{U + (F + C)}.
#' @param matU Optional survival-transition matrix \eqn{U} (square, finite, nonnegative).
#'   Required when \code{framework = "R0"} and/or when \code{matA} is \code{NULL}.
#' @param matF Optional fecundity matrix \eqn{F} (square, finite, nonnegative).
#'   Interpreted as part of reproduction \eqn{R = F + C}.
#' @param matC Optional clonal reproduction matrix \eqn{C} (square, finite, nonnegative).
#'   Treated as reproduction and combined with \code{matF} as \eqn{R = F + C}.
#' @param groups A non-empty list of integer vectors specifying aggregation
#'   groups. Each stage in \code{1:n} must appear exactly once across the list.
#' @param framework Character scalar; either \code{"lambda"} or \code{"R0"}.
#'   Determines whether aggregation is based on the projection matrix \eqn{A}
#'   or an \eqn{R_0}-based reference matrix.
#' @param criterion Character scalar; either \code{"standard"} for standard aggregation
#'  or \code{"elasticity"} for elasticity-consistent aggregation.
#' @param tol Numeric tolerance used in positivity/zero-mass checks and balancing
#'   calculations for the elasticity-consistent case.
#' @param ... Reserved for future use.
#'
#' @details
#' When \code{framework = "R0"}, this function requires \code{matU} and at least one
#' of \code{matF} or \code{matC} in order to form the effective reproductive matrix
#' \eqn{R = F + C} and the next generation matrix
#' \eqn{K = R (I - U)^{-1}}. The matrix \eqn{R} is used internally for computing
#' reference quantities in the \code{"R0"} framework but is not returned.
#' The aggregated effective reproduction can be obtained as
#' \code{matF_agg + matC_agg} when both components are supplied.
#'
#' All returned matrices are aggregated using the same weighting rules implied by
#' the selected \code{framework} and \code{criterion}.
#'
#' Effectiveness measures how closely the aggregated model reproduces the behavior
#' of the original model under the chosen \code{framework} and \code{criterion}. When
#' \code{effectiveness} is high (close to 1), applying the original model and then
#' aggregating gives nearly the same result as aggregating first and then applying the
#' aggregated model.
#'
#' The returned element \code{effectiveness} is computed for both criteria:
#' \itemize{
#'   \item For \code{criterion = "standard"}, effectiveness is computed using weights
#'     based on the reference stable stage distribution \code{w}.
#'   \item For \code{criterion = "elasticity"}, effectiveness is computed after
#'     balancing transformations (with weights based on \code{w * v}).
#' }
#'
#' @return A named list with elements:
#' \describe{
#'   \item{\code{framework}}{The matched framework used (\code{"lambda"} or \code{"R0"}).}
#'   \item{\code{criterion}}{The matched criterion used (\code{"standard"} or \code{"elasticity"}).}
#'   \item{\code{matA_agg}}{Aggregated projection matrix \eqn{A_{\mathrm{agg}}}.}
#'   \item{\code{matU_agg}}{Aggregated survival-transition matrix \eqn{U_{\mathrm{agg}}}, or \code{NULL} if \code{matU} was not supplied.}
#'   \item{\code{matF_agg}}{Aggregated fecundity matrix \eqn{F_{\mathrm{agg}}}, or \code{NULL} if \code{matF} was not supplied.}
#'   \item{\code{matC_agg}}{Aggregated clonal reproduction matrix \eqn{C_{\mathrm{agg}}}, or \code{NULL} if \code{matC} was not supplied.}
#'   \item{\code{effectiveness}}{Numeric effectiveness measure for the aggregation (definition depends on \code{criterion}).}
#' }

#'
#' @references
#' Bienvenu, F., Akcay, E., Legendre, S. and McCandlish, D.M. (2017). The genealogical
#' decomposition of a matrix population model with applications to the aggregation
#' of stages. \emph{Theoretical Population Biology}, 115, 69-80.
#' \doi{10.1016/j.tpb.2017.04.002}
#'
#' Hooley, D. E. (2000). Collapsed matrices with (almost) the same eigenstuff.
#' \emph{The College Mathematics Journal}, 31(4), 297-299.
#' \doi{10.1080/07468342.2000.11974162}
#'
#' Salguero-Gomez, R. & Plotkin, J. B. (2010). Matrix dimensions bias
#' demographic inferences: implications for comparative plant demography.
#' \emph{The American Naturalist}, 176, 710-722. \doi{10.1086/657044}
#'
#' @examples
#' # Example aggregation of a 3x3 projection matrix to 2x2 using groups:
#' # group 1 = stage 1; group 2 = stages 2 and 3.
#' A <- matrix(c(
#'   0.0, 1.0, 2.0,
#'   0.5, 0.0, 0.0,
#'   0.0, 0.8, 0.9
#' ), nrow = 3, byrow = TRUE)
#'
#' res_std <- mpm_aggregate(
#'   matA = A,
#'   groups = list(c(1), c(2, 3)),
#'   framework = "lambda",
#'   criterion = "standard"
#' )
#' res_std$matA_agg
#' res_std$effectiveness
#'
#' res_el <- mpm_aggregate(
#'   matA = A,
#'   groups = list(c(1), c(2, 3)),
#'   framework = "lambda",
#'   criterion = "elasticity"
#' )
#' res_el$matA_agg
#' res_el$effectiveness
#'
#' @export

mpm_aggregate <- function(matA = NULL,
                          matU = NULL,
                          matF = NULL,
                          matC = NULL,
                          groups,
                          framework = c("lambda", "R0"),
                          criterion = c("standard", "elasticity"),
                          tol = 1e-12,
                          ...) {
  framework <- match.arg(framework)
  criterion <- match.arg(criterion)

  .is_square <- function(M)
    is.matrix(M) && nrow(M) == ncol(M)

  .check_nonnegative <- function(M, name) {
    if (is.null(M))
      return(invisible(TRUE))
    if (!is.matrix(M))
      stop(sprintf("%s must be a matrix.", name), call. = FALSE)
    if (any(!is.finite(M)))
      stop(sprintf("%s contains non-finite values.", name), call. = FALSE)
    if (any(M < 0))
      stop(sprintf("%s must be nonnegative (contains values < 0).", name),
           call. = FALSE)
    invisible(TRUE)
  }

  .check_dim <- function(M, n, name) {
    if (is.null(M))
      return(invisible(TRUE))
    if (!.is_square(M) || any(dim(M) != c(n, n))) {
      stop(sprintf("%s must be %dx%d.", name, n, n), call. = FALSE)
    }
    invisible(TRUE)
  }

  # Build Q_eff from reference w and partition P (m x n).
  # Q_eff is n x m with columns summing to 1 within each group, proportional to w.
  .build_Q_eff_from_w <- function(P,
                               w,
                               tol = 1e-12,
                               rcond_tol = 1e-12) {
    if (!is.matrix(P))
      stop("P must be a matrix.", call. = FALSE)

    n <- ncol(P)
    m <- nrow(P)

    if (length(w) != n)
      stop("w must have length ncol(P).", call. = FALSE)
    if (any(!is.finite(w)))
      stop("w contains non-finite values.", call. = FALSE)
    if (any(w < 0))
      stop("w must be nonnegative.", call. = FALSE)

    # Each aggregated group has positive mass under w
    wg <- as.numeric(P %*% w)
    if (any(wg <= tol)) {
      stop(
        "Cannot construct Q_eff: at least one group has ~0 total mass under w (P %*% w <= tol).",
        call. = FALSE
      )
    }

    W <- diag(w, nrow = n, ncol = n)
    M <- P %*% W %*% t(P)  # m x m

    rc <- rcond(M)
    if (!is.finite(rc) || rc < rcond_tol) {
      stop(
        "Cannot invert P %*% W %*% t(P): matrix is singular or ill-conditioned (rcond < rcond_tol).",
        call. = FALSE
      )
    }

    Q_eff <- W %*% t(P) %*% solve(M)  # n x m
    return(Q_eff)
  }


  .build_P_eff_elasticity <- function(P,
                                   w,
                                   v,
                                   tol = 1e-12,
                                   rcond_tol = 1e-12) {
    if (!is.matrix(P))
      stop("P must be a matrix.", call. = FALSE)

    n <- ncol(P)

    if (length(v) != n)
      stop("v must have length ncol(P).", call. = FALSE)
    if (length(w) != n)
      stop("w must have length ncol(P).", call. = FALSE)
    if (any(!is.finite(v)) ||
        any(!is.finite(w)))
      stop("v and w must be finite.", call. = FALSE)
    if (any(v < 0) ||
        any(w < 0))
      stop("v and w must be nonnegative.", call. = FALSE)

    # Each aggregated group has positive mass under w
    wg <- as.numeric(P %*% w)
    if (any(wg <= tol)) {
      stop(
        "Cannot construct P_eff: at least one group has ~0 total mass under w (P %*% w <= tol).",
        call. = FALSE
      )
    }

    # Each aggregated group has positive mass under v*w (needed for invertibility of P V W P^T)
    vwg <- as.numeric(P %*% (v * w))
    if (any(vwg <= tol)) {
      stop(
        "Cannot construct P_eff: at least one group has ~0 total mass under v*w (P %*% (v*w) <= tol).",
        call. = FALSE
      )
    }

    W <- diag(w, nrow = n, ncol = n)
    V <- diag(v, nrow = n, ncol = n)

    A_left <- P %*% W %*% t(P)        # m x m
    M <- P %*% V %*% W %*% t(P)       # m x m
    B_right <- P %*% V                # m x n

    rc <- rcond(M)
    if (!is.finite(rc) || rc < rcond_tol) {
      stop(
        "Cannot invert P %*% V %*% W %*% t(P): matrix is singular or ill-conditioned (rcond < rcond_tol).",
        call. = FALSE
      )
    }

    P_eff <- A_left %*% solve(M) %*% B_right
    return(P_eff)
  }



  .aggregate <- function(M, P_eff, Q_eff) {
    if (is.null(M))
      return(NULL)
    return(P_eff %*% M %*% Q_eff)
  }

  .effectiveness <- function(A, B, P, WTS) {
    if (is.null(A))
      return(NULL)
    return(norm(B %*% P %*% sqrt(WTS), type = "F")^2 / norm(P %*%
                                                              A %*% sqrt(WTS), type = "F")^2)
  }


  # ---- validate raw inputs ----
  .check_nonnegative(matA, "matA")
  .check_nonnegative(matU, "matU")
  .check_nonnegative(matF, "matF")
  .check_nonnegative(matC, "matC")

  # ---- build effective reproduction R = F + C ----
  matR <- NULL
  if (!is.null(matF) && !is.null(matC)) {
    if (!.is_square(matF) ||
        !.is_square(matC) || any(dim(matF) != dim(matC))) {
      stop("matF and matC must be square matrices with identical dimensionalities.",
           call. = FALSE)
    }
    matR <- matF + matC
  } else if (!is.null(matF)) {
    if (!.is_square(matF))
      stop("matF must be square.", call. = FALSE)
    matR <- matF
  } else if (!is.null(matC)) {
    if (!.is_square(matC))
      stop("matC must be square.", call. = FALSE)
    matR <- matC
  }

  # ---- determine effective A (always used for returning A_agg) ----
  if (!is.null(matA)) {
    if (!.is_square(matA))
      stop("matA must be square.", call. = FALSE)
    matA_eff <- matA
  } else {
    if (is.null(matU) || is.null(matR)) {
      stop(
        "If matA is NULL, matU and at least one of matF/matC must be supplied so A = U + (F + C) can be formed.",
        call. = FALSE
      )
    }
    if (!.is_square(matU) ||
        !.is_square(matR) || any(dim(matU) != dim(matR))) {
      stop("matU and (matF + matC) must be square with identical dimensionalities.",
           call. = FALSE)
    }
    matA_eff <- matU + matR
  }

  .check_nonnegative(matA_eff, "Effective A")
  .check_irreducible_hj(matA_eff, "Effective A")
  n <- nrow(matA_eff)

  # Check component dims against n
  .check_dim(matU, n, "matU")
  .check_dim(matF, n, "matF")
  .check_dim(matC, n, "matC")
  if (!is.null(matR))
    .check_dim(matR, n, "matR (matF + matC)")

  # Partition matrix P (m x n)
  P <- mpm_partition(groups, n = n)
  m <- nrow(P)

  # ---- choose reference eigenstructure depending on framework ----
  # These are the "new w and v" you referenced for the R0 framework.
  ref <- NULL

  if (framework == "lambda") {
    # reference matrix is A
    if (criterion == "standard") {
      lam <- spectral_radius(matA_eff)
      ref <- list(lambda = lam,
                  w = stable_stage(matA_eff, tol = tol, ensure_positive = TRUE))
    } else {
      # elasticity
      ref <- dominant_eigen(matA_eff, tol = tol, ensure_positive = TRUE)
    }
  } else {
    # framework == "R0"
    if (is.null(matU) || is.null(matR)) {
      stop(
        "framework='R0' requires matU and at least one of matF/matC (reproduction).",
        call. = FALSE
      )
    }

    #    n <- nrow(matU)
    I <- diag(n)

    # Fundamental matrix N = (I - U)^(-1)
    N <- tryCatch(
      solve(I - matU),
      error = function(e)
        NULL
    )
    if (is.null(N)) {
      stop("Could not invert (I - U). R0 framework requires (I - U) to be invertible.",
           call. = FALSE)
    }

    # Next generation matrix and its Perron eigenvalue
    K <- matR %*% N
    R0 <- spectral_radius(K)

    if (!is.finite(R0) || R0 <= 0) {
      stop("Computed R0 is not positive/finite; check U and R.", call. = FALSE)
    }

    # Reference matrix for aggregation eigenstructure in the R0 framework
    A_R0ref <- matR + R0 * matU


    if (criterion == "standard") {
      ref <- list(w = stable_stage(A_R0ref, tol = tol, ensure_positive = TRUE))
    } else {
      # elasticity
      ref <- dominant_eigen(A_R0ref, tol = tol, ensure_positive = TRUE)

      ref$lambda <- NULL
    }
  }



  # ---- build P_eff and Q_eff for each of the four cases ----
  # Q_eff always depends on the appropriate reference w.
  Q_eff <- .build_Q_eff_from_w(P, ref$w, tol = tol, rcond_tol = tol)

  if (criterion == "standard") {
    P_eff <- P
  } else {
    #criterion == "elasticity"
    P_eff <- .build_P_eff_elasticity(
      P = P,
      w = ref$w,
      v = ref$v,
      tol = tol,
      rcond_tol = tol
    )
  }

  # ---- aggregate matrices using P_eff and Q_eff ----
  # What you aggregate depends on framework; we return both A_agg and components where available.
  out <- list(framework = framework, criterion = criterion)

  # Always return aggregated A (built from effective A)
  out$matA_agg <- .aggregate(matA_eff, P_eff, Q_eff)

  # Also return aggregated components where available
  out$matU_agg <- .aggregate(matU, P_eff, Q_eff)

  # The aggregated F when available
  out$matF_agg <- .aggregate(matF, P_eff, Q_eff)

  # Optional bookkeeping: C alone
  out$matC_agg <- .aggregate(matC, P_eff, Q_eff)

  #add effectiveness measures
  if (criterion == "standard") {
    refw <- as.numeric(ref$w)
    if (min(refw) < tol)
      stop("Balancing A requires its stable stage distribution > 0",
           call. = FALSE)
    out$effectiveness <- .effectiveness(
      A = matA_eff,
      B = out$matA_agg,
      P = P,
      WTS = diag(refw, n, n)
    )
  }

  if (criterion == "elasticity") {
    # works with balanced matrices
    refv <- as.numeric(ref$v)
    refw <- as.numeric(ref$w)
    if (min(refv) < tol)
      stop("Balancing A requires its reproductive values > 0", call. = FALSE)
    A_balanced <- diag(refv, n, n) %*% matA_eff %*% diag(1 / refv, n, n)
    B_w <-  P %*% refw
    B_w <- as.numeric(B_w)
    W <- diag(refw, nrow = n, ncol = n)
    B_v <- solve(P %*% W %*% t(P)) %*% P %*% W %*% refv
    B_v <- as.numeric(B_v)
    if (min(B_v) < tol)
      stop("Balancing B requires its reproductive values > 0", call. = FALSE)
    B_balanced <- diag(B_v, m, m) %*% out$matA_agg %*% diag(1 / B_v, m, m)
    out$effectiveness <- .effectiveness(
      A = A_balanced,
      B = B_balanced,
      P = P,
      WTS = diag(refw * refv, n, n)
    )
  }

  return(out)
}

test_that("mpm_partition: builds correct partition matrix for valid groups",
          {
            groups <- list(c(1, 3), c(2, 4))
            P <- mpm_partition(groups)

            expect_true(is.matrix(P))
            expect_equal(dim(P), c(2, 4))

            # expected pattern
            P_expected <- matrix(
              c(
                1, 0, 1, 0,
                0, 1, 0, 1
              ),
              nrow = 2, byrow = TRUE
            )
            expect_equal(P, P_expected)

            # Each column has exactly one 1 (since each stage used exactly once)
            expect_equal(colSums(P), rep(1, 4))
            # Row sums match group sizes
            expect_equal(rowSums(P), c(2, 2))
          })

test_that("mpm_partition: respects explicit n (and catches out-of-range indices)",
          {
            groups <- list(c(1, 2), c(3))
            P <- mpm_partition(groups, n = 3)
            expect_equal(dim(P), c(2, 3))

            # With n larger than max(idx), all stages must still appear exactly once -> should error
            expect_error(mpm_partition(groups, n = 4),
                         regexp = "missing stages",
                         ignore.case = TRUE)

            # n smaller than max(idx) should error due to indices outside 1:n
            expect_error(mpm_partition(groups, n = 2),
                         regexp = "outside 1:n",
                         ignore.case = TRUE)
          })

test_that("mpm_partition: errors when groups is not a non-empty list", {
  expect_error(mpm_partition(groups = 1:3),
               regexp = "non-empty list",
               ignore.case = TRUE)

  expect_error(mpm_partition(groups = list()),
               regexp = "non-empty list",
               ignore.case = TRUE)
})

test_that("mpm_partition: errors when groups contain no indices after unlist()",
          {
            expect_error(mpm_partition(groups = list(integer(0), integer(0))),
                         regexp = "at least one index",
                         ignore.case = TRUE)
          })

test_that("mpm_partition: errors on non-finite or non-integer indices (global check)",
          {
            expect_error(mpm_partition(groups = list(c(1, NA), c(2))),
                         regexp = "non-finite",
                         ignore.case = TRUE)

            expect_error(mpm_partition(groups = list(c(1, Inf), c(2))),
                         regexp = "non-finite",
                         ignore.case = TRUE)

            expect_error(mpm_partition(groups = list(c(1, 2.2), c(3))),
                         regexp = "must be integers",
                         ignore.case = TRUE)
          })

test_that("mpm_partition: errors when indices are outside 1:n", {
  expect_error(mpm_partition(groups = list(c(0, 1), c(2))),
               regexp = "outside 1:n",
               ignore.case = TRUE)

  expect_error(mpm_partition(groups = list(c(1, 4), c(2)), n = 3),
               regexp = "outside 1:n",
               ignore.case = TRUE)
})

test_that("mpm_partition: errors on missing or duplicated stages", {
  # Missing stage 3 (when n inferred as max(idx)=2? actually max=2 so not missing)
  # Force n=3 so stage 3 is required but absent:
  expect_error(mpm_partition(groups = list(c(1), c(2)), n = 3),
               regexp = "missing stages",
               ignore.case = TRUE)

  # Duplicated stage 2
  expect_error(mpm_partition(groups = list(c(1, 2), c(2, 3)), n = 3),
               regexp = "duplicated stages",
               ignore.case = TRUE)
})

test_that("mpm_partition: errors on invalid n values", {
  # non-numeric / wrong length / non-finite / < 1
  expect_error(mpm_partition(groups = list(1), n = "3"),
               regexp = "positive integer|must be",
               ignore.case = TRUE)

  expect_error(mpm_partition(groups = list(1), n = c(3, 4)),
               regexp = "positive integer|must be",
               ignore.case = TRUE)

  expect_error(
    mpm_partition(groups = list(1), n = NA_real_),
    regexp = "positive integer|must be",
    ignore.case = TRUE
  )

  expect_error(mpm_partition(groups = list(1), n = 0),
               regexp = "positive integer",
               ignore.case = TRUE)

  # non-integer numeric
  expect_error(mpm_partition(groups = list(1), n = 3.3),
               regexp = "n must be an integer",
               ignore.case = TRUE)
})

test_that("mpm_partition: per-group checks inside the loop are enforced", {
  # Empty element
  expect_error(mpm_partition(groups = list(c(1), integer(0)), n = 2),
               regexp = "Each original stage",
               ignore.case = TRUE)

  # Non-finite inside a group (this is also caught earlier globally, but test anyway)
  expect_error(mpm_partition(groups = list(c(1), c(NA)), n = 2),
               regexp = "non-finite",
               ignore.case = TRUE)

  # Non-integer inside a group (also caught earlier globally)
  expect_error(mpm_partition(groups = list(c(1), c(2.5)), n = 2),
               regexp = "must be integers",
               ignore.case = TRUE)

  # Out-of-range inside a group
  expect_error(mpm_partition(groups = list(c(1), c(3)), n = 2),
               regexp = "outside 1:n",
               ignore.case = TRUE)
})

test_that("mpm_aggregate: basic functionality for lambda framework (standard + elasticity)",
          {
            # A strictly positive matrix is irreducible and nonnegative by construction.
            A <- matrix(
              c(
                0.10, 1.00, 6.35,
                0.33, 0.10, 0.05,
                0.02, 0.50, 0.20
              ),
              nrow = 3, byrow = TRUE
            )
            groups <- list(c(1), c(2, 3))

            res_std <- mpm_aggregate(
              matA = A,
              groups = groups,
              framework = "lambda",
              criterion = "standard"
            )
            res_elas <- mpm_aggregate(
              matA = A,
              groups = groups,
              framework = "lambda",
              criterion = "elasticity"
            )

            # Common expectations for output
            expect_type(res_std, "list")
            expect_equal(res_std$framework, "lambda")
            expect_equal(res_std$criterion, "standard")
            expect_true(is.matrix(res_std$matA_agg))
            expect_equal(dim(res_std$matA_agg), c(2, 2))
            expect_true(all(is.finite(res_std$matA_agg)))

            expect_type(res_elas, "list")
            expect_equal(res_elas$framework, "lambda")
            expect_equal(res_elas$criterion, "elasticity")
            expect_true(is.matrix(res_elas$matA_agg))
            expect_equal(dim(res_elas$matA_agg), c(2, 2))
            expect_true(all(is.finite(res_elas$matA_agg)))

            # Effectiveness should exist and be finite
            expect_true(is.numeric(res_std$effectiveness))
            expect_true(is.finite(res_std$effectiveness))
            expect_true(is.numeric(res_elas$effectiveness))
            expect_true(is.finite(res_elas$effectiveness))
          })

test_that("mpm_aggregate: aggregation preserves eigen components (lambda framework)",
          {
            #test that lambda and stable stage distributions are consistent.
            #example from Hooley (2000)
            A <- matrix(
              c(
                0.00, 0.00, 37.00, 64.00, 82.00,
                0.06, 0.00,  0.00,  0.00,  0.00,
                0.00, 0.34,  0.00,  0.00,  0.00,
                0.00, 0.00,  0.16,  0.00,  0.00,
                0.00, 0.00,  0.00,  0.08,  0.00
              ),
              nrow = 5, byrow = TRUE
            )
            groups <- list(c(1), c(2, 3, 4), c(5))

            #first test lambda framework
            res_std <- mpm_aggregate(
              matA = A,
              groups = groups,
              framework = "lambda",
              criterion = "standard"
            )
            res_elas <- mpm_aggregate(
              matA = A,
              groups = groups,
              framework = "lambda",
              criterion = "elasticity"
            )

            eigen_std <- dominant_eigen(res_std$matA_agg)
            eigen_elas <- dominant_eigen(res_elas$matA_agg)
            eigen_true <- dominant_eigen(A)
            #lambdas equal?
            expect_equal(as.numeric(eigen_std$lambda),
                         as.numeric(eigen_true$lambda),
                         tolerance = 1e-8)
            expect_equal(as.numeric(eigen_elas$lambda),
                         as.numeric(eigen_true$lambda),
                         tolerance = 1e-8)
            #stable stage distributions consistent?
            P <- mpm_partition(groups = groups, n = 5)
            expect_equal(as.numeric(eigen_std$w),
                         as.numeric(P %*% eigen_true$w),
                         tolerance = 1.e-8)
            expect_equal(as.numeric(eigen_elas$w),
                         as.numeric(P %*% eigen_true$w),
                         tolerance = 1.e-8)
            #reproductive values consistent?
            w <- eigen_true$w
            v <- eigen_true$v
            W <- diag(w)
            vtest <- solve(P %*% W %*% t(P)) %*% P %*% (w * eigen_true$v)
            expect_equal(as.numeric(eigen_elas$v), as.numeric(vtest), tolerance =
                           1.e-8)
          })

test_that("mpm_aggregate: aggregation preserves eigen components (R0 framework)",
          {
            #test that lambda and stable stage distributions are consistent.
            #example from Hooley (2000)
            A <- matrix(
              c(
                0.00, 0.00, 37.00, 64.00, 82.00,
                0.06, 0.00,  0.00,  0.00,  0.00,
                0.00, 0.34,  0.00,  0.00,  0.00,
                0.00, 0.00,  0.16,  0.00,  0.00,
                0.00, 0.00,  0.00,  0.08,  0.00
              ),
              nrow = 5, byrow = TRUE
            )
            groups <- list(c(1), c(2, 3, 4), c(5))

            U <- matrix(0, 5, 5)
            F <- matrix(0, 5, 5)
            U[2:5, ] <- A[2:5, ]
            F[1, ] <- A[1, ]

            res_std <- mpm_aggregate(
              matA = A,
              matU = U,
              matF = F,
              groups = groups,
              framework = "R0",
              criterion = "standard"
            )
            res_elas <- mpm_aggregate(
              matA = A,
              matU = U,
              matF = F,
              groups = groups,
              framework = "R0",
              criterion = "elasticity"
            )

            #aggregated generation matrices
            K_std <- res_std$matF_agg %*% solve(diag(3) - res_std$matU_agg)
            K_elas <- res_elas$matF_agg %*% solve(diag(3) - res_elas$matU_agg)
            K_true <- F %*% solve(diag(5) - U)
            R0_std <- spectral_radius(K_std)
            R0_elas <- spectral_radius(K_elas)
            R0_true <- spectral_radius(K_true)

            #R0s equal?
            expect_equal(as.numeric(R0_std), as.numeric(R0_true), tolerance = 1e-8)
            expect_equal(as.numeric(R0_elas), as.numeric(R0_true), tolerance = 1e-8)

            #get cohort eigenstuff
            eigen_std <- dominant_eigen(res_std$matF_agg + R0_std * res_std$matU)
            eigen_elas <- dominant_eigen(res_elas$matF_agg + R0_elas * res_elas$matU)
            eigen_true <- dominant_eigen(F + R0_true * U)

            #cohort stable stage distributions consistent?
            P <- mpm_partition(groups = groups, n = 5)
            expect_equal(as.numeric(eigen_std$w),
                         as.numeric(P %*% eigen_true$w),
                         tolerance = 1.e-8)
            expect_equal(as.numeric(eigen_elas$w),
                         as.numeric(P %*% eigen_true$w),
                         tolerance = 1.e-8)

            #cohort reproductive values consistent?
            w <- eigen_true$w
            v <- eigen_true$v
            W <- diag(w)
            vtest <- solve(P %*% W %*% t(P)) %*% P %*% (w * eigen_true$v)
            expect_equal(as.numeric(eigen_elas$v), as.numeric(vtest), tolerance =
                           1.e-8)
          })

test_that("mpm_aggregate: R0 framework runs and returns expected components",
          {
            # Build a simple irreducible A = U + R, with U subdiagonal survival and R as reproduction.
            U <- matrix(0, 3, 3)
            U[2, 1] <- 0.33
            U[3, 2] <- 0.50

            # Reproduction: put some positive values in first row
            F <- matrix(0, 3, 3)
            F[1, ] <- c(0.10, 1.00, 6.35)

            # A is effective A = U + (F + C); here C omitted
            A <- U + F

            groups <- list(c(1), c(2, 3))

            res_std <- mpm_aggregate(
              matA = A,
              matU = U,
              matF = F,
              groups = groups,
              framework = "R0",
              criterion = "standard"
            )

            res_elas <- mpm_aggregate(
              matA = A,
              matU = U,
              matF = F,
              groups = groups,
              framework = "R0",
              criterion = "elasticity"
            )

            expect_equal(res_std$framework, "R0")
            expect_equal(res_std$criterion, "standard")
            expect_true(is.matrix(res_std$matA_agg))
            expect_equal(dim(res_std$matA_agg), c(2, 2))
            expect_true(is.matrix(res_std$matU_agg))
            expect_equal(dim(res_std$matU_agg), c(2, 2))
            expect_true(is.matrix(res_std$matF_agg))
            expect_equal(dim(res_std$matF_agg), c(2, 2))
            expect_true(is.finite(res_std$effectiveness))

            expect_equal(res_elas$framework, "R0")
            expect_equal(res_elas$criterion, "elasticity")
            expect_true(is.matrix(res_elas$matA_agg))
            expect_equal(dim(res_elas$matA_agg), c(2, 2))
            expect_true(is.finite(res_elas$effectiveness))

            # matC_agg may be NULL if matC not provided; do not require it.
            expect_true(is.null(res_std$matC_agg) ||
                          is.matrix(res_std$matC_agg))
            expect_true(is.null(res_elas$matC_agg) ||
                          is.matrix(res_elas$matC_agg))
          })

test_that("mpm_aggregate: errors on invalid groupings", {
  A <- matrix(
    c(
      0.2, 0.4, 0.1,
      0.3, 0.2, 0.5,
      0.1, 0.3, 0.2
    ),
    nrow = 3, byrow = TRUE
  )
  # Missing stage 3
  groups_missing <- list(c(1), c(2))
  expect_error(
    mpm_aggregate(matA = A, groups = groups_missing),
    regexp = "missing stages|included exactly once|missing",
    ignore.case = TRUE
  )

  # Duplicate stage 2
  groups_dup <- list(c(1, 2), c(2, 3))
  expect_error(
    mpm_aggregate(matA = A, groups = groups_dup),
    regexp = "duplicated stages|included exactly once|duplicated",
    ignore.case = TRUE
  )
})

test_that("mpm_aggregate: errors on non-square, negative, or reducible matrices",
          {
            # Non-square
            A_ns <- matrix(1, nrow = 2, ncol = 3)
            expect_error(mpm_aggregate(matA = A_ns, groups = list(c(1), c(2))))

            # Negative entry
            A_neg <- diag(3)
            A_neg[1, 2] <- -0.1
            expect_error(mpm_aggregate(matA = A_neg, groups = list(c(1), c(2, 3))))

            # Reducible example: block diagonal (should fail your .check_irreducible_hj)
            A_red <- matrix(0, 4, 4)
            A_red[1:2, 1:2] <- matrix(
              c(
                0.2, 0.3,
                0.1, 0.2
              ),
              nrow = 2, byrow = TRUE
            )

            A_red[3:4, 3:4] <- matrix(
              c(
                0.2, 0.3,
                0.1, 0.2
              ),
              nrow = 2, byrow = TRUE
            )

            expect_error(mpm_aggregate(matA = A_red, groups = list(c(1, 2), c(3, 4))))
          })

#test some edge conditions, where mpm_aggregate returns a 1x1 matrix
#make sure this works as expected
test_that("mpm_aggregate: test if properly returns a 1x1 matrix", {
  #get true results
  matA <- matrix(
    c(
      10,  9,
      0.1, 0.2
    ),
    nrow = 2, byrow = TRUE
  )
  matU <- matA
  matU[1, ] <- 0
  matF <- matA
  matF[2, ] <- 0
  true_lambda <- spectral_radius(matA)
  true_R0 <- spectral_radius(matF %*% solve(diag(2) - matU))
  #get aggregated results
  res <- mpm_aggregate(
    matA = matA,
    groups = list(c(1, 2)),
    framework = "lambda",
    criterion = "standard"
  )
  expect_equal(res$matA_agg, as.matrix(true_lambda), tolerance = 1.e-8)
  res <- mpm_aggregate(
    matA = matA,
    groups = list(c(1, 2)),
    framework = "lambda",
    criterion = "elasticity"
  )
  expect_equal(res$matA_agg, as.matrix(true_lambda), tolerance = 1.e-8)

  res <- mpm_aggregate(
    matA = NULL,
    matU = matU,
    matF = matF,
    groups = list(c(1, 2)),
    framework = "R0",
    criterion = "standard"
  )
  agg_R0 <- spectral_radius(res$matF_agg %*% solve(diag(1) - res$matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)
  res <- mpm_aggregate(
    matA = NULL,
    matU = matU,
    matF = matF,
    groups = list(c(1, 2)),
    framework = "R0",
    criterion = "elasticity"
  )
  agg_R0 <- spectral_radius(res$matF_agg %*% solve(diag(1) - res$matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)
})

#now test what happens when you give a 1x1 matrix as input
test_that("mpm_aggregate: test if 1x1 matrix works as input", {
  #get true results
  matA = matrix(.5, 1, 1)
  matF = as.matrix(0.25, 1, 1)
  matU = as.matrix(0.25, 1, 1)
  true_lambda <- spectral_radius(matA)
  true_R0 <- spectral_radius(matF %*% solve(diag(1) - matU))

  #compare with aggregated results
  res <- mpm_aggregate(
    matA = matA,
    groups = list(c(1)),
    framework = "lambda",
    criterion = "standard"
  )
  expect_equal(res$matA_agg, as.matrix(true_lambda), tolerance = 1.e-8)
  res <- mpm_aggregate(
    matA = matA,
    groups = list(c(1)),
    framework = "lambda",
    criterion = "elasticity"
  )
  expect_equal(res$matA_agg, as.matrix(true_lambda), tolerance = 1.e-8)
  res <- mpm_aggregate(
    matA = NULL,
    matU = matU,
    matF = matF,
    groups = list(c(1)),
    framework = "R0",
    criterion = "standard"
  )
  agg_R0 <- spectral_radius(res$matF_agg %*% solve(diag(1) - res$matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)
  res <- mpm_aggregate(
    matA = NULL,
    matU = matU,
    matF = matF,
    groups = list(c(1)),
    framework = "R0",
    criterion = "elasticity"
  )
  agg_R0 <- spectral_radius(res$matF_agg %*% solve(diag(1) - res$matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)
})

get_fun <- function(name) {
  # Prefer exported function if present
  if (exists(name,
             where = asNamespace("mpmaggregate"),
             inherits = FALSE)) {
    return(get(name, envir = asNamespace("mpmaggregate")))
  }
  # Else fall back to internal
  getFromNamespace(name, "mpmaggregate")
}

test_that("dominant_eigen: returns normalized dominant eigencomponents",
          {
            dominant_eigen <- get_fun("dominant_eigen")

            # Strictly positive => PF applies cleanly
            A <- matrix(
              c(
                0.10, 1.00, 6.35,
                0.33, 0.10, 0.05,
                0.02, 0.50, 0.20
              ),
              nrow = 3, byrow = TRUE
            )

            dom <- dominant_eigen(A, tol = 1e-12, ensure_positive = TRUE)

            expect_true(is.list(dom))
            expect_true(all(c("lambda", "w", "v") %in% names(dom)))

            lambda <- dom$lambda
            w <- dom$w
            v <- dom$v

            expect_true(is.numeric(lambda) &&
                          length(lambda) == 1 && is.finite(lambda))
            expect_true(is.numeric(w) && length(w) == ncol(A))
            expect_true(is.numeric(v) && length(v) == ncol(A))

            expect_equal(sum(w), 1, tolerance = 1e-10)
            expect_equal(sum(v * w), 1, tolerance = 1e-10)

            # Eigen relations (approx)
            expect_equal(as.numeric(A %*% w), as.numeric(lambda * w), tolerance = 1e-8)
            expect_equal(as.numeric(t(v) %*% A), as.numeric(lambda * t(v)), tolerance = 1e-8)
          })

test_that("dominant_eigen: errors on invalid input", {
  dominant_eigen <- get_fun("dominant_eigen")

  A_ns <- matrix(1, 2, 3)
  expect_error(dominant_eigen(A_ns))

  A_nf <- diag(2)
  A_nf[1, 1] <- Inf
  expect_error(dominant_eigen(A_nf))
})

test_that("leslie_dominant_eigen: returns dominant eigencomponents for a Leslie matrix",
          {
            leslie_dominant_eigen <- get_fun("leslie_dominant_eigen")

            A <- matrix(
              c(
                0.10, 1.00, 6.35,
                0.33, 0.00, 0.00,
                0.00, 0.50, 0.00
              ),
              nrow = 3, byrow = TRUE
            )

            dom <- leslie_dominant_eigen(A)

            expect_true(is.list(dom))
            expect_true(all(c("lambda", "w", "v") %in% names(dom)))

            lambda <- dom$lambda
            w <- dom$w
            v <- dom$v

            expect_true(is.numeric(lambda) &&
                          length(lambda) == 1 && is.finite(lambda))
            expect_true(is.numeric(w) && length(w) == ncol(A))
            expect_true(is.numeric(v) && length(v) == ncol(A))

            # Normalizations
            expect_equal(sum(w), 1, tolerance = 1e-10)
            expect_equal(sum(v * w), 1, tolerance = 1e-10)

            # Eigen relations (approx)
            expect_equal(as.numeric(A %*% w), as.numeric(lambda * w), tolerance = 1e-8)
            expect_equal(as.numeric(t(v) %*% A), as.numeric(lambda * t(v)), tolerance = 1e-8)
          })

test_that("leslie_dominant_eigen: errors on non-Leslie input", {
  leslie_dominant_eigen <- get_fun("leslie_dominant_eigen")

  # breaks Leslie form
  A_bad <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.10, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )

  expect_error(leslie_dominant_eigen(A_bad))
})

test_that("is_leslie: identifies a valid Leslie matrix", {
  is_leslie <- get_fun("is_leslie")

  A <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.00, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )
  expect_true(is_leslie(A))
})

test_that("is_leslie: rejects non-Leslie matrices", {
  is_leslie <- get_fun("is_leslie")

  # Non-square
  A_ns <- matrix(0, 2, 3)
  expect_false(is_leslie(A_ns))

  # Has nonzero outside first row and subdiagonal
  A_bad <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.00, 0.00,
      0.00, 0.50, 0.10
    ),
    nrow = 3, byrow = TRUE
  )
  expect_false(is_leslie(A_bad))

  # Negative entry
  A_neg <- matrix(
    c(
      0.10, 1.00, 6.35,
      -0.33, 0.00, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )
  expect_false(is_leslie(A_neg))
})

test_that("leslie_disaggregate: returns a Leslie matrix with m dividing n", {
  leslie_disaggregate <- get_fun("leslie_disaggregate")
  is_leslie <- get_fun("is_leslie")

  A <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.00, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )

  # Choose m that does not divide n=3 (e.g., m=2)
  B <- leslie_disaggregate(A, 2)

  expect_true(is.matrix(B))
  expect_equal(nrow(B), ncol(B))
  expect_true(all(is.finite(B)))
  expect_true(all(B >= 0))

  expect_true(is_leslie(B))

  # Now guaranteed divisible:
  expect_equal(nrow(B) %% 2, 0)
})

test_that("leslie_disaggregate: errors on non-Leslie input", {
  leslie_disaggregate <- get_fun("leslie_disaggregate")

  A_bad <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.10, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )

  expect_error(leslie_disaggregate(A_bad, 2))
})

test_that("leslie_disaggregate: errors on invalid m", {
  leslie_disaggregate <- get_fun("leslie_disaggregate")

  A <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.00, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )

  expect_error(leslie_disaggregate(A, 0))
  expect_error(leslie_disaggregate(A, -1))
  expect_error(leslie_disaggregate(A, NA_integer_))
  expect_error(
    leslie_disaggregate(A, m = 2.9),
    "m must be an integer."
  )
})

test_that("spectral_radius: computes max modulus eigenvalue for simple cases",
          {
            spectral_radius <- get_fun("spectral_radius")

            A <- diag(c(1, 2, 3))
            expect_equal(spectral_radius(A), 3)

            B <- matrix(c(0, 1, 2, 0), 2, 2, byrow = TRUE)
            # eigenvalues are +/- sqrt(2), spectral radius is sqrt(2)
            expect_equal(spectral_radius(B), sqrt(2), tolerance = 1e-10)
          })

test_that("spectral_radius: handles non-finite and non-square inputs appropriately",
          {
            spectral_radius <- get_fun("spectral_radius")

            A_ns <- matrix(1, 2, 3)
            expect_error(spectral_radius(A_ns))

            A_nf <- diag(2)
            A_nf[1, 1] <- NaN
            expect_error(spectral_radius(A_nf))
          })

#' mpmaggregate: Aggregation of matrix population models
#'
#' @description
#' \pkg{mpmaggregate} aggregates matrix population models (MPMs) under both the
#' \eqn{\lambda} (stable population growth rate) and \eqn{R_0} (net reproductive rate)
#' frameworks, using either standard or elasticity-consistent aggregation.
#'
#' @details
#' The package provides methods for aggregating matrix population models while
#' preserving key demographic quantities under either the \eqn{\lambda} or
#' \eqn{R_0} framework. Aggregation can be performed using either standard or
#' elasticity-consistent criteria, depending on whether reproductive values are
#' to be preserved in addition to growth rates.
#'
#' \itemize{
#'   \item \strong{framework = "lambda"}: aggregation targets the dominant eigenvalue
#'     \eqn{\lambda} and its associated eigenvectors.
#'   \item \strong{framework = "R0"}: aggregation targets the dominant eigenvalue
#'     \eqn{R_0} of a next-generation matrix derived from reproductive and
#'     survival components.
#'   \item \strong{criterion = "standard"}: preserves \eqn{\lambda} and the stable
#'     stage distribution (\eqn{\lambda} framework) or \eqn{R_0} and the cohort
#'     stable stage distribution (\eqn{R_0} framework).
#'   \item \strong{criterion = "elasticity"}: additionally preserves reproductive
#'     values (\eqn{\lambda}) or cohort reproductive values (\eqn{R_0} framework).
#' }
#'
#' Aggregation is implemented using effective left and right operators,
#' \eqn{P_{\mathrm{eff}}} and \eqn{Q_{\mathrm{eff}}}, which map an original matrix
#' \eqn{M} (dimension \eqn{n}) to an aggregated matrix
#' \eqn{M_{\mathrm{agg}}} (dimension \eqn{m}):
#' \deqn{M_{\mathrm{agg}} = P_{\mathrm{eff}} \, M \, Q_{\mathrm{eff}}.}
#'
#' These operators are constructed from the partition matrix \eqn{P} and weights
#' that depend on the selected framework and criterion.
#'
#' Aggregation can be performed in two primary settings:
#' \itemize{
#'   \item General-to-general matrix population model aggregation via \code{\link{mpm_aggregate}}.
#'     Groupings are supplied by the user as a list of stage indices (see
#'     \code{\link{mpm_partition}}).
#'   \item Leslie-to-Leslie matrix population model aggregation via
#'     \code{\link{leslie_aggregate}}, which aggregates age classes into cohort
#'     blocks and may apply a disaggregation step when required.
#' }
#'
#' Effectiveness of aggregation measures the quality of an aggregation by how closely the
#' aggregated model reproduces the behavior of the original model under the chosen
#' framework and criterion.
#'
#' The package assumes that inputs are nonnegative matrices and (when required for
#' computing Perron eigenvectors used in aggregation) that the population projection
#' matrix is irreducible.
#'
#' @section Main functions:
#' \itemize{
#'   \item \code{\link{mpm_aggregate}}: general-to-general MPM aggregation using
#'     user-defined groups of stages.
#'   \item \code{\link{leslie_aggregate}}: Leslie-to-Leslie MPM aggregation into
#'     \code{ngroups} contiguous age classes.
#'   \item \code{\link{generation_time}}: return the generation time in either the
#'     lambda framework or R0 (cohort-based) framework.
#'   \item \code{\link{mpm_elasticity}}: compute elasticities of either the dominant
#'     eigenvalue \eqn{\lambda} (\eqn{\lambda} framework) or the net reproductive rate \eqn{R_0}
#'     (\eqn{R_0} framework) with respect to the entries of the population projection
#'     matrix \code{matA}.
#' }
#'
#' @section Utility functions:
#' \itemize{
#'   \item \code{\link{mpm_partition}}: convert a user-supplied grouping list into a
#'     partitioning matrix.
#'   \item \code{\link{spectral_radius}}: compute the spectral radius (dominant
#'     eigenvalue modulus) of a square matrix.
#'   \item \code{\link{dominant_eigen}}: return the dominant eigenvalue and
#'     its associated stable stage distribution and reproductive values for a
#'     population projection matrix.
#'   \item \code{\link{leslie_dominant_eigen}}: return the dominant eigenvalue
#'     and its associated stable age distribution and reproductive values for a Leslie
#'     population projection matrix.
#'   \item \code{\link{is_leslie}}: check whether the input matrix is a
#'     Leslie population projection matrix.
#'   \item \code{\link{leslie_disaggregate}}: disaggregate a Leslie population projection
#'     model so that its dimensionality is compatible with that of the aggregated model.
#'   \item \code{\link{stable_stage}}: return the stable stage distribution (w) associated
#'     with the dominant eigenvalue, scaled so that \code{sum(w) = 1}.
#'   \item \code{\link{reproductive_values}}: return the reproductive values (v) associated
#'     with the dominant eigenvalue, scaled so that \code{sum(v * w) = 1}.
#'   \item \code{\link{leslie_stable_age}}: return the stable age distribution (w) associated
#'     with the dominant eigenvalue of a Leslie matrix, scaled so that \code{sum(w) = 1}.
#'   \item \code{\link{leslie_reproductive_values}}: return the reproductive values (v) associated
#'     with the dominant eigenvalue of a Leslie matrix, scaled so that \code{sum(v * w) = 1}.
#' }
#'
#' @references
#'
#'  Bienvenu, F., Akçay, E., Legendre, S. & McCandlish, D.M. (2017). The genealogical
#'  decomposition of a matrix population model with applications to the aggregation
#'  of stages. \emph{Theoretical Population Biology}, 115, 69-80. \doi{10.1016/j.tpb.2017.04.002}
#'
#'  Bienvenu, F. & Legendre, S. (2015). A new approach to the generation time in matrix
#'  population models. \emph{The American Naturalist}, 185(6), 834-843. \doi{10.1086/681104}
#'
#'  Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#'  interpretation} (2nd ed.). Sinauer.
#'
#'  Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models
#'  with application to ten diverse animal species. \emph{Population Ecology}, 65(3),
#'  146-166. \doi{10.1002/1438-390X.12149}
#'
#'  Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
#'  application: Elasticity-consistent aggregation of Leslie matrix population
#'  models for comparative demography. \emph{bioRxiv}, preprint. \doi{10.64898/2026.02.04.703802}
#'
#'  Salguero-Gómez, R., & Gamelon, M., eds. (2021). \emph{Demographic methods across the
#'  tree of life}. Oxford University Press.
#'
#'  Salguero-Gómez, R. & Plotkin, J. B. (2010). Matrix dimensions bias
#'  demographic inferences: implications for comparative plant demography. \emph{The
#'  American Naturalist}, 176, 710-722. \doi{10.1086/657044}
#'
#'
#' @seealso
#' \code{\link{mpm_aggregate}}, \code{\link{leslie_aggregate}}
#'
"_PACKAGE"

#' Aggregate Leslie-to-Leslie matrix population model
#'
#' Aggregates a Leslie matrix into a smaller Leslie matrix with \code{ngroups}
#' contiguous age classes.
#'
#' This function assumes the input is a valid Leslie matrix and checks the Leslie
#' structure internally. If the original dimensionality \eqn{n} is not an integer
#' multiple of \eqn{m}, a disaggregation step is applied internally so that the
#' expanded dimensionality is divisible by \eqn{m} before aggregation proceeds.
#'
#' Aggregation can be performed under two frameworks (\code{"lambda"} or \code{"R0"})
#' and two criteria (\code{"standard"} or \code{"elasticity"}), which determine how
#' age classes are weighted during aggregation.
#'
#' Irreducibility requirement: The input Leslie matrix \code{matA} must be
#' irreducible. This function checks irreducibility of \code{matA} and will fail
#' if it is reducible. (Any internal disaggregation preserves irreducibility.)
#'
#' @param matA A square numeric matrix. Must be finite, nonnegative, and satisfy
#'   the structural constraints of a Leslie matrix (checked internally).
#' @param ngroups Positive integer giving the number of aggregated age groups
#'   (the dimension \eqn{m} of the aggregated model). Must satisfy \code{ngroups <= n}.
#' @param framework Character scalar; either \code{"lambda"} or \code{"R0"}.
#'   Determines whether aggregation is based on the projection matrix \eqn{A}
#'   or an \eqn{R_0}-based reference matrix built from the implied \eqn{U} and
#'   reproductive component of the Leslie matrix.
#' @param criterion Character scalar; either \code{"standard"} for standard aggregation
#'  or \code{"elasticity"} for elasticity-consistent aggregation.
#' @param tol Numeric tolerance used in positivity/zero-mass checks and balancing
#'   calculations for the elasticity-consistent case.
#' @param ... Reserved for future use.
#'
#' @details
#' The Leslie matrix is internally decomposed into a reproductive component \eqn{R}
#' (the first row) and a survival component \eqn{U} (the subdiagonal and any survival
#' terms implied by the Leslie structure). For \code{framework = "R0"}, the
#' next-generation matrix \eqn{K = R (I - U)^{-1}} is formed and its dominant
#' eigenvalue is used as \eqn{R_0}.
#'
#' Aggregation proceeds on a \eqn{k}-step matrix where \eqn{k = n/m},
#' with \eqn{n} the dimension of the input Leslie matrix and
#' \eqn{m =} \code{ngroups}. The function returns an
#' \eqn{m \times m} \eqn{k}-step Leslie matrix (stored as
#' \code{matAk_agg}) describing transitions among the aggregated age classes.
#'
#' Effectiveness measures how closely the aggregated model reproduces the behavior
#' of the original model under the chosen \code{framework} and \code{criterion}. When
#' \code{effectiveness} is high (close to 1), applying the original model and then
#' aggregating gives nearly the same result as aggregating first and then applying the
#' aggregated model.
#'
#' The returned element \code{effectiveness} is computed for both criteria:
#' \itemize{
#'   \item For \code{criterion = "standard"}, effectiveness is computed using weights
#'     based on the reference stable age distribution \code{w}.
#'   \item For \code{criterion = "elasticity"}, effectiveness is computed after
#'     balancing transformations (with weights based on \code{w * v}).
#' }
#'
#' @return A named list with elements:
#' \describe{
#'   \item{\code{framework}}{The matched framework used (\code{"lambda"} or \code{"R0"}).}
#'   \item{\code{criterion}}{The matched criterion used (\code{"standard"} or \code{"elasticity"}).}
#'   \item{\code{matAk_agg}}{Aggregated \eqn{k}-step Leslie matrix (\eqn{m \times m}).}
#'   \item{\code{effectiveness}}{A numeric effectiveness measure for the aggregation (definition depends on \code{criterion}).}
#' }
#'
#' @references
#' Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application to
#' ten diverse animal species. \emph{Population Ecology}, 65(3), 146-166.
#' \doi{10.1002/1438-390X.12149}
#'
#' Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
#' application: Elasticity-consistent aggregation of Leslie matrix population
#' models for comparative demography. \emph{bioRxiv}, preprint.
#' \doi{10.64898/2026.02.04.703802}
#'
#' @examples
#' # A simple 3x3 Leslie matrix (fertility rates in first row; survival probabilities on subdiagonal)
#' A <- matrix(c(
#'   0.0, 1.2, 1.8,
#'   0.5, 0.0, 0.0,
#'   0.0, 0.7, 0.0
#' ), nrow = 3, byrow = TRUE)
#'
#' # Aggregate to 2 age groups
#' res_std <- leslie_aggregate(
#'   matA = A,
#'   ngroups = 2,
#'   framework = "lambda",
#'   criterion = "standard"
#' )
#' res_std$matAk_agg
#' res_std$effectiveness
#'
#' res_el <- leslie_aggregate(
#'   matA = A,
#'   ngroups = 2,
#'   framework = "lambda",
#'   criterion = "elasticity"
#' )
#' res_el$matAk_agg
#' res_el$effectiveness
#'
#' @export


leslie_aggregate <- function(matA,
                             ngroups,
                             framework = c("lambda", "R0"),
                             criterion = c("standard", "elasticity"),
                             tol = 1e-12,
                             ...) {

  framework <- match.arg(framework)
  criterion <- match.arg(criterion)
  .is_square <- function(M) is.matrix(M) && nrow(M) == ncol(M)
  .check_nonnegative <- function(M, name) {
    if (is.null(M))
      return(invisible(TRUE))
    if (!is.matrix(M))
      stop(sprintf("%s must be a matrix.", name), call. = FALSE)
    if (any(!is.finite(M)))
      stop(sprintf("%s contains non-finite values.", name),
           call. = FALSE)
    if (any(M < 0))
      stop(sprintf("%s must be nonnegative (contains values < 0).",
                   name), call. = FALSE)
    invisible(TRUE)
  }
  .check_dim <- function(M, n, name) {
    if (is.null(M))
      return(invisible(TRUE))
    if (!.is_square(M) || any(dim(M) != c(n, n))) {
      stop(sprintf("%s must be %dx%d.", name, n, n), call. = FALSE)
    }
    invisible(TRUE)
  }

  # Build Q_eff from reference w and partition P (m x n).
  # Q_eff is n x m with columns summing to 1 within each group, proportional to w.

  .build_Q_eff_from_w <- function(P, w, tol = 1e-12, rcond_tol = 1e-12) {
    if (!is.matrix(P))
      stop("P must be a matrix.", call. = FALSE)
    n <- ncol(P)
    if (length(w) != n)
      stop("w must have length ncol(P).", call. = FALSE)
    if (any(!is.finite(w)))
      stop("w contains non-finite values.", call. = FALSE)
    if (any(w < 0))
      stop("w must be nonnegative.", call. = FALSE)

    # Each aggregated group has positive mass under w
    wg <- as.numeric(P %*% w)
    if (any(wg <= tol)) {
      stop("Cannot construct Q_eff: at least one group has ~0 total mass under w (P %*% w <= tol).",
           call. = FALSE)
    }
    W <- diag(w, nrow = n, ncol = n)
    M <- P %*% W %*% t(P)
    rc <- rcond(M)
    if (!is.finite(rc) || rc < rcond_tol) {
      stop("Cannot invert P %*% W %*% t(P): matrix is singular or ill-conditioned (rcond < rcond_tol).",
           call. = FALSE)
    }
    Q_eff <- W %*% t(P) %*% solve(M)
    return(Q_eff)
  }
  .build_P_eff_elasticity <- function(P, w, v, tol = 1e-12,
                                      rcond_tol = 1e-12) {
    if (!is.matrix(P))
      stop("P must be a matrix.", call. = FALSE)
    n <- ncol(P)
    if (length(v) != n)
      stop("v must have length ncol(P).", call. = FALSE)
    if (length(w) != n)
      stop("w must have length ncol(P).", call. = FALSE)
    if (any(!is.finite(v)) || any(!is.finite(w)))
      stop("v and w must be finite.", call. = FALSE)
    if (any(v < 0) || any(w < 0))
      stop("v and w must be nonnegative.", call. = FALSE)

    # Each aggregated group has positive mass under w
    wg <- as.numeric(P %*% w)
    if (any(wg <= tol)) {
      stop("Cannot construct P_eff: at least one group has ~0 total mass under w (P %*% w <= tol).",
           call. = FALSE)
    }

    # Each aggregated group has positive mass under v*w (needed for invertibility of P V W P^T)
    vwg <- as.numeric(P %*% (v * w))
    if (any(vwg <= tol)) {
      stop("Cannot construct P_eff: at least one group has ~0 total mass under v*w (P %*% (v*w) <= tol).",
           call. = FALSE)
    }
    W <- diag(w, nrow = n, ncol = n)
    V <- diag(v, nrow = n, ncol = n)
    A_left <- P %*% W %*% t(P)
    M <- P %*% V %*% W %*% t(P)
    B_right <- P %*% V
    rc <- rcond(M)
    if (!is.finite(rc) || rc < rcond_tol) {
      stop("Cannot invert P %*% V %*% W %*% t(P): matrix is singular or ill-conditioned (rcond < rcond_tol).",
           call. = FALSE)
    }
    P_eff <- A_left %*% solve(M) %*% B_right
    return(P_eff)
  }
  .aggregate <- function(M, P_eff, Q_eff) {
    if (is.null(M))
      return(NULL)
    return(P_eff %*% M %*% Q_eff)
  }
  .effectiveness <- function(A, B, P, WTS) {
    if (is.null(A))
      return(NULL)
    return(norm(B %*% P %*% sqrt(WTS), type = "F")^2/norm(P %*%
                                                            A %*% sqrt(WTS), type = "F")^2)
  }

  # ---- validate raw inputs ----
  .check_nonnegative(matA, "matA")
  if (!.is_square(matA))
    stop("matA must be square.", call. = FALSE)
  if (!is_leslie(matA))
    stop("leslie_aggregate: Input matrix must be a Leslie matrix.", call. = FALSE)
  .check_irreducible_hj(matA, "matA")
  n <- nrow(matA)
  m <- ngroups
  if (length(ngroups) != 1 || !is.numeric(ngroups) || is.na(ngroups) ||
      ngroups %% 1 != 0 || ngroups < 1) {
    stop("ngroups must be a positive integer.", call. = FALSE)
  }
  if (m > n)
    stop("ngroups must be less than or equal to the dimensionality of matA",
         call. = FALSE)

  # ---- check to see if a disaggregation step is needed
  if (n%%m != 0) {
    matA <- leslie_disaggregate(matA, m)
  }
  n <- nrow(matA)
  k <- n/m

  if (n%%m != 0)
    stop("Internal error: disaggregation did not yield n divisible by m")

  # ---- it is now guaranteed that m divides n evenly
  # ---- build reproduction matrix R

  matR <- matrix(0, nrow = n, ncol = n)
  matR[1, ] = matA[1, ]

  # ---- build survival probability matrix U
  matU <- matrix(0, nrow = n, ncol = n)
  if (n > 1)
    matU[2:n, ] = matA[2:n, ]

  # Partition matrix P (m x n)
  e <- rep(1, k)
  Id <- diag(m)
  P <- kronecker(Id, t(e))

  # ---- choose reference eigen-structure depending on framework ----
  # These are the "new w and v" you referenced for the R0 framework.

  ref <- NULL
  if (framework == "lambda") {
    if (criterion == "standard") {
      lam <- spectral_radius(matA)
      ref <- list(lambda = lam, w = leslie_stable_age(matA))
    }
    else {
      ref <- leslie_dominant_eigen(matA)
    }
  }
  else {    # framework == "R0"
    I <- diag(n)

    # Fundamental matrix N = (I - U)^(-1)

    N <- tryCatch(solve(I - matU), error = function(e) NULL)
    if (is.null(N)) {
      stop("Could not invert (I - U). R0 framework requires (I - U) to be invertible.",
           call. = FALSE)
    }

    # Next-generation matrix and its Perron root

    K <- matR %*% N
    R0 <- spectral_radius(K)
    if (!is.finite(R0) || R0 <= 0) {
      stop("Computed R0 is not positive/finite; check U and R.",
           call. = FALSE)
    }

    # Reference matrix for aggregation eigenstructure in the R0 framework
    # divide by R0 so that it has Leslie matrix form
    A_R0ref <- (matR + R0 * matU)/R0
    if (criterion == "standard") {
      ref <- list(w = leslie_stable_age(A_R0ref))
    }
    else {
      ref <- leslie_dominant_eigen(A_R0ref)
      ref$lambda <- NULL
    }
  }

  # ---- build P_eff and Q_eff for each of the four cases ----
  # Q_eff always depends on the appropriate reference w.

  Q_eff <- .build_Q_eff_from_w(P, ref$w, tol = tol, rcond_tol = tol)
  if (criterion == "standard") {
    P_eff <- P
  }
  else {
    P_eff <- .build_P_eff_elasticity(P = P, w = ref$w, v = ref$v,
                                     tol = tol, rcond_tol = tol)
  }

  # ---- aggregate matrices using P_eff and Q_eff ----

  out <- list(framework = framework, criterion = criterion)

  # k-step ahead projection matrix Ak
  if (framework == "lambda") {
    Ak <- expm::`%^%`(matA, k)
  }
  if (framework == "R0") {
    I <- diag(n)
    Uk <- expm::`%^%`(matU, k)
    Rk <- (expm::`%^%`(A_R0ref, k) - Uk) * R0
    Ak <- Uk + Rk
  }
  out$matAk_agg <- .aggregate(Ak, P_eff, Q_eff)

  #add effectiveness measures
  if (criterion == "standard") {
    out$effectiveness <- .effectiveness(A = Ak, B = out$matAk_agg,
                                        P = P, WTS = diag(ref$w))
  }
  if (criterion == "elasticity") {
    if (min(ref$v) < tol)
      stop("Balancing A requires its reproductive values > 0",
           call. = FALSE)
    Ak_balanced <- diag(ref$v) %*% Ak %*% diag(1/ref$v)
    B_w <- P %*% ref$w
    B_w <- c(B_w)
    W <- diag(ref$w, nrow = n, ncol = n)
    B_v <- solve(P %*% W %*% t(P)) %*% P %*% W %*% ref$v
    B_v <- c(B_v)
    if (min(B_v) < tol)
      stop("Balancing B requires its reproductive values > 0",
           call. = FALSE)
    B_balanced <- diag(B_v, m, m) %*% out$matAk_agg %*% diag(1/B_v,
                                                             m, m)
    out$effectiveness <- .effectiveness(A = Ak_balanced,
                                        B = B_balanced, P = P, WTS = diag(ref$w * ref$v))
  }
  return(out)
}

#' Compute the spectral radius of a matrix
#'
#' Returns the spectral radius of a square matrix, defined as the maximum
#' modulus of its eigenvalues:
#' \deqn{\rho(A) = \max |\lambda_i|}
#'
#' This function is primarily intended for use with matrix population models,
#' where the spectral radius corresponds to the stable growth rate
#' (i.e., \eqn{\lambda}) when the governing matrix is square and nonnegative.
#' It is also used to calculate net reproductive rate \eqn{R_0} from a
#' next generation matrix.
#'
#' @param A A square numeric matrix.
#'
#' @return A single numeric value giving the spectral radius of \code{A}.
#'
#' @references
#' Horn, R. A. and Johnson, C. R. (2013). \emph{Matrix analysis}. Cambridge University Press.
#'
#' @examples
#' A <- matrix(c(
#'   0, 1,
#'   0.5, 0
#' ), nrow = 2, byrow = TRUE)
#'
#' spectral_radius(A)
#'
#' @export
spectral_radius <- function(A) {
  if (!is.matrix(A)) {
    stop("A must be a matrix.", call. = FALSE)
  }
  if (nrow(A) != ncol(A)) {
    stop("A must be square.", call. = FALSE)
  }
  if (any(!is.finite(A))) {
    stop("A contains non-finite values.", call. = FALSE)
  }

  ev <- eigen(A, only.values = TRUE)$values
  max(Mod(ev))
}

#' Compute the stable stage distribution
#'
#' Returns the stable stage distribution for a square nonnegative matrix \code{A}, defined as the
#' right eigenvector associated with the dominant eigenvalue. The returned vector \code{w}
#' is normalized so that \code{sum(w) = 1}.
#'
#' For matrix population models with nonegative irreducible matrix, stable stage densities are strictly
#' positive.
#'
#' @param A A square numeric matrix.
#' @param tol Numeric tolerance used for positivity/normalization checks.
#' @param ensure_positive Logical. If \code{TRUE}, attempts to orient the eigenvector so its
#'   entries are mostly positive by flipping its sign if necessary.
#'
#' @return A numeric vector of length \code{nrow(A)} giving the stable stage distribution,
#'   normalized to sum to 1.
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' @examples
#' #irreducible example
#' A <- matrix(c(
#'   0, 1,
#'   0.5, 0
#' ), nrow = 2, byrow = TRUE)
#'
#' w <- stable_stage(A)
#' w
#' sum(w)
#'
#' @export
stable_stage <- function(A,
                         tol = 1e-12,
                         ensure_positive = TRUE) {
  if (!is.matrix(A))
    stop("A must be a matrix.", call. = FALSE)
  if (nrow(A) != ncol(A))
    stop("A must be square.", call. = FALSE)
  if (any(!is.finite(A)))
    stop("A contains non-finite values.", call. = FALSE)

  # Use the package's spectral radius helper
  rho <- spectral_radius(A)

  # Compute eigen decomposition
  e <- eigen(A)

  # Identify eigenvalue corresponding to the spectral radius
  idx <- which.min(abs(Mod(e$values) - rho))
  w <- Re(e$vectors[, idx])

  if (ensure_positive) {
    # Eigenvectors are defined up to sign; orient toward positive mass
    if (sum(w) < 0)
      w <- -w
  }

  s <- sum(w)
  if (!is.finite(s) || abs(s) <= tol) {
    stop("Could not normalize stable stage distribution (sum ~ 0).",
         call. = FALSE)
  }

  w <- w / s

  # For typical MPM usage, stable stage distribution should be nonnegative
  if (any(w < -tol)) {
    stop(
      "Stable stage distribution has negative entries; ",
      "A may be reducible or not satisfy Perron-Frobenius conditions.",
      call. = FALSE
    )
  }

  # Clip tiny numerical negatives and renormalize
  w[w < 0] <- 0
  w <- w / sum(w)

  return(w)
}

#' Compute reproductive values
#'
#' Returns the reproductive value vector \code{v} for a square nonegative matrix \code{A}, defined as the
#' left eigenvector associated with the dominant eigenvalue (spectral radius). The vector is
#' normalized in the usual demographic way so that \code{sum(v * w) = 1}, where \code{w} is the
#' stable stage distribution returned by \code{\link{stable_stage}}.
#'
#' For matrix population models with nonegative irreducible matrix, reproductive values are strictly
#' positive (up to scaling).
#'
#' @param A A square numeric nonnegative matrix.
#' @param tol Numeric tolerance used for normalization/positivity checks.
#' @param ensure_positive Logical. If \code{TRUE}, attempts to orient the eigenvector so its
#'   entries are mostly positive by flipping its sign if necessary.
#'
#' @return A numeric vector of length \code{nrow(A)} giving reproductive values, scaled so
#'   that \code{sum(v * w) = 1}.
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' @examples
#' #irreducible example
#' A <- matrix(c(
#'   0, 1,
#'   0.5, 0
#' ), nrow = 2, byrow = TRUE)
#'
#' w <- stable_stage(A)
#' v <- reproductive_values(A)
#' v
#' sum(v * w)  # should be 1
#'
#' @export
reproductive_values <- function(A,
                                tol = 1e-12,
                                ensure_positive = TRUE) {
  if (!is.matrix(A))
    stop("A must be a matrix.", call. = FALSE)
  if (nrow(A) != ncol(A))
    stop("A must be square.", call. = FALSE)
  if (any(!is.finite(A)))
    stop("A contains non-finite values.", call. = FALSE)

  # Call helpers as requested
  rho <- spectral_radius(A)
  w <- stable_stage(A, tol = tol, ensure_positive = ensure_positive)

  # Left eigenvector is right eigenvector of t(A)
  e <- eigen(t(A))
  idx <- which.min(abs(Mod(e$values) - rho))
  v <- Re(e$vectors[, idx])

  if (ensure_positive) {
    if (sum(v) < 0)
      v <- -v
  }

  # Normalize so sum(v * w) = 1
  vw <- sum(v * w)
  if (!is.finite(vw) || abs(vw) <= tol) {
    stop("Could not normalize reproductive values (sum(v * w) ~ 0).",
         call. = FALSE)
  }
  v <- v / vw

  # Typical MPM expectation: nonnegative reproductive values (allow tiny numerical negatives)
  if (any(v < -tol)) {
    stop(
      "Reproductive values have negative entries; ",
      "A may be reducible or not satisfy Perron-Frobenius conditions.",
      call. = FALSE
    )
  }

  # Clip tiny negatives and re-normalize to preserve sum(v*w)=1
  v[v < 0] <- 0
  vw2 <- sum(v * w)
  if (!is.finite(vw2) || abs(vw2) <= tol) {
    stop("Re-normalization failed after clipping; check A.", call. = FALSE)
  }
  v <- v / vw2

  return(v)
}

#' Dominant eigen-elements of a population projection matrix
#'
#' Returns the dominant eigenvalue and its associated right/left eigenvectors
#' (stable stage distribution \code{w} and reproductive values \code{v}) for a square
#' population projection matrix \code{A}.
#'
#' The output is normalized using the conventions in \code{\link{stable_stage}} and
#' \code{\link{reproductive_values}}:
#' \itemize{
#'   \item \code{w} is scaled so that \code{sum(w) = 1}
#'   \item \code{v} is scaled so that \code{sum(v * w) = 1}
#' }
#'
#' @param A A square numeric matrix.
#' @param tol Numeric tolerance passed to \code{\link{stable_stage}} and
#'   \code{\link{reproductive_values}}.
#' @param ensure_positive Logical. Passed to \code{\link{stable_stage}} and
#'   \code{\link{reproductive_values}}. If \code{TRUE}, attempts to orient eigenvectors
#'   so entries are mostly positive by flipping sign if needed.
#'
#' @return A named list with elements:
#' \itemize{
#'   \item \code{lambda}: spectral radius of \code{A}
#'   \item \code{w}: stable stage distribution (right eigenvector), \code{sum(w)=1}
#'   \item \code{v}: reproductive values (left eigenvector), \code{sum(v*w)=1}
#' }
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' @examples
#' A <- matrix(c(
#'   0,   1,   2,
#'   0.5, 0,   0,
#'   0,   0.8, 0.9
#' ), nrow = 3, byrow = TRUE)
#'
#' dom <- dominant_eigen(A)
#' dom$lambda
#' sum(dom$w)
#' sum(dom$v * dom$w)
#'
#' @seealso \code{\link{spectral_radius}}, \code{\link{stable_stage}},
#' \code{\link{reproductive_values}}
#' @export
dominant_eigen <- function(A,
                           tol = 1e-12,
                           ensure_positive = TRUE) {
  if (!is.matrix(A))
    stop("A must be a matrix.", call. = FALSE)
  if (nrow(A) != ncol(A))
    stop("A must be square.", call. = FALSE)
  if (any(!is.finite(A)))
    stop("A contains non-finite values.", call. = FALSE)

  lambda <- spectral_radius(A)
  w <- stable_stage(A, tol = tol, ensure_positive = ensure_positive)
  v <- reproductive_values(A, tol = tol, ensure_positive = ensure_positive)

  return(list(lambda = lambda, w = w, v = v))
}



#' Stable age distribution for a Leslie matrix
#'
#' Computes the stable age distribution for a Leslie matrix \code{A} using the standard
#' recursion based on subdiagonal survival probabilities and the dominant eigenvalue.
#' The output is scaled so that \code{sum(w) = 1}.
#'
#' A length-1 positive numeric is treated as a 1x1 Leslie matrix and returns \code{1}.
#'
#' @param A A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
#'   will be coerced to a 1x1 matrix.
#'
#' @return A numeric vector of length \code{nrow(A)} giving the stable age distribution,
#'   scaled so that \code{sum(w) = 1}.
#'
#' @examples
#' leslie_stable_age(1)
#'
#' L <- matrix(c(
#'   0, 2, 1,
#'   0.6, 0, 0,
#'   0, 0.7, 0
#' ), nrow = 3, byrow = TRUE)
#' leslie_stable_age(L)
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' Demetrius, L. (1974). Demographic parameters and natural selection.
#' \emph{Proceedings of the National Academy of Sciences}, 71(12), 4645-4647.
#' \doi{10.1073/pnas.71.12.4645}
#'
#' @export
leslie_stable_age <- function(A) {
  # Allow scalar 1x1 case
  if (is.null(dim(A))) {
    if (is.numeric(A) && length(A) == 1L && is.finite(A) && A > 0) {
      A <- matrix(A, nrow = 1, ncol = 1)
    }
  }

  if (!is_leslie(A))
    stop("Input matrix must be a Leslie matrix.", call. = FALSE)

  n <- nrow(A)
  if (n == 1L)
    return(1)

  lambda <- spectral_radius(A)
  if (!is.finite(lambda) || lambda <= 0) {
    stop(
      "Spectral radius of A is not positive; cannot compute a stable age distribution.",
      call. = FALSE
    )
  }

  # Subdiagonal survivals s_i = A[i+1, i], with s[1] = 1 for the recursion start
  s <- c(1, A[cbind(2:n, 1:(n - 1))])

  w <- numeric(n)
  w[1] <- 1
  for (i in 2:n) {
    w[i] <- w[i - 1] * s[i] / lambda
  }

  w <- w / sum(w)
  return(w)
}

#' Reproductive values for a Leslie matrix
#'
#' Computes the reproductive value vector \code{v} for a Leslie matrix \code{A}, scaled so that
#' \code{sum(v * w) = 1}, where \code{w} is the stable age distribution returned by
#' \code{\link{leslie_stable_age}}.
#'
#' This implementation follows Demetrius (1974) equation expressed in terms of
#' the stable age distribution and fertility rates.
#'
#' A length-1 positive numeric is treated as a 1x1 Leslie matrix and returns \code{1}.
#'
#' @param A A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
#'   will be coerced to a 1x1 matrix.
#'
#' @return A numeric vector of length \code{nrow(A)} giving reproductive values, normalized so that
#'   \code{sum(v * w) = 1}.
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' Demetrius, L. (1974). Demographic parameters and natural selection.
#' \emph{Proceedings of the National Academy of Sciences}, 71(12), 4645-4647.
#' \doi{10.1073/pnas.71.12.4645}
#'
#' @examples
#' leslie_reproductive_values(1)
#'
#' L <- matrix(c(
#'   0, 2, 1,
#'   0.6, 0, 0,
#'   0, 0.7, 0
#' ), nrow = 3, byrow = TRUE)
#' v <- leslie_reproductive_values(L)
#' w <- leslie_stable_age(L)
#' sum(v * w)  # should be 1
#'
#' @export
leslie_reproductive_values <- function(A) {
  # Allow scalar 1x1 case
  if (is.null(dim(A))) {
    if (is.numeric(A) && length(A) == 1L && is.finite(A) && A > 0) {
      A <- matrix(A, nrow = 1, ncol = 1)
    }
  }

  if (!is_leslie(A))
    stop("Input matrix must be a Leslie matrix.", call. = FALSE)

  n <- nrow(A)
  if (n == 1L)
    return(1)

  # Call spectral_radius as requested (also sanity: must be positive)
  lambda <- spectral_radius(A)
  if (!is.finite(lambda) || lambda <= 0) {
    stop("Spectral radius of A is not positive; cannot compute reproductive values.",
         call. = FALSE)
  }

  # Stable age distribution (calls your recursion-based method)
  w <- leslie_stable_age(A)

  # Fertility vector (first row)
  m <- A[1, ]

  # Demetrius-style scaling constant:
  # k = 1 / sum_i ( w_i * m_i * i )
  denom <- sum(w * m * seq_len(n))
  if (!is.finite(denom) || denom <= 0) {
    stop(
      "Could not compute Demetrius scaling constant (denominator <= 0). Check fertilities and w.",
      call. = FALSE
    )
  }
  k <- 1 / denom

  v <- numeric(n)
  for (i in seq_len(n)) {
    num <- k * sum(m[i:n] * w[i:n])
    if (w[i] == 0) {
      stop("Stable age distribution has a zero entry; cannot compute v[i] = num / w[i].",
           call. = FALSE)
    }
    v[i] <- num / w[i]
  }

  # Normalize so sum(v * w) = 1 (usual demographic scaling)
  vw <- sum(v * w)
  if (!is.finite(vw) || vw <= 0) {
    stop("Could not normalize reproductive values (sum(v * w) <= 0).",
         call. = FALSE)
  }
  v <- v / vw

  return(v)
}

#' Dominant eigen-elements of a Leslie matrix
#'
#' Returns the dominant eigenvalue (\eqn{\lambda}) and the stable age distribution
#' \code{w} and reproductive values \code{v} for a Leslie matrix \code{A}.
#'
#' Normalization:
#' The output is normalized using the conventions in \code{\link{leslie_stable_age}} and
#' \code{\link{leslie_reproductive_values}}
#' \itemize{
#'   \item \code{w} is scaled so that \code{sum(w) = 1}
#'   \item \code{v} is scaled so that \code{sum(v * w) = 1}
#' }
#'
#'
#' @param A A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
#'   will be coerced to a 1x1 matrix.
#'
#' @return A named list with elements:
#' \itemize{
#'   \item \code{lambda}: spectral radius of \code{A}
#'   \item \code{w}: stable age distribution, \code{sum(w)=1}
#'   \item \code{v}: reproductive values, \code{sum(v*w)=1}
#' }
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' Demetrius, L. (1974). Demographic parameters and natural selection.
#' \emph{Proceedings of the National Academy of Sciences}, 71(12), 4645-4647.
#' \doi{10.1073/pnas.71.12.4645}
#'
#' @examples
#' leslie_dominant_eigen(1)
#'
#' L <- matrix(c(
#'   0, 2, 1,
#'   0.6, 0, 0,
#'   0, 0.7, 0
#' ), nrow = 3, byrow = TRUE)
#' dom <- leslie_dominant_eigen(L)
#' dom$lambda
#' sum(dom$w)
#' sum(dom$v * dom$w)
#'
#' @seealso \code{\link{spectral_radius}}, \code{\link{leslie_stable_age}},
#' \code{\link{leslie_reproductive_values}}
#' @export
leslie_dominant_eigen <- function(A) {
  # Allow scalar 1x1 case
  if (is.null(dim(A))) {
    if (is.numeric(A) && length(A) == 1L && is.finite(A) && A > 0) {
      A <- matrix(A, nrow = 1, ncol = 1)
    }
  }

  if (!is_leslie(A))
    stop("Input matrix must be a Leslie matrix.", call. = FALSE)

  n <- nrow(A)
  if (n == 1L) {
    lambda <- spectral_radius(A)  # equals A[1,1] for 1x1
    return(list(lambda = lambda, w = 1, v = 1))
  }

  lambda <- spectral_radius(A)
  w <- leslie_stable_age(A)
  v <- leslie_reproductive_values(A)

  return(list(lambda = lambda, w = w, v = v))
}

test_that("generation_time: input validation works", {
  matU <- matrix(
    c(
      0.2, 0.0,
      0.3, 0.4
    ),
    nrow = 2, byrow = TRUE
  )

  matF <- matrix(
    c(
      0.0, 1.2,
      0.0, 0.0
    ),
    nrow = 2, byrow = TRUE
  )

  # matF / matU must not be NULL
  expect_error(
    generation_time(
      matF = NULL,
      matU = matU,
      framework = "lambda"
    ),
    "`matF` must not be NULL"
  )
  expect_error(
    generation_time(
      matF = matF,
      matU = NULL,
      framework = "lambda"
    ),
    "`matU` must not be NULL"
  )

  # must be numeric matrices
  expect_error(
    generation_time(
      matF = list(1, 2),
      matU = matU,
      framework = "lambda"
    ),
    "must be a numeric matrix"
  )

  # must be square
  matF_ns <- matrix(1, nrow = 2, ncol = 3)
  expect_error(generation_time(
    matF = matF_ns,
    matU = matU,
    framework = "lambda"
  ),
  "must be square")

  # dimensions must match
  matU3 <- diag(3)
  expect_error(
    generation_time(
      matF = matF,
      matU = matU3,
      framework = "lambda"
    ),
    "must have identical dimensions"
  )

  # matC dimensions must match
  matC_bad <- diag(3)
  expect_error(
    generation_time(
      matF = matF,
      matU = matU,
      matC = matC_bad,
      framework = "lambda"
    ),
    "matC.*same dimensions"
  )

  # must be nonnegative
  matF_neg <- matF
  matF_neg[1, 1] <- -0.1
  expect_error(
    generation_time(
      matF = matF_neg,
      matU = matU,
      framework = "lambda"
    ),
    "must be nonnegative"
  )

  # no NA
  matU_na <- matU
  matU_na[1, 1] <- NA_real_
  expect_error(generation_time(
    matF = matF,
    matU = matU_na,
    framework = "lambda"
  ),
  "contains NA")
})


test_that("generation_time: lambda framework returns expected structure and value (mocked)",
          {
            matU <- matrix(
              c(
                0.2, 0.0,
                0.3, 0.4
              ),
              nrow = 2, byrow = TRUE
            )

            matF <- matrix(
              c(
                0.0, 1.2,
                0.0, 0.0
              ),
              nrow = 2, byrow = TRUE
            )

            # matR == matF here
            # Choose mocked eigen outputs
            w <- c(1, 1)
            v <- c(2, 3)
            lambda <- 4
            denom <- as.numeric(t(v) %*% matF %*% w)
            expected_gt <- lambda / denom

            local_mocked_bindings(
              dominant_eigen = function(...)
                list(w = w, v = v, lambda = lambda)
            )

            out <- generation_time(matF = matF,
                                   matU = matU,
                                   framework = "lambda")

            expect_type(out, "list")
            expect_named(out, c("framework", "generation_time"))
            expect_equal(out$framework, "lambda")
            expect_true(is.numeric(out$generation_time) &&
                          length(out$generation_time) == 1)
            expect_equal(out$generation_time, expected_gt, tolerance = 1e-12)
          })


test_that("generation_time: R0 framework runs and returns expected value (mocked)",
          {
            matU <- matrix(
              c(
                0.2, 0.0,
                0.3, 0.4
              ),
              nrow = 2, byrow = TRUE
            )

            matF <- matrix(
              c(
                0.0, 1.2,
                0.0, 0.0
              ),
              nrow = 2, byrow = TRUE
            )

            # Mocked eigen outputs used after Acohort is constructed
            w <- c(1, 1)
            v <- c(2, 3)
            lambda_acohort <- 5
            denom <- as.numeric(t(v) %*% matF %*% w)  # same denom as above (2.4)
            expected_gt <- lambda_acohort / denom

            local_mocked_bindings(
              spectral_radius = function(...)
                1.7,
              # any positive number; just to pass and build Acohort
              dominant_eigen  = function(...)
                list(w = w, v = v, lambda = lambda_acohort)
            )

            out <- generation_time(matF = matF,
                                   matU = matU,
                                   framework = "R0")

            expect_type(out, "list")
            expect_named(out, c("framework", "generation_time"))
            expect_equal(out$framework, "R0")
            expect_true(is.numeric(out$generation_time) &&
                          length(out$generation_time) == 1)
            expect_equal(out$generation_time, expected_gt, tolerance = 1e-12)
          })


test_that("generation_time: R0 framework errors when (I - U) is singular", {
  # Make I - U singular: U has eigenvalue 1, e.g. identity
  matU <- diag(2)
  matF <- matrix(c(0, 1, 0, 0), nrow = 2, byrow = TRUE)

  # We mock spectral_radius/dominant_eigen, but we should fail before they are needed.
  local_mocked_bindings(
    spectral_radius = function(...)
      1.0,
    dominant_eigen  = function(...)
      list(
        w = c(1, 1),
        v = c(1, 1),
        lambda = 1
      )
  )

  expect_error(generation_time(
    matF = matF,
    matU = matU,
    framework = "R0"
  ),
  "singular")
})

test_that("generation_time: matches famous teasel example", {
  matA <- matrix(
    c(
      0,     0,     0,     0,     0,     322.388,
      0.966, 0,     0,     0,     0,     0,
      0.013, 0.010, 0.125, 0,     0,     3.448,
      0.007, 0,     0.125, 0.238, 0,     30.170,
      0.008, 0,     0.038, 0.245, 0.167, 0.862,
      0,     0,     0,     0.023, 0.750, 0
    ),
    nrow = 6, byrow = TRUE
  )
  matF <- matrix(0, 6, 6)
  matU <- matrix(0, 6, 6)
  matF[, 6] <- matA[, 6]
  matU[1:6, 1:5] <- matA[1:6, 1:5]
  gt = generation_time(matF = matF,
                       matU = matU,
                       framework = "lambda")$generation_time
  # "2.91" calculated in Bienvenu & Legendre (2015).
  expect_equal(round(gt, 2), 2.91, tolerance = 1.e-2)
})

test_that("generation_time: test against cohort generation time of Ellner (2018)",
          {
            matA <- matrix(
              c(
                0,     0,     0,     0,     0,     322.388,
                0.966, 0,     0,     0,     0,     0,
                0.013, 0.010, 0.125, 0,     0,     3.448,
                0.007, 0,     0.125, 0.238, 0,     30.170,
                0.008, 0,     0.038, 0.245, 0.167, 0.862,
                0,     0,     0,     0.023, 0.750, 0
              ),
              nrow = 6, byrow = TRUE
            )
            matF <- matrix(0, 6, 6)
            matU <- matrix(0, 6, 6)
            matF[, 6] <- matA[, 6]
            matU[1:6, 1:5] <- matA[1:6, 1:5]
            gt = generation_time(matF = matF,
                                 matU = matU,
                                 framework = "R0")$generation_time
            #Ellner calculation of cohort-based generation time
            IminusU_inverse <- solve(diag(6) - matU)
            K <- matF %*% IminusU_inverse
            eigK <- dominant_eigen(K)
            gt_test <- c(eigK$v %*% IminusU_inverse %*% eigK$w) / c(t(eigK$v) %*% eigK$w)
            expect_equal(gt, gt_test, tolerance = 1e-10)
          })

test_that(
  "leslie_aggregate: lambda framework (standard + elasticity) returns expected structure",
  {
    # A simple 3x3 Leslie matrix
    A <- matrix(
      c(
        0.10, 1.00, 6.35,
        0.33, 0.00, 0.00,
        0.00, 0.50, 0.00
      ),
      nrow = 3, byrow = TRUE
    )

    res_std <- leslie_aggregate(
      matA = A,
      ngroups = 2,
      framework = "lambda",
      criterion = "standard"
    )

    res_elas <- leslie_aggregate(
      matA = A,
      ngroups = 2,
      framework = "lambda",
      criterion = "elasticity"
    )

    expect_type(res_std, "list")
    expect_equal(res_std$framework, "lambda")
    expect_equal(res_std$criterion, "standard")
    expect_true(is.matrix(res_std$matAk_agg))
    expect_equal(dim(res_std$matAk_agg), c(2, 2))
    expect_true(all(is.finite(res_std$matAk_agg)))
    expect_true(is.finite(res_std$effectiveness))

    expect_type(res_elas, "list")
    expect_equal(res_elas$framework, "lambda")
    expect_equal(res_elas$criterion, "elasticity")
    expect_true(is.matrix(res_elas$matAk_agg))
    expect_equal(dim(res_elas$matAk_agg), c(2, 2))
    expect_true(all(is.finite(res_elas$matAk_agg)))
    expect_true(is.finite(res_elas$effectiveness))
  }
)

test_that("leslie_aggregate: aggregation preserves eigen components (lambda framework)",
          {
            #test that lambda and stable stage distributions are consistent.
            #example from Hooley (2000)
            A <- matrix(
              c(
                0.00, 0.00, 37.00, 64.00, 82.00,
                0.06, 0.00,  0.00,  0.00,  0.00,
                0.00, 0.34,  0.00,  0.00,  0.00,
                0.00, 0.00,  0.16,  0.00,  0.00,
                0.00, 0.00,  0.00,  0.08,  0.00
              ),
              nrow = 5, byrow = TRUE
            )

            m <- 4
            n <- 5
            k <- n / m
            res_std <- leslie_aggregate(
              matA = A,
              ngroups = m,
              framework = "lambda",
              criterion = "standard"
            )
            res_elas <- leslie_aggregate(
              matA = A,
              ngroups = m,
              framework = "lambda",
              criterion = "elasticity"
            )

            eigen_std <- leslie_dominant_eigen(res_std$matAk_agg)
            eigen_elas <- leslie_dominant_eigen(res_elas$matAk_agg)
            eigen_true <- leslie_dominant_eigen(A)

            #lambdas consistent?
            expect_equal(as.numeric(eigen_std$lambda),
                         as.numeric(eigen_true$lambda)^k,
                         tolerance = 1e-8)
            expect_equal(as.numeric(eigen_elas$lambda),
                         as.numeric(eigen_true$lambda)^k,
                         tolerance = 1e-8)
            #stable stage distributions consistent?
            C <- leslie_disaggregate(A, m)
            eigen_C <- leslie_dominant_eigen(C)
            n <- nrow(C)
            k <- n / m
            # Partition matrix P (m x n)
            e <- rep(1, k)
            Id <- diag(m)
            P <- kronecker(Id, t(e))

            expect_equal(as.numeric(eigen_std$w),
                         as.numeric(P %*% eigen_C$w),
                         tolerance = 1.e-8)
            expect_equal(as.numeric(eigen_elas$w),
                         as.numeric(P %*% eigen_C$w),
                         tolerance = 1.e-8)
            #reproductive values consistent?
            w <- eigen_C$w
            v <- eigen_C$v
            W <- diag(w)
            vtest <- solve(P %*% W %*% t(P)) %*% P %*% (w * v)
            expect_equal(as.numeric(eigen_elas$v), as.numeric(vtest), tolerance =
                           1.e-8)
          })

test_that("leslie_aggregate: aggregation preserves eigen components (R0 framework)",
          {
            #test that lambda and stable stage distributions are consistent.
            #example from Hooley (2000)
            A <- matrix(
              c(
                0.00, 0.00, 37.00, 64.00, 82.00,
                0.06, 0.00,  0.00,  0.00,  0.00,
                0.00, 0.34,  0.00,  0.00,  0.00,
                0.00, 0.00,  0.16,  0.00,  0.00,
                0.00, 0.00,  0.00,  0.08,  0.00
              ),
              nrow = 5, byrow = TRUE
            )

            m <- 4
            n <- 5

            U <- matrix(0, 5, 5)
            F <- matrix(0, 5, 5)
            U[2:5, ] <- A[2:5, ]
            F[1, ] <- A[1, ]

            res_std <- leslie_aggregate(
              matA = A,
              ngroups = m,
              framework = "R0",
              criterion = "standard"
            )
            res_elas <- leslie_aggregate(
              matA = A,
              ngroups = m,
              framework = "R0",
              criterion = "elasticity"
            )

            #construct aggregated generation matrices
            F_std <- matrix(0, m, m)
            F_std[1, ] <- res_std$matAk_agg[1, ]
            U_std <- matrix(0, m, m)
            U_std[2:m, ] <- res_std$matAk_agg[2:m, ]
            F_elas <- matrix(0, m, m)
            F_elas[1, ] <- res_elas$matAk_agg[1, ]
            U_elas <- matrix(0, m, m)
            U_elas[2:m, ] <- res_elas$matAk_agg[2:m, ]
            K_std <- F_std %*% solve(diag(m) - U_std)
            K_elas <- F_elas %*% solve(diag(m) - U_elas)
            K_true <- F %*% solve(diag(n) - U)

            R0_std <- spectral_radius(K_std)
            R0_elas <- spectral_radius(K_elas)
            R0_true <- spectral_radius(K_true)

            #R0s equal?
            expect_equal(as.numeric(R0_std), as.numeric(R0_true), tolerance = 1e-8)
            expect_equal(as.numeric(R0_elas), as.numeric(R0_true), tolerance = 1e-8)

            C <- leslie_disaggregate(A, m)
            eigen_C <- leslie_dominant_eigen(C)
            n <- nrow(C)
            k <- n / m
            # Partition matrix P (m x n)
            e <- rep(1, k)
            Id <- diag(m)
            P <- kronecker(Id, t(e))

            U_C <- matrix(0, n, n)
            U_C[2:n, ] <- C[2:n, ]
            F_C <- matrix(0, n, n)
            F_C[1, ] <- C[1, ]
            K_C <- F_C %*% solve(diag(n) - U_C)
            R0 <- spectral_radius(K_C)

            #test whether R0 equals R0_true
            expect_equal(as.numeric(R0), as.numeric(R0_true), tolerance = 1.e-8)

            C_cohort <- (F_C + R0 * U_C) / R0
            eigen_C <- leslie_dominant_eigen(C_cohort)

            #get aggregated cohort eigenstuff
            cohort_std <- (F_std + R0_std * U_std) / R0_std
            cohort_elas <- (F_elas + R0_elas * U_elas) / R0_elas
            eigen_std <- leslie_dominant_eigen(cohort_std)
            eigen_elas <- leslie_dominant_eigen(cohort_elas)

            #cohort stable stage distributions consistent?
            expect_equal(as.numeric(eigen_std$w),
                         as.numeric(P %*% eigen_C$w),
                         tolerance = 1.e-8)
            expect_equal(as.numeric(eigen_elas$w),
                         as.numeric(P %*% eigen_C$w),
                         tolerance = 1.e-8)

            #cohort reproductive values consistent?
            w <- eigen_C$w
            v <- eigen_C$v
            W <- diag(w)
            vtest <- solve(P %*% W %*% t(P)) %*% P %*% (w * v)
            expect_equal(as.numeric(eigen_elas$v), as.numeric(vtest), tolerance =
                           1.e-8)
          })


test_that("leslie_aggregate: R0 framework runs for a valid Leslie matrix", {
  A <- matrix(
    c(
      0.10, 1.00, 6.35,
      0.33, 0.00, 0.00,
      0.00, 0.50, 0.00
    ),
    nrow = 3, byrow = TRUE
  )

  res_std <- leslie_aggregate(
    matA = A,
    ngroups = 2,
    framework = "R0",
    criterion = "standard"
  )

  expect_equal(res_std$framework, "R0")
  expect_equal(res_std$criterion, "standard")
  expect_true(is.matrix(res_std$matAk_agg))
  expect_equal(dim(res_std$matAk_agg), c(2, 2))
  expect_true(all(is.finite(res_std$matAk_agg)))
  expect_true(is.finite(res_std$effectiveness))

  res_elas <- leslie_aggregate(
    matA = A,
    ngroups = 2,
    framework = "R0",
    criterion = "elasticity"
  )

  expect_equal(res_elas$framework, "R0")
  expect_equal(res_elas$criterion, "elasticity")
  expect_true(is.matrix(res_elas$matAk_agg))
  expect_equal(dim(res_elas$matAk_agg), c(2, 2))
  expect_true(all(is.finite(res_elas$matAk_agg)))
  expect_true(is.finite(res_elas$effectiveness))
})

test_that("leslie_aggregate: errors when A is not Leslie or ngroups invalid",
          {
            # Not a Leslie (has nonzero outside first row and subdiagonal)
            A_not <- matrix(
              c(
                0.10, 1.00, 6.35,
                0.33, 0.10, 0.20,
                0.00, 0.50, 0.00
              ),
              nrow = 3, byrow = TRUE
            )

            expect_error(
              leslie_aggregate(
                matA = A_not,
                ngroups = 2,
                framework = "lambda",
                criterion = "standard"
              ),
              regexp = "must be a Leslie",
              ignore.case = TRUE
            )

            # ngroups > n
            A <- matrix(
              c(
                0.10, 1.00, 6.35,
                0.33, 0.00, 0.00,
                0.00, 0.50, 0.00
              ),
              nrow = 3, byrow = TRUE
            )
            expect_error(leslie_aggregate(matA = A, ngroups = 99),
                         regexp = "ngroups.*less than or equal",
                         ignore.case = TRUE)

            # ngroups not an integer
            expect_error(
              leslie_aggregate(matA = A, ngroups = 2.9),
              regexp = "ngroups.*integer",
              ignore.case = TRUE
            )
          })

#add edge tests to see how Leslie_aggregate deals with 1x1 output
test_that("leslie_aggregate: test if properly returns a 1x1 matrix", {
  #get true results
  matA <- matrix(
    c(
      10,  9,
      0.1, 0
    ),
    nrow = 2, byrow = TRUE
  )
  matU <- matA
  matU[1, ] <- 0
  matF <- matA
  matF[2, ] <- 0
  m <- 1
  n <- nrow(matA)
  k <- n / m
  ngroups <- m
  true_lambda <- spectral_radius(matA)
  true_R0 <- spectral_radius(matF %*% solve(diag(2) - matU))

  #get aggregated results
  res <- leslie_aggregate(
    matA = matA,
    ngroups = 1,
    framework = "lambda",
    criterion = "standard"
  )
  expect_equal(c(res$matAk_agg), true_lambda^k, tolerance = 1.e-8)
  res <- leslie_aggregate(
    matA = matA,
    ngroups = 1,
    framework = "lambda",
    criterion = "elasticity"
  )
  expect_equal(c(res$matAk_agg), true_lambda^k, tolerance = 1.e-8)


  res <- leslie_aggregate(
    matA = matA,
    ngroups = 1,
    framework = "R0",
    criterion = "standard"
  )
  matF_agg <- as.matrix(res$matAk_agg)
  matU_agg <- as.matrix(0)
  agg_R0 <- spectral_radius(matF_agg %*% solve(diag(1) - matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)

  res <- leslie_aggregate(
    matA = matA,
    ngroups = 1,
    framework = "R0",
    criterion = "elasticity"
  )
  matF_agg <- as.matrix(res$matAk_agg)
  matU_agg <- as.matrix(0)
  agg_R0 <- spectral_radius(matF_agg %*% solve(diag(1) - matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)
})

#now test what happens when you give a 1x1 Leslie matrix as input
test_that("leslie_aggregate: test if 1x1 matrix works as input", {
  #get true results
  matA = matrix(.5, 1, 1)
  matF = matA
  matU = as.matrix(0, 1, 1)
  true_lambda <- spectral_radius(matA)
  true_R0 <- spectral_radius(matF %*% solve(diag(1) - matU))
  m <- 1
  n <- nrow(matA)
  ngroups <- m
  k <- n / m

  #compare with aggregated results
  #lambda framework
  res <- leslie_aggregate(
    matA = matA,
    ngroups = m,
    framework = "lambda",
    criterion = "standard"
  )
  expect_equal(c(res$matAk_agg), true_lambda^k, tolerance = 1.e-8)

  res <- leslie_aggregate(
    matA = matA,
    ngroups = m,
    framework = "lambda",
    criterion = "elasticity"
  )
  expect_equal(c(res$matAk_agg), true_lambda^k, tolerance = 1.e-8)

  #R0 framework
  res <- leslie_aggregate(
    matA = matA,
    ngroups = m,
    framework = "R0",
    criterion = "standard"
  )
  matF_agg <- res$matAk_agg
  matU_agg <- as.matrix(0)
  agg_R0 <- spectral_radius(matF_agg %*% solve(diag(1) - matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)

  res <- leslie_aggregate(
    matA = matA,
    ngroups = m,
    framework = "R0",
    criterion = "elasticity"
  )
  matF_agg <- res$matAk_agg
  matU_agg <- as.matrix(0)
  agg_R0 <- spectral_radius(matF_agg %*% solve(diag(1) - matU_agg))
  expect_equal(true_R0, agg_R0, tolerance = 1.e-8)
})

# tests/testthat/test-mpm_elasticity.R


test_that("mpm_elasticity: lambda framework input checks", {
  matA <- matrix(
    c(
      0.2, 0.0,
      0.3, 0.4
    ),
    nrow = 2, byrow = TRUE
  )

  # matA required
  expect_error(mpm_elasticity(matA = NULL, framework = "lambda"),
               "`matA` must be provided")

  # components must be NULL in lambda framework
  matF <- matrix(
    c(
      0, 1,
      0, 0
    ),
    nrow = 2, byrow = TRUE
  )

  matU <- matrix(
    c(
      0.2, 0.0,
      0.3, 0.4
    ),
    nrow = 2, byrow = TRUE
  )

  expect_error(mpm_elasticity(
    matA = matA,
    matF = matF,
    framework = "lambda"
  ),
  "only `matA` may be supplied")
  expect_error(mpm_elasticity(
    matA = matA,
    matU = matU,
    framework = "lambda"
  ),
  "only `matA` may be supplied")
  expect_error(mpm_elasticity(
    matA = matA,
    matC = matF,
    framework = "lambda"
  ),
  "only `matA` may be supplied")

  # matA must be a numeric matrix
  expect_error(mpm_elasticity(matA = list(1, 2), framework = "lambda"),
               "must be a numeric matrix")

  # matA must be square
  matA_ns <- matrix(1, nrow = 2, ncol = 3)
  expect_error(mpm_elasticity(matA = matA_ns, framework = "lambda"),
               "must be square")

  # matA must be nonnegative
  matA_neg <- matA
  matA_neg[1, 1] <- -0.01
  expect_error(mpm_elasticity(matA = matA_neg, framework = "lambda"),
               "must be nonnegative")

  # matA must not contain NA
  matA_na <- matA
  matA_na[1, 2] <- NA_real_
  expect_error(mpm_elasticity(matA = matA_na, framework = "lambda"), "contains NA")
})


test_that("mpm_elasticity: R0 framework input checks", {
  matF <- matrix(
    c(
      0.0, 1.2,
      0.0, 0.0
    ),
    nrow = 2, byrow = TRUE
  )

  matU <- matrix(
    c(
      0.2, 0.0,
      0.3, 0.4
    ),
    nrow = 2, byrow = TRUE
  )

  # matA must be NULL
  matA <- matF + matU
  expect_error(mpm_elasticity(
    matA = matA,
    matF = matF,
    matU = matU,
    framework = "R0"
  ),
  "`matA` must be NULL")

  # matF and matU required
  expect_error(
    mpm_elasticity(
      matF = NULL,
      matU = matU,
      framework = "R0"
    ),
    "`matF` and `matU` must be provided"
  )
  expect_error(
    mpm_elasticity(
      matF = matF,
      matU = NULL,
      framework = "R0"
    ),
    "`matF` and `matU` must be provided"
  )

  # must be numeric matrices
  expect_error(mpm_elasticity(
    matF = list(1, 2),
    matU = matU,
    framework = "R0"
  ),
  "must be a numeric matrix")

  # must be square
  matF_ns <- matrix(1, nrow = 2, ncol = 3)
  expect_error(mpm_elasticity(
    matF = matF_ns,
    matU = matU,
    framework = "R0"
  ),
  "must be square")

  # dims must match
  matU3 <- diag(3)
  expect_error(mpm_elasticity(
    matF = matF,
    matU = matU3,
    framework = "R0"
  ),
  "identical dimensionality")

  # matC dims must match if provided
  matC_bad <- diag(3)
  expect_error(
    mpm_elasticity(
      matF = matF,
      matU = matU,
      matC = matC_bad,
      framework = "R0"
    ),
    "identical dimensionality"
  )

  # nonnegativity
  matU_neg <- matU
  matU_neg[2, 2] <- -0.01
  expect_error(mpm_elasticity(
    matF = matF,
    matU = matU_neg,
    framework = "R0"
  ),
  "must be nonnegative")
})


test_that("mpm_elasticity: lambda framework returns expected structure and numeric matrix (mocked)",
          {
            matA <- matrix(
              c(
                0.2, 0.0,
                0.3, 0.4
              ),
              nrow = 2, byrow = TRUE
            )

            # Mock irreducibility checker (avoid dependence on graph structure in unit tests)
            local_mocked_bindings(
              .check_irreducible_hj = function(A, name = "matA", tol = 0)
                invisible(TRUE)
            )

            # Mock dominant_eigen to provide deterministic w, v, lambda
            w <- c(1, 2)
            v <- c(3, 4)
            lambda <- 5

            local_mocked_bindings(
              dominant_eigen = function(...)
                list(w = w, v = v, lambda = lambda)
            )

            out <- mpm_elasticity(matA = matA, framework = "lambda")

            expect_type(out, "list")
            expect_named(out, c("framework", "elasticity"))
            expect_equal(out$framework, "lambda")
            expect_true(is.matrix(out$elasticity))
            expect_equal(dim(out$elasticity), dim(matA))
            expect_true(all(is.finite(out$elasticity)))

            # Expected value from internal .elast: diag(v) %*% A %*% diag(w) / lambda
            expected <- diag(v) %*% matA %*% diag(w) / lambda
            expect_equal(out$elasticity, expected, tolerance = 1e-12)
          })


test_that("mpm_elasticity: R0 framework returns expected structure and numeric matrix (mocked)",
          {
            matF <- matrix(
              c(
                0.0, 1.2,
                0.0, 0.0
              ),
              nrow = 2, byrow = TRUE
            )

            matU <- matrix(
              c(
                0.2, 0.0,
                0.3, 0.4
              ),
              nrow = 2, byrow = TRUE
            )

            matR <- matF
            matA <- matR + matU

            # Mock irreducibility checker
            local_mocked_bindings(
              .check_irreducible_hj = function(A, name = "matA", tol = 0)
                invisible(TRUE)
            )

            # Mock spectral_radius and dominant_eigen (for Acohort)
            R0 <- 1.7
            w <- c(1, 2)
            v <- c(3, 4)

            local_mocked_bindings(
              spectral_radius = function(...)
                R0,
              dominant_eigen  = function(...)
                list(w = w, v = v, lambda = 999) # lambda unused in R0 branch
            )

            out <- mpm_elasticity(matF = matF,
                              matU = matU,
                              framework = "R0",
                              normalize=FALSE)

            expect_type(out, "list")
            expect_named(out, c("framework", "elasticity"))
            expect_equal(out$framework, "R0")
            expect_true(is.matrix(out$elasticity))
            expect_equal(dim(out$elasticity), dim(matA))
            expect_true(all(is.finite(out$elasticity)))

            Acohort <- matR + R0 * matU
            den <- as.numeric(t(v) %*% matR %*% w)
            expected <- diag(v) %*% Acohort %*% diag(w) / den

            expect_equal(out$elasticity, expected, tolerance = 1e-12)
          })


test_that("mpm_elasticity: R0 framework errors when (I - U) is singular", {
  # Make I - U singular: U = I
  matU <- diag(2)
  matF <- matrix(
    c(
      0, 1,
      0, 0
    ),
    nrow = 2, byrow = TRUE
  )

  # Mock irreducibility checker (we should fail after constructing matA but before spectral radius is needed)
  local_mocked_bindings(
    .check_irreducible_hj = function(A, name = "matA", tol = 0)
      invisible(TRUE),
    spectral_radius = function(...)
      1.0,
    dominant_eigen  = function(...)
      list(
        w = c(1, 1),
        v = c(1, 1),
        lambda = 1
      )
  )

  expect_error(mpm_elasticity(
    matF = matF,
    matU = matU,
    framework = "R0"
  ),
  "singular")
})


test_that("elasticity (lambda framework) matches numerical derivative", {
  ## small, irreducible, strictly positive matrix
  matA <- matrix(
    c(
      0.3, 0.1,
      0.4, 0.2
    ),
    nrow = 2, byrow = TRUE
  )

  ## compute analytic elasticity
  out <- mpm_elasticity(matA = matA, framework = "lambda")
  E_analytic <- out$elasticity

  ## baseline lambda
  lambda0 <- dominant_eigen(matA, ensure_positive = TRUE)$lambda

  ## numerical derivative settings
  eps <- 1e-6
  n <- nrow(matA)
  E_numeric <- matrix(NA_real_, n, n)

  ## finite-difference elasticity
  for (i in seq_len(n)) {
    for (j in seq_len(n)) {
      if (matA[i, j] == 0) {
        E_numeric[i, j] <- 0
      } else {
        matA_pert <- matA
        matA_pert[i, j] <- matA_pert[i, j] + eps

        lambda_pert <- dominant_eigen(matA_pert, ensure_positive = TRUE)$lambda

        d_lambda <- (lambda_pert - lambda0) / eps

        E_numeric[i, j] <- (matA[i, j] / lambda0) * d_lambda
      }
    }
  }

  ## compare analytic vs numerical elasticity
  expect_equal(E_analytic, E_numeric, tolerance = 1e-4)
})


test_that("elasticity (R0 framework) matches numerical derivative of R0 w.r.t. entries of matA",
          {
            # Construct matF (reproduction) and matU (transition) with disjoint nonzero supports.
            # This makes 'perturb entry of matA' unambiguous: it corresponds to exactly one component.
            matF <- matrix(
              c(
                0.0, 1.1, 0.4,
                0.0, 0.0, 0.0,
                0.0, 0.0, 0.0
              ),
              nrow = 3, byrow = TRUE
            )

            # NOTE: row 1, col 2 and col 3 are zero here to avoid overlap with matF
            matU <- matrix(
              c(
                0.15, 0.0, 0.0,
                0.30, 0.10, 0.25,
                0.05, 0.35, 0.10
              ),
              nrow = 3, byrow = TRUE
            )


            # matC optional; keep NULL here
            matC <- NULL

            # Analytic elasticity from your function
            out <- mpm_elasticity(matF = matF,
                              matU = matU,
                              framework = "R0",
                              normalize = FALSE)
            E_analytic <- out$elasticity

            # Define baseline A, R, and R0 (same construction used in your code)
            matR <- matF
            matA <- matR + matU

            n <- nrow(matA)
            I_minus_U <- diag(n) - matU

            # Baseline R0 from next-generation matrix
            K0 <- matR %*% solve(I_minus_U)
            R0_0 <- spectral_radius(K0)

            # Finite-difference settings
            eps <- 1e-6
            E_numeric <- matrix(0, n, n)

            # Helper to recompute R0 after perturbing a single entry in either matR or matU
            R0_from_parts <- function(R, U) {
              ImU <- diag(nrow(U)) - U
              K <- R %*% solve(ImU)
              spectral_radius(K)
            }

            for (i in seq_len(n)) {
              for (j in seq_len(n)) {
                # Elasticity at zero entries is defined as 0 because of the prefactor a_ij / R0.
                if (matA[i, j] == 0) {
                  E_numeric[i, j] <- 0
                  next
                }

                # Decide which component owns this A entry (disjoint supports assumed)
                in_R <- (matR[i, j] != 0)
                in_U <- (matU[i, j] != 0)

                # Sanity: with disjoint supports, exactly one should be TRUE for nonzero A entries
                expect_true(xor(in_R, in_U))

                # Central difference when possible (keeps better numerical accuracy)
                # For central diff on R entries, must keep R nonnegative; same for U entries.
                R_plus <- matR
                R_minus <- matR
                U_plus <- matU
                U_minus <- matU

                if (in_R) {
                  R_plus[i, j]  <- R_plus[i, j]  + eps
                  if (matR[i, j] > eps) {
                    R_minus[i, j] <- R_minus[i, j] - eps
                  } else {
                    R_minus <- NULL
                  }
                } else {
                  # in_U
                  U_plus[i, j]  <- U_plus[i, j]  + eps
                  if (matU[i, j] > eps) {
                    U_minus[i, j] <- U_minus[i, j] - eps
                  } else {
                    U_minus <- NULL
                  }
                }

                R0_plus <- R0_from_parts(R_plus, U_plus)

                if (is.null(R_minus) || is.null(U_minus)) {
                  # Forward difference fallback
                  dR0 <- (R0_plus - R0_0) / eps
                } else {
                  R0_minus <- R0_from_parts(R_minus, U_minus)
                  dR0 <- (R0_plus - R0_minus) / (2 * eps)
                }

                # Elasticity definition: e_ij = (a_ij / R0) * dR0/da_ij
                E_numeric[i, j] <- (matA[i, j] / R0_0) * dR0
              }
            }

            # Compare analytic vs numeric elasticities.
            # Tolerance is intentionally modest because eigenvalue-based R0 can be noisy under finite differences.
            expect_equal(E_analytic, E_numeric, tolerance = 1e-8)
          })

#' Fine-scale Asian elephant projection matrix
#'
#' A female-only projection matrix for an Asian elephant population,
#' included with the package for examples and vignettes. The matrix
#' has dimension 60 × 60 and a projection interval of 1 year.
#'
#' @details
#' Population/model ID: 249274 — a fine-grained model with a 1-year
#' projection interval for the Periyar Reserve elephant population
#' in India (Goswami et al., 2014).
#'
#' The matrix was originally retrieved from the COMADRE Animal Matrix
#' Database using the `Rcompadre` package and bundled with this package
#' so examples and vignettes can run without requiring internet access.
#' The final entry was zeroed out so that the matrix is a true Leslie
#' matrix.
#'
#' @format A numeric matrix with 60 rows and 60 columns.
#'
#' @source
#' COMADRE Animal Matrix Database, MatrixID 249274.
#'
#' @references
#' Goswami, V. R., Vasudev, D., & Oli, M. K. (2014).
#' The importance of conflict-induced mortality for conservation planning
#' in areas of human–elephant co-occurrence.
#' \emph{Biological Conservation}, 176, 191–198.
#' \doi{10.1016/j.biocon.2014.05.026}
"matA_elephant2"

#' Greatest common divisor of a set of integers
#'
#' Computes the greatest common divisor (GCD) of a numeric vector \code{x} of positive integers.
#' Used by \code{\link{LCM}}.
#'
#' @param x A numeric vector of positive integers.
#'
#' @return A positive integer giving the GCD of \code{x}.
#' @keywords internal
GCD <- function(x) {
  m <- min(x)
  while (any(x %% m > 0)) {
    m <- m - 1
  }
  return(m)
}

#' Least common multiple of two integers
#'
#' Computes the least common multiple (LCM) of two positive integers.
#' Used by \code{\link{leslie_disaggregate}}.
#'
#' @param a A positive integer.
#' @param b A positive integer.
#'
#' @return A positive integer giving the LCM of \code{a} and \code{b}.
#' @keywords internal
LCM <- function(a, b) {
  x <- c(a, b)
  gcd <- GCD(x)
  return((a * b) / gcd)
}

#' Disaggregate a Leslie matrix population model to a compatible dimensionality
#'
#' Expands an \eqn{n \times n} Leslie matrix \code{A} to an
#' \eqn{n_{new} \times n_{new}} Leslie matrix, where
#' \eqn{n_{new} = \mathrm{LCM}(n, m)}.
#'
#' Disaggregation is required when aggregating a Leslie model to dimensionality \code{m}
#' whenever \code{m} does not divide \code{n} evenly. The expanded matrix introduces a finer
#' age structure that is compatible with both dimensionalities.
#'
#' Each original age class is subdivided into
#' \eqn{n_{new} / n} sub-classes. Fertility rates from original age class \eqn{i}
#' are placed at the end of the corresponding block, deterministic aging within blocks
#' is represented by ones on the subdiagonal, and original survival probabilities are
#' inserted at block boundaries.
#'
#' @param A A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
#'   is treated as a 1x1 Leslie matrix.
#' @param m Target aggregated dimensionality (positive integer).
#'
#' @return A Leslie matrix of dimensionality \eqn{n_{new} \times n_{new}},
#'   where \eqn{n_{new} = \mathrm{LCM}(n, m)}.
#'
#' @references
#' Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application to
#' ten diverse animal species. \emph{Population Ecology}, 65(3), 146-166.
#' \doi{10.1002/1438-390X.12149}
#'
#' Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
#' application: Elasticity-consistent aggregation of Leslie matrix population
#' models for comparative demography. \emph{bioRxiv}, preprint.
#' \doi{10.64898/2026.02.04.703802}
#'
#' @examples
#' L <- matrix(c(
#'   0, 2, 1,
#'   0.6, 0, 0,
#'   0, 0.7, 0
#' ), nrow = 3, byrow = TRUE)
#' leslie_disaggregate(L, m = 2)
#'
#' @export
leslie_disaggregate <- function(A, m) {
  # Allow scalar 1x1 Leslie matrix
  if (is.null(dim(A))) {
    if (is.numeric(A) && length(A) == 1L && is.finite(A) && A > 0) {
      A <- matrix(A, nrow = 1, ncol = 1)
    }
  }

  if (!is_leslie(A))
    stop("leslie_disaggregate: Input matrix must be a Leslie matrix.", call. = FALSE)

  if (!is.numeric(m) || length(m) != 1L || !is.finite(m) || m < 1) {
    stop("m must be a positive integer.", call. = FALSE)
  }
  if (abs(m - round(m)) > 0)
    stop("m must be an integer.", call. = FALSE)
  m <- as.integer(m)

  n <- nrow(A)
  if (n == 1L)
    return(A)

  nnew <- as.integer(LCM(n, m))
  m1 <- as.integer(nnew / n)  # number of sub-ages per original age

  A_expanded <- matrix(0, nrow = nnew, ncol = nnew)

  # Fertility rates: place A[1, i] at column i*m1
  fert <- A[1, ]
  for (i in seq_len(n)) {
    A_expanded[1, i * m1] <- fert[i]
  }

  # Deterministic aging within the fine grid
  for (i in 2:nnew) {
    A_expanded[i, i - 1] <- 1
  }

  # Insert original survival probabilities at block boundaries
  surv <- A[cbind(2:n, 1:(n - 1))]
  for (i in seq_len(n - 1)) {
    A_expanded[i * m1 + 1, i * m1] <- surv[i]
  }

  return(A_expanded)
}

#' Generation time from a matrix population model
#'
#' Returns a generation time for either the \eqn{\lambda} or \eqn{R_0} framework.
#'
#' @param matF A square fecundity/sexual reproduction matrix (numeric matrix).
#'   Must not be \code{NULL}.
#' @param matU A square survival/growth/transition matrix (numeric matrix).
#'   Must not be \code{NULL}.
#' @param matC Optional square clonal reproduction matrix (numeric matrix),
#'   same dimensions as \code{matF} and \code{matU}. Default \code{NULL}.
#' @param framework Character scalar. Must be either \code{"lambda"} or \code{"R0"}.
#' @param tol Numeric tolerance used for nonnegativity checks and near-zero screening.
#'
#' @return A list with elements:
#' \itemize{
#'   \item \code{framework}: the chosen framework (\code{"lambda"} or \code{"R0"}).
#'   \item \code{generation_time}: generation time (numeric scalar).
#' }
#'
#' @details
#' The full projection matrix \code{matA} is constructed internally as
#' \code{matA = matU + matF (+ matC)} and is not accepted as an input because
#' \code{matA} alone is not sufficient to define the required decomposition.
#'
#' Eigen-elements (e.g., \eqn{\lambda}, \eqn{R_0}, stable stage distribution,
#' and reproductive values) are obtained via the function \code{\link{dominant_eigen}}.
#'
#' If \code{matC} is supplied, the reproductive matrix is defined as
#' \code{matR = matF + matC}. Otherwise, \code{matR = matF}.
#'
#' This function performs input validation, constructs \code{matA} and \code{matR},
#' and sets up the framework-specific branch for computing generation time.
#'
#'
#' @references
#' Bienvenu, F. & Legendre, S. (2015). A new approach to the generation time in
#' matrix population models. \emph{The American Naturalist}, 185(6), 834-843.
#' \doi{10.1086/681104}
#'
#' Ellner, S. P. (2018). Generation time in structured populations.
#' \emph{The American Naturalist}, 192(1), 105-110.
#' \doi{10.1086/697539}
#'
#' @examples

#' matU <- matrix(c(0.2, 0.0,
#'                 0.3, 0.4), nrow = 2, byrow = TRUE)
#' matF <- matrix(c(0.0, 1.2,
#'                 0.0, 0.0), nrow = 2, byrow = TRUE)
#' generation_time(matF = matF, matU = matU, framework = "lambda")
#' generation_time(matF = matF, matU = matU, framework = "R0")
#'
#' @seealso \code{\link{dominant_eigen}}, \code{\link{spectral_radius}}.
#'
#' @export
generation_time <- function(matF,
                            matU,
                            matC = NULL,
                            framework = c("lambda", "R0"),
                            tol = 1e-12) {
  framework <- match.arg(framework)

  .stop_if_not_matrix <- function(x, nm) {
    if (!is.matrix(x) || !is.numeric(x)) {
      stop(sprintf("`%s` must be a numeric matrix.", nm), call. = FALSE)
    }
  }

  .is_square <- function(x)
    nrow(x) == ncol(x)

  .stop_if_neg <- function(x, nm) {
    if (any(is.na(x)))
      stop(sprintf("`%s` contains NA values.", nm), call. = FALSE)
    if (any(x < -tol))
      stop(sprintf("`%s` must be nonnegative.", nm), call. = FALSE)
  }

  if (is.null(matF))
    stop("`matF` must not be NULL.", call. = FALSE)
  if (is.null(matU))
    stop("`matU` must not be NULL.", call. = FALSE)

  .stop_if_not_matrix(matF, "matF")
  .stop_if_not_matrix(matU, "matU")
  if (!is.null(matC))
    .stop_if_not_matrix(matC, "matC")

  if (!.is_square(matF) ||
      !.is_square(matU) || (!is.null(matC) && !.is_square(matC))) {
    stop("All supplied matrices must be square.", call. = FALSE)
  }

  dF <- dim(matF)
  if (!all(dim(matU) == dF)) {
    stop("`matF` and `matU` must have identical dimensions.", call. = FALSE)
  }
  if (!is.null(matC) && !all(dim(matC) == dF)) {
    stop("`matC` must have the same dimensions as `matF` and `matU`.",
         call. = FALSE)
  }

  .stop_if_neg(matF, "matF")
  .stop_if_neg(matU, "matU")
  if (!is.null(matC))
    .stop_if_neg(matC, "matC")

  matR <- if (is.null(matC))
    matF
  else
    (matF + matC)
  matA <- matU + matF + if (is.null(matC))
    0
  else
    matC

  .gen_time <- function(matR, w, v, sradius, tol) {
    den <- c(t(v) %*% matR %*% w)
    if (!is.finite(den) || abs(den) < tol) {
      stop(
        "Cannot compute generation time: denominator t(v) %*% matR %*% w is zero or non-finite.",
        call. = FALSE
      )
    }
    sradius / den
  }

  if (framework == "lambda") {
    ref <- dominant_eigen(matA, tol = tol, ensure_positive = TRUE)
  } else {
    n <- nrow(matA)
    I_minus_U <- diag(n) - matU
    qr_IU <- qr(I_minus_U)
    if (qr_IU$rank < n) {
      stop("Matrix (I - matU) is singular; cannot compute the next-generation matrix.",
           call. = FALSE)
    }
    K <- matR %*% solve(I_minus_U)
    R0 <- spectral_radius(K)
    Acohort <- matR + R0 * matU
    ref <- dominant_eigen(Acohort, tol = tol, ensure_positive = TRUE)
  }

  need <- c("w", "v", "lambda")
  if (!all(need %in% names(ref))) {
    stop(
      "`dominant_eigen()` must return a list with components `w`, `v`, and `lambda`.",
      call. = FALSE
    )
  }

  w <- ref$w
  v <- ref$v
  sradius <- ref$lambda #ref$lambda is actually R0 in the "R0" framework

  gt <- .gen_time(
    matR = matR,
    w = w,
    v = v,
    sradius = sradius,
    tol = tol
  )

  list(framework = framework, generation_time = gt)
}

#' Elasticity of \eqn{\lambda} or \ifelse{latex}{\eqn{R_0}}{R0} with respect to entries of \code{matA}
#'
#' Compute elasticities of either the dominant eigenvalue \eqn{\lambda} (\eqn{\lambda}
#' framework) or the net reproductive rate \eqn{R_0} (\eqn{R_0} framework) with respect
#' to the entries of the population projection matrix \code{matA}.
#'
#' The function enforces a strict separation of inputs by framework:
#' \itemize{
#'   \item \code{framework = "lambda"}: \code{matA} must be provided and
#'     \code{matF}, \code{matU}, and \code{matC} must be \code{NULL}.
#'   \item \code{framework = "R0"}: \code{matF} and \code{matU} must be provided,
#'     \code{matC} is optional, and \code{matA} must be \code{NULL}. The projection
#'     matrix is constructed internally as \code{matA = matR + matU}, where
#'     \code{matR = matF (+ matC)}.
#' }
#'
#' Irreducibility of the resulting \code{matA} is enforced using the internal helper
#' \code{.check_irreducible_hj} (defined elsewhere in the package).
#'
#' Eigen-elements (e.g., dominant eigenvalue, left and right eigenvectors) are
#' obtained via the function \code{\link{dominant_eigen}}.
#'
#' @param matA A square projection matrix (numeric matrix). Required when
#'   \code{framework = "lambda"}. Must be \code{NULL} when \code{framework = "R0"}.
#' @param matF A square fecundity/sexual reproduction matrix (numeric matrix).
#'   Required when \code{framework = "R0"}. Must be \code{NULL} when
#'   \code{framework = "lambda"}.
#' @param matU A square survival/growth/transition matrix (numeric matrix).
#'   Required when \code{framework = "R0"}. Must be \code{NULL} when
#'   \code{framework = "lambda"}.
#' @param matC Optional square clonal reproduction matrix (numeric matrix).
#'   Only allowed when \code{framework = "R0"}.
#' @param framework Character scalar. Must be either \code{"lambda"} or \code{"R0"}.
#' @param normalize Logical. Must be either \code{TRUE} or \code{FALSE}. This is
#'   relevant only when \code{framework = "R0"}. When true, the elasticity of
#'   \code{"R0"} with respect to the entries of \code{"matA"} are scaled to
#'   sum to 1.
#' @param tol Numeric tolerance used for nonnegativity checks. Default \code{1e-12}.
#'
#' @return A list with elements:
#' \itemize{
#'   \item \code{framework}: the chosen framework.
#'   \item \code{elasticity}: a matrix of elasticities with the same dimensionality as
#'     \code{matA}.
#' }
#'
#' @details
#' Lambda framework: Elasticity is the elasticity of \eqn{\lambda} with respect
#' to the entries of \code{matA}.
#'
#' R0 framework: Elasticity is the elasticity of \eqn{R_0} with respect
#' to the entries of the internally constructed \code{matA}.
#'
#' @examples
#' ## Lambda framework: matA provided directly
#' matA <- matrix(
#'   c(0.2, 1.2,
#'     0.3, 0.4),
#'   nrow = 2, byrow = TRUE
#' )
#' out_lambda <- mpm_elasticity(matA = matA, framework = "lambda")
#' str(out_lambda)
#'
#' ## R0 framework: matA constructed from matF and matU
#' matU <- matrix(
#'   c(0.2, 0.0,
#'     0.3, 0.4),
#'   nrow = 2, byrow = TRUE
#' )
#' matF <- matrix(
#'   c(0.0, 1.2,
#'     0.0, 0.0),
#'   nrow = 2, byrow = TRUE
#' )
#' out_R0 <- mpm_elasticity(matF = matF, matU = matU, framework = "R0")
#' str(out_R0)
#'
#' @references
#' Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
#' interpretation} (2nd ed.). Sinauer.
#'
#' @seealso \code{\link{dominant_eigen}}, \code{\link{spectral_radius}}
#'
#' @export
mpm_elasticity <- function(matA = NULL,
                       matF = NULL,
                       matU = NULL,
                       matC = NULL,
                       framework = c("lambda", "R0"),
                       normalize = TRUE,
                       tol = 1e-12) {
  framework <- match.arg(framework)

  .stop_if_not_matrix <- function(x, nm) {
    if (!is.matrix(x) || !is.numeric(x)) {
      stop(sprintf("`%s` must be a numeric matrix.", nm), call. = FALSE)
    }
  }

  .is_square <- function(x)
    nrow(x) == ncol(x)

  .stop_if_neg <- function(x, nm) {
    if (any(is.na(x))) {
      stop(sprintf("`%s` contains NA values.", nm), call. = FALSE)
    }
    if (any(x < -tol)) {
      stop(sprintf("`%s` must be nonnegative.", nm), call. = FALSE)
    }
  }

  # ---- Validate inputs and construct matA (and matR for R0) ----
  if (framework == "lambda") {
    if (is.null(matA)) {
      stop("`matA` must be provided when `framework = \"lambda\"`.",
           call. = FALSE)
    }
    if (!is.null(matF) || !is.null(matU) || !is.null(matC)) {
      stop(
        "When `framework = \"lambda\"`, only `matA` may be supplied; `matF`, `matU`, and `matC` must be NULL.",
        call. = FALSE
      )
    }

    .stop_if_not_matrix(matA, "matA")
    if (!.is_square(matA))
      stop("`matA` must be square.", call. = FALSE)
    .stop_if_neg(matA, "matA")

  } else {
    # framework == "R0"

    if (!is.null(matA)) {
      stop(
        "When `framework = \"R0\"`, `matA` must be NULL and will be constructed internally from `matF`, `matU`, and optional `matC`.",
        call. = FALSE
      )
    }
    if (is.null(matF) || is.null(matU)) {
      stop("`matF` and `matU` must be provided when `framework = \"R0\"`.",
           call. = FALSE)
    }

    .stop_if_not_matrix(matF, "matF")
    .stop_if_not_matrix(matU, "matU")
    if (!is.null(matC))
      .stop_if_not_matrix(matC, "matC")

    if (!.is_square(matF) || !.is_square(matU) ||
        (!is.null(matC) && !.is_square(matC))) {
      stop("All supplied matrices must be square.", call. = FALSE)
    }

    d <- dim(matF)
    if (!all(dim(matU) == d) ||
        (!is.null(matC) && !all(dim(matC) == d))) {
      stop("`matF`, `matU`, and `matC` (if provided) must have identical dimensionality.",
           call. = FALSE)
    }

    .stop_if_neg(matF, "matF")
    .stop_if_neg(matU, "matU")
    if (!is.null(matC))
      .stop_if_neg(matC, "matC")

    matR <- if (is.null(matC))
      matF
    else
      (matF + matC)
    matA <- matR + matU
  }

  # ---- irreducibility check (after matA is finalized) ----
  .check_irreducible_hj(matA, name = "matA", tol = tol)

  .elast <- function(A, w, v, den) {
    if (!is.finite(den) || abs(den) < tol) {
      stop("Cannot compute elasticity: denominator is zero or non-finite.",
           call. = FALSE)
    }
    w <- as.numeric(w)
    v <- as.numeric(v)
    W <- diag(w)
    V <- diag(v)
    V %*% A %*% W / den
  }

  # ---- framework-specific computations ----
  if (framework == "lambda") {
    # lambda-framework elasticity of lambda w.r.t. entries of matA
    ref <- dominant_eigen(matA, tol = tol, ensure_positive = TRUE)
    den <- ref$lambda
    E <- .elast(
      A = matA,
      w = ref$w,
      v = ref$v,
      den = den
    )

  } else {
    # R0-framework elasticity of R0 w.r.t. entries of matA
    n <- nrow(matA)
    I_minus_U <- diag(n) - matU
    qr_IU <- qr(I_minus_U)
    if (qr_IU$rank < n) {
      stop("Matrix (I - matU) is singular; cannot compute the next-generation matrix.",
           call. = FALSE)
    }

    K <- matR %*% solve(I_minus_U)
    R0 <- spectral_radius(K)
    Acohort <- matR + R0 * matU
    ref <- dominant_eigen(Acohort, tol = tol, ensure_positive = TRUE)
    den <- c(t(ref$v) %*% matR %*% ref$w)
    E <- .elast(
      A = Acohort,
      w = ref$w,
      v = ref$v,
      den = den
    )
    if (normalize) {
      sumE <- sum(E)
      if (sumE < tol) {
        stop("Elasticity matrix of R0 sums to 0 and cannot be normalized.",
             call. = FALSE) #stop
      } #if sumE
      E <- E / sum(E)
    }#if normalize
  } #else (framework = "R0")

  list(framework = framework, elasticity = E)
}

#' Test whether a matrix is a Leslie matrix
#'
#' Checks whether a matrix \code{A} has Leslie form: nonzero entries are allowed only in the first row
#' (fertility rates) and on the subdiagonal (survival probabilities). Survival probabilities
#' must be in \eqn{(0, 1]} (within tolerance), fertility rates must be nonnegative, and
#' at least one fertility rate must be positive.
#'
#' A 1x1 matrix with a positive entry is treated as a valid (degenerate) Leslie matrix.
#'
#' @param A A numeric square matrix. A length-1 positive numeric will be coerced to a 1x1 matrix.
#' @param tol Numeric tolerance used for structural comparisons and bounds checks.
#'
#' @return Logical; \code{TRUE} if \code{A} is a Leslie matrix, otherwise \code{FALSE}.
#'
#' @examples
#' L <- matrix(c(
#'   0, 2, 1,
#'   0.6, 0, 0,
#'   0, 0.7, 0
#' ), nrow = 3, byrow = TRUE)
#' is_leslie(L)
#'
#' @export
is_leslie <- function(A, tol = sqrt(.Machine$double.eps)) {
  # Allow a single positive number as a 1x1 Leslie matrix
  if (is.null(dim(A))) {
    if (is.numeric(A) && length(A) == 1L && is.finite(A) && A > 0) {
      A <- matrix(A, nrow = 1, ncol = 1)
    } else {
      return(FALSE)
    }
  }

  if (!is.matrix(A) || !is.numeric(A))
    return(FALSE)
  if (nrow(A) != ncol(A))
    return(FALSE)
  if (any(!is.finite(A)))
    return(FALSE)

  n <- nrow(A)

  # 1x1 case: positive entry => Leslie
  if (n == 1L)
    return(A[1, 1] > 0)

  fert <- A[1, ]
  # Subdiagonal survival probability entries
  surv <- A[cbind(2:n, 1:(n - 1))]

  # Fertility rates must be nonnegative and not all zero
  if (any(fert < -tol))
    return(FALSE)
  if (!(sum(fert) > tol))
    return(FALSE)

  # Survival probability must be in (0, 1] (within tolerance)
  # Require strictly positive (not just nonnegative) for Leslie aging
  if (any(surv <= tol))
    return(FALSE)
  if (any(surv > 1 + tol))
    return(FALSE)

  # Structural check: entries not in first row or subdiagonal must be ~0
  mask_allowed <- matrix(FALSE, n, n)
  mask_allowed[1, ] <- TRUE
  mask_allowed[cbind(2:n, 1:(n - 1))] <- TRUE

  off <- A[!mask_allowed]
  if (any(abs(off) > tol))
    return(FALSE)

  return(TRUE)
}

#' Coarse-scale Asian elephant projection matrix
#'
#' A female-only projection matrix for an Asian elephant population,
#' included with the package for examples and vignettes. The matrix
#' has dimension 12 × 12 and a projection interval of 5 years.
#'
#' @details
#' Population/model ID: 249273 — a coarser model with a 5-year projection
#' interval for the Nagarhole National Park elephant population in India
#' (Chelliah et al., 2013).
#'
#' The matrix was originally retrieved from the COMADRE Animal Matrix
#' Database using the `Rcompadre` package and bundled with this package
#' so examples and vignettes can run without requiring internet access.
#'
#' @format A numeric matrix with 12 rows and 12 columns.
#'
#' @source
#' COMADRE Animal Matrix Database, MatrixID 249273.
#'
#' @references
#' Chelliah, K., Bukka, H., & Sukumar, R. (2013).
#' Modeling harvest rates and numbers from age and sex ratios:
#' a demonstration for elephant populations.
#' \emph{Biological Conservation}, 165, 54–61.
#' \doi{10.1016/j.biocon.2013.05.008}
"matA_elephant1"

testthat::test_that("No exported-name collisions on CRAN", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("collidr")

  suppressPackageStartupMessages(library(collidr))

  res <- CRAN_package_collisions("mpmaggregate")
  testthat::expect_length(res$packages, 0)
})

---
title: "Aggregating Asian Elephant Matrix Population Models for Comparative Demography with mpmaggregate"
author:
  - name: "Richard A. Hinrichsen"
    orcid: "0000-0003-0761-3005"
output:
  rmarkdown::html_vignette:
    number_sections: false
    mathjax: default
vignette: >
  %\VignetteIndexEntry{Aggregating Asian Elephant Matrix Population Models for Comparative Demography with mpmaggregate}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
**ORCID:** <https://orcid.org/0000-0003-0761-3005>
```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```


## Overview

Age-structured matrix population models (MPMs) can differ in age-class width
and projection interval, even when describing the same species. This creates 
a practical problem in comparative demography: how can models that represent 
life cycles at different temporal resolution be compared?

In this vignette, we demonstrate how to use the package **`mpmaggregate`** to aggregate a 
fine-grained, annual (1-year) Asian elephant Leslie MPM into a coarser, 5-year 
Leslie MPM, and then compare the aggregated model to an independently parameterized coarser 
Leslie MPM that represents a different population. Because age classes in a Leslie model 
are defined by the model’s time step, aggregating from 1-year to 5-year intervals 
necessarily redefines the age-class structure (i.e., age classes become 5-year bins) 
while producing transitions appropriate to the longer projection interval.

We illustrate this approach using two Asian elephant populations from
**COMADRE** database (Salguero‐Gómez et al. 2016) accessed via **`Rcompadre`** (Jones et al. 2022):

- Population/model ID: **249273** — a coarser model with a 5-year projection interval, 
Nagarhole National Park population, India (Chelliah et al. 2013)  
- Population/model ID: **249274** — a fine-grained model with a 1-year projection interval, 
Periyar Reserve population, India (Goswami et al. 2014)

Both MPMs are Leslie matrix models parameterized using a post-breeding census formulation. 
Our focus is the aggregation of the Leslie MPM for the Priiyar Reserve population
with an annual projection interval to a Leslie MPM with a 5-year projection interval. 
To do this, we use `leslie_aggregate()`, which performs Leslie-to-Leslie MPM aggregation 
under four alternative assumptions defined by the choice of demographic **framework** 
(`"lambda"` or `"R0"`) and **criterion** (`"standard"` or `"elasticity"`):

1. `framework = "lambda"`, `criterion = "standard"`  
2. `framework = "lambda"`, `criterion = "elasticity"`  
3. `framework = "R0"`, `criterion = "standard"`  
4. `framework = "R0"`, `criterion = "elasticity"`

`criterion = "elasticity"` implies elasticity-consitent aggregation (Hinrichsen et al., 2026).

Throughout, we seek to uncover the substantive demographic differences between the two
Asian elephant populations, using $\lambda$, $R_0$, generation times, and elasticities.  We also 
examine at how results differ depending on framework 
(`"lambda"` or `"R0"`) and criterion  (`"standard"` or `"elasticity"`).

## Loading packages

We begin by loading the `mpmaggregate` package, which provides the functions used 
throughout this vignette for aggregating matrix population models under the \(\lambda\) 
and \(R_0\) demographic frameworks. The package `knitr` is used for report 
generation, and `kableExtra` is used for table formatting.

```{r packages}
library(mpmaggregate)
library(kableExtra)
library(rphylopic)
```
## Data acquisition (COMADRE via Rcompadre)

The Asian Elephant data is stored in the data directory.  Retrieve data using data() command. 

```{r fetch-comadre}

# Matrices for Asian elephant populations included in this package were
# originally retrieved from COMADRE using the Rcompadre package.

# Not run: requires Rcompadre and internet access

# library(Rcompadre)
# comadre <- cdb_fetch("comadre",version = "4.23.3.1")
# mpm_elephant1 <- comadre[comadre$MatrixID == 249273, ]
# matA_elephant1 <- matA(mpm_elephant1)[[1]]             # Projection interval is 5 years
# matA_elephant1 <- matA_elephant1[1:12,1:12]            # Confine to females 
# mpm_elephant2 <- comadre[comadre$MatrixID == 249274, ]
# matA_elephant2 <- matA(mpm_elephant2)[[1]]             # Projection interval is 1 year
# matA_elephant2[60, 60] <- 0                            # Remove statis to form a true Leslie Matrix
# usethis::use_data(matA_elephant1, matA_elephant2,      # Write data (development)
# overwrite = TRUE)    

# Load data locally
data(matA_elephant1, matA_elephant2, package="mpmaggregate")
matA_coarse <- matA_elephant1 # 12x12 5-year projection interval
matA_fine <- matA_elephant2   # 60x60 1-year projection interval
```
## Define the aggregation target

We aggregate the Periyar Reserve Asian elephant MPM from an annual time step to a 5-year time step, allowing
us to directly compare its vital rates to those of the Nagarhole National Park Asian elephant MPM.

Rigorous methods for Leslie-to-Leslie MPM aggregation are implemented using the function 
`leslie_aggregate()`, which provides four aggregation options. If the goal is to preserve 
the asymptotic population growth rate, one uses `framework = "lambda"`. Alternatively, 
if the goal is to preserve the net reproductive rate, one uses `framework = "R0"`.

Within either framework, aggregation is performed using a standard or an 
elasticity-consistent criterion. The standard aggregator ensures consistency of the stable age 
distribution implied by the model, while the elasticity-consistent aggregator additionally 
ensures consistency of reproductive values. In the `R0` framework, the stable age distribution 
is the cohort stable age distribution, and reproductive values are the cohort reproductive values.

We now run `leslie_aggregate()` under the four combinations.

## Run aggregation under four option sets
```{r leslie-aggregation}

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "lambda",
  criterion = "standard"
)
matB_lambda_stand <- res$matAk_agg
matB_lambda_stand_eff <- res$effectiveness

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "lambda",
  criterion = "elasticity"
)
matB_lambda_elast <- res$matAk_agg
matB_lambda_elast_eff <- res$effectiveness

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "R0",
  criterion = "standard"
)
matB_R0_stand <- res$matAk_agg
matB_R0_stand_eff<- res$effectiveness

res <- leslie_aggregate(
  matA_fine,
  ngroups = 12,
  framework = "R0",
  criterion = "elasticity"
)
matB_R0_elast <- res$matAk_agg
matB_R0_elast_eff <- res$effectiveness
```
## Comparison to an existing coarse (5-year) matrix

The COMADRE matrix for Asian elephants in the Nagarhole National Park already has 
`ProjectionInterval == 5`, so we compare the aggregated 5-year matrices 
for Asian elephants in the Periyar Reserve against it.

## Demographic metrics for Leslie matrices 

For Leslie matrices, fertility rates occupy the first row, while survival probabilities 
occupy the subdiagonal. This structure allows direct decomposition of the projection 
matrix \( \mathbf{A} \) into survival probability \( \mathbf{U} \)and fertility 
rate \( \mathbf{F} \)components.

```{r leslie-metrics}

# Decompose a Leslie matrix into its components U and F
leslie_UF <- function(A) {
  F <- matrix(0, nrow(A), ncol(A))
  F[1, ] <- A[1, ]
  U <- A
  U[1, ] <- 0
  list(U = U, F = F)
}

# Return R0, the net reproductive rate
get_R0_leslie <- function(A) {
  uf <- leslie_UF(A)
  U <- uf$U
  F <- uf$F
  
  I <- diag(nrow(U))
  N <- solve(I - U)
  K <- F %*% N
  spectral_radius(K)
}

# Return generation time in the lambda framework
get_generation_time_lambda <- function(A) {
  uf <- leslie_UF(A)
  U <- uf$U
  F <- uf$F
  I <- diag(nrow(U))
  N <- solve(I - U)
  gt = generation_time(F, U, framework = "lambda")$generation_time
  return(gt)
}

# Return cohort generation time
get_generation_time_R0 <- function(A) {
  uf <- leslie_UF(A)
  U <- uf$U
  F <- uf$F
  I <- diag(nrow(U))
  N <- solve(I - U)
  gt = generation_time(F, U, framework = "R0")$generation_time
  return(gt)
}

```

## Comparison of λ and \(R_0\) across five models. First develop a list of models with associated projection matrices

```{r define-models}
models <- list(
  "Orig. coarse"             = matA_coarse,
  "Agg λ + standard"         = matB_lambda_stand,
  "Agg λ + elasticity"       = matB_lambda_elast,
  "Agg R0 + standard"        = matB_R0_stand,
  "Agg R0 + elasticity"      = matB_R0_elast
)
```

## Next table the results for λ and \(R_0\)

```{r lambda - r0 - table}

#Form the data frame that contains the table information
results <- data.frame(
  Model  = names(models),
  lambda = sapply(models, spectral_radius),
  R0     = sapply(models, get_R0_leslie),
  Ta     = sapply(models, get_generation_time_lambda)*5,  #measure in years
  Tc     = sapply(models, get_generation_time_R0)*5,      #measure in years
  Eff    = c(NA,matB_lambda_stand_eff,matB_lambda_elast_eff,
             matB_R0_stand_eff,matB_R0_elast_eff),
  row.names = NULL,
  stringsAsFactors = FALSE
)

n_rows <- nrow(results)

#Construct a table using function kable
knitr::kable(
  results,
  col.names = c(
    "Model",
    "<em>&lambda;</em>",
    "<em>R</em><sub>0</sub>",
    "<em>T</em><sub>a</sub> (years)",
    "<em>T</em><sub>c</sub> (years)",
    "<em>&rho;</em><sup>2</sup>"
  ),
  digits = 3,
  caption = paste0(
  "<strong>Table 1.</strong> Demographic comparison between two Asian elephant populations, ",
  "represented by an original coarse-stage Leslie model from Nagarhole National Park ",
  "(MatrixID 249273) and aggregated versions of a Leslie model from Periyar Reserve ",
  "(MatrixID 249274). Metrics include population growth rate (<em>&lambda;</em>), ",
  "net reproductive rate (<em>R</em><sub>0</sub>), generation time (years; ",
  "<em>T</em><sub>a</sub>), cohort generation time (years; ",
  "<em>T</em><sub>c</sub>), and aggregation effectiveness (<em>&rho;</em><sup>2</sup>). ",
  "&ldquo;Standard&rdquo; denotes use of a standard aggregator and ",
  "&ldquo;elasticity&rdquo; denotes elasticity-consistent aggregation."
  ),
  format = "html",
  escape = FALSE
) |>
  kable_styling(full_width = TRUE) |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  row_spec(nrow(results), extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> The projection interval associated with the population ",
      "growth rate (<em>&lambda;</em>) is 5 years.</span>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )
```


**How to read Table 1.** Across aggregation approaches, estimates of \(\lambda\) and \(R_0\) 
for the Periyar Reserve population are similar and indicate faster population growth 
and shorter generation time relative to the Nagarhole National Park population. Each row 
corresponds to a Leslie matrix population model: the original coarse model from Nagarhole 
National Park (COMADRE) and four aggregated versions derived from the annual Periyar Reserve 
model. Differences among the aggregated models reflect the aggregation objective 
(preserving \(\lambda\) versus preserving \(R_0\)) and the fitting criterion 
(standard versus elasticity-consistent). Values of aggregation effectiveness, 
\(\rho^2\), indicate good agreement with the original model for the Periyar Reserve 
population.

### Interpretation notes

- Aggregation under the \(\lambda\) framework prioritizes preservation of long-term population growth.
- Aggregation under the \(R_0\) framework emphasizes lifetime reproductive output.

## Plot figures
**Figure 1. Vital rates across five Leslie models.**
Panel (a) shows fertility rates (first-row entries of the Leslie matrix, \(F_i\)).
Panel (b) shows survival probabilities (subdiagonal entries, \(S_i = A_{i+1,i}\)).
Bars are grouped by vital rate; within each group, colors correspond to the five 
different models (original coarse model for the Nagarhole National Park population  
and four 5-year aggregations of the model for the Periyar Reserve population). 
Only vital rates that are nonzero in at least one model are shown. The models in 
the Figure Legend are as described in the caption of Table 1.


```{r vital-rates-barplot, fig.height=8, fig.width=10}

#Asian elephant
#"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

# Not run: requires internet access

# uuid <- "7c9ab182-175d-4f02-96d0-09c1e5212bff"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 691)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/elephant.png")

#Load the image file that is stored locally 
img_file <- system.file("extdata", "elephant.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/elephant.png"
}

img <- png::readPNG(img_file)
# ---- Helpers to extract Leslie vital rates from A ----
extract_fertility <- function(A) {
  as.numeric(A[1, ])
}

extract_survival <- function(A) {
  n <- nrow(A)
  s_sub  <- A[cbind(2:n, 1:(n - 1))]  # A[i+1, i]
  c(s_sub)
}

model_names <- names(models)

# Choose distinct colors for the 5 models (base R friendly)
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Build rate-by-model matrices so:
# rows = models (5 bars), cols = vital rates (groupings)
F_by_model <- t(sapply(models, extract_fertility))  # 5 x n
S_by_model <- t(sapply(models, extract_survival))   # 5 x n-1

# Name columns (vital rates)
n <- nrow(models[[1]])
colnames(F_by_model) <- paste0("F", 1:n)
colnames(S_by_model) <- c(paste0("S", 1:(n - 1)))

# Keep only vital rates that are nonzero in at least one model
keep_F <- colSums(abs(F_by_model) > 0) > 0
keep_S <- colSums(abs(S_by_model) > 0) > 0

F_plot <- F_by_model[, keep_F, drop = FALSE]
S_plot <- S_by_model[, keep_S, drop = FALSE]

# Leave extra space on the right for a legend that won't overlap bars
op <- par(
  mfrow = c(2, 1),
  mar = c(7, 4, 3, 7),
  oma = c(0, 0, 0, 0)
)

op <- par(
  mfrow = c(2, 1),
  mar = c(5, 4, 1.5, 7),
  oma = c(0, 0, 0, 0)
)

ylim = c(0, max(F_plot))
mydiff = ylim[2] - ylim[1]
ylim[2] = ylim[2] + .3 * mydiff

# Panel (a): Fertility rates
barplot(
  F_plot,
  ylim = ylim,
  beside = TRUE,
  col = cols,
  border = "white",
  las = 2,
  ylab = "Fertility rate",
  main = "Fertility rates"
)
mtext("(a)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .9,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

# One legend for both panels, placed in the right margin of the top panel
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.18, 0.00),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)
ylim <-  c(0, max(S_plot))
mydiff <- ylim[2] - ylim[1]
ylim[2] <- ylim[2] + .3 * mydiff

# Panel (b): Survival probs
barplot(
  S_plot,
  ylim = ylim,
  beside = TRUE,
  col = cols,
  border = "white",
  las = 2,
  ylab = "Survival probability",
  main = "Survival probabilities"
)
mtext("(b)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .9,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

# Legend for bottom panel (same placement logic)
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.18, 0.00),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

par(op)
```
**How to read Figure 1.** Each x-axis label is a vital rate. Fertility rate, \(F_i\) 
is the contribution of age class \(i\) to newborns over the projection interval. 
Survival probability, \(S_i\) is the probability of surviving and advancing from age class \(i\) 
to \(i+1\) over the projection interval. Because the four aggregated matrices are 
all 5-year models, differences among them reflect the aggregation objective
(\(\lambda\) vs \(R_0\)) and the fitting criterion (standard vs elasticity-consistent), 
rather than differences in projection interval. The figure shows that the Nagarhole 
National Park population has lower population growth rate than the Periyar Reserve population 
due to lower fertility rates.The silhouette is from PhyloPic 
(https://www.phylopic.org; Keesey, 2023) and were added using the rphylopic R 
package (Gearty & Jones, 2023). The silhouette was contributed as follows: 
"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

**Figure 2. Elasticities of \(\lambda\) with respect to matrix entries.**
Elasticities quantify the proportional effect on \(\lambda\) of proportional changes 
in each vital rate. Panel (a) shows elasticities with respect to fertility rates 
(first row), and panel (b) shows elasticities with respect to survival probabilities 
(subdiagonal). Bars are grouped by vital rate, with colors indicating the five
different models (original coarse model for the Nagarhole National Park population  
and four 5-year aggregations of the model for the Periyar Reserve population). 
Elasticities sum to 1 within each matrix, so the panels show how the influence 
on \(\lambda\) is distributed across life-history components. The models in the 
figure legend are as described in the caption of Table 1.


```{r lambda-elasticities-barplot, fig.height=8, fig.width=10}
#Asian elephant
#"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

# Not run: requires internet access

# uuid <- "7c9ab182-175d-4f02-96d0-09c1e5212bff"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 691)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/elephant.png")

#Load a local copy of the image file
img_file <- system.file("extdata", "elephant.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/elephant.png"
}

img <- png::readPNG(img_file)

# Compute elasticity matrices for each model (same dimensions as A)
E_models <- lapply(models, function(A) {
  E <- mpm_elasticity(A, framework = "lambda")$elasticity
  as.matrix(E)
})

# ---- Helpers to extract Leslie elasticities from an elasticity matrix ----
extract_fertility_elast <- function(E) {
  as.numeric(E[1, ])
}

extract_survival_elast <- function(E) {
  n <- nrow(E)
  e_sub  <- E[cbind(2:n, 1:(n - 1))]  # elasticity of A[i+1, i]
  as.numeric(e_sub)
}

model_names <- names(models)

# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Build elasticity-by-model matrices so:
# rows = models (5 bars), cols = vital rates (groupings)
F_el_by_model <- t(sapply(E_models, extract_fertility_elast))  # 5 x n
S_el_by_model <- t(sapply(E_models, extract_survival_elast))   # 5 x n-1

# Name columns (vital rates)
n <- nrow(models[[1]])
colnames(F_el_by_model) <- paste0("F", 1:n)
colnames(S_el_by_model) <- c(paste0("S", 1:(n - 1)))

# Keep only elasticities that are nonzero in at least one model
keep_F <- colSums(abs(F_el_by_model) > 0) > 0
keep_S <- colSums(abs(S_el_by_model) > 0) > 0

F_el_plot <- F_el_by_model[, keep_F, drop = FALSE]
S_el_plot <- S_el_by_model[, keep_S, drop = FALSE]

# Leave extra space on the right for legends that won't overlap bars
op <- par(
  mfrow = c(2, 1),
  mar = c(7, 4, 3, 7),
  oma = c(0, 0, 0, 0)
)

op <- par(
  mfrow = c(2, 1),
  mar = c(5, 4, 1.5, 7),
  oma = c(0, 0, 0, 0)
)

# Panel (a): Elasticities with respect to fertility rates
barplot(
  F_el_plot,
  beside = TRUE,
  ylim = c(0, 0.20),
  col = cols,
  border = "white",
  las = 2,
  ylab = "Elasticity of λ",
  main = "Elasticities with respect to fertility rates"
)
mtext("(a)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

# Panel (b): Elasticities with respect to survival probabilities
barplot(
  S_el_plot,
  beside = TRUE,
  ylim = c(0, .20),
  col = cols,
  border = "white",
  las = 2,
  ylab = "Elasticity of λ",
  main = "Elasticities with respect to survival probabilities"
)
mtext("(b)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

par(op)
```
**How to read Figure 2.** A larger elasticity (of lambda with respect to matrix 
entries) for a given vital rate means that small proportional changes to that vital 
rate would have a comparatively larger effect on long-term population growth. For 
Asian elephants, elasticities are  concentrated in survival probabilities rather 
than fertility rates, reflecting the strong influence of survivorship on \(\lambda\). 
Aggregation methods yield similar elasticities.  Fertility rates and late-life 
survival probabilities have higher elasticities in the Nagarhole National Park 
population than in the Periyar Reserve population.The silhouette is from PhyloPic 
(https://www.phylopic.org; Keesey, 2023) and were added using the rphylopic R 
package (Gearty & Jones, 2023). The silhouette was contributed as follows: 
"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

**Figure 3. Elasticities of \(R_0\) with respect to matrix entries.**
Elasticities quantify the proportional effect on \(R_0\) of proportional changes 
in each vital rate. Panel (a) shows elasticities with respect to fertility rates 
(first row), and panel (b) shows elasticities with respect to survival probabilities 
(subdiagonal). Bars are grouped by vital rate, with colors indicating the five
different models (original coarse model for the Nagarhole National Park population  
and four 5-year aggregations of the model for the Periyar Reserve population).
Elasticities are normalized to sum to 1 within each matrix, so the panels 
show how the influence on \(R_0\) is distributed across life-history components. 
The models in the figure legend are as described in caption of Table 1.


```{r R0-elasticities-barplot, fig.height=8, fig.width=10}

#Asian elephant
#"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

# Not run: requires internet access

# uuid <- "7c9ab182-175d-4f02-96d0-09c1e5212bff"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 691)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/elephant.png")


img_file <- system.file("extdata", "elephant.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/elephant.png"
}

img <- png::readPNG(img_file)

# Compute elasticity matrices for each model (same dimensions as A)
# These are elasticities of R0 wrt to matrix entries
# normalized to sum to 1.
E_models <- lapply(models, function(A) {
  n <- nrow(A)
  matU <- A
  matU[1,] <- 0
  matF <- matrix(0,n,n)
  matF[1,] <- A[1,]
  E <- mpm_elasticity(matA=NULL, matF = matF, matU = matU, 
                  framework = "R0", normalize = TRUE)$elasticity
  as.matrix(E)
})

# ---- Helpers to extract Leslie elasticities from an elasticity matrix ----
extract_fertility_elast <- function(E) {
  as.numeric(E[1, ])
}

extract_survival_elast <- function(E) {
  n <- nrow(E)
  e_sub  <- E[cbind(2:n, 1:(n - 1))]  # elasticity of A[i+1, i]
  as.numeric(e_sub)
}

model_names <- names(models)

# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Build elasticity-by-model matrices so:
# rows = models (5 bars), cols = vital rates (groupings)
F_el_by_model <- t(sapply(E_models, extract_fertility_elast))  # 5 x n
S_el_by_model <- t(sapply(E_models, extract_survival_elast))   # 5 x n-1

# Name columns (vital rates)
n <- nrow(models[[1]])
colnames(F_el_by_model) <- paste0("F", 1:n)
colnames(S_el_by_model) <- c(paste0("S", 1:(n - 1)))

# Keep only elasticities that are nonzero in at least one model
keep_F <- colSums(abs(F_el_by_model) > 0) > 0
keep_S <- colSums(abs(S_el_by_model) > 0) > 0

F_el_plot2 <- F_el_by_model[, keep_F, drop = FALSE]
S_el_plot2 <- S_el_by_model[, keep_S, drop = FALSE]

# Leave extra space on the right for legends that won't overlap bars
op <- par(
  mfrow = c(2, 1),
  mar = c(7, 4, 3, 7),
  oma = c(0, 0, 0, 0)
)

op <- par(
  mfrow = c(2, 1),
  mar = c(5, 4, 1.5, 7),
  oma = c(0, 0, 0, 0)
)

# Panel (a): Elasticities with respect to fertility rates
barplot(
  F_el_plot2,
  beside = TRUE,
  ylim = c(0, 0.20),
  col = cols,
  border = "white",
  las = 2,
  ylab = expression("Normalized elasticity of " * R[0] * ""),
  main = "Elasticities with respect to fertility rates"
)
mtext("(a)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)
par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

# Panel (b): Elasticities with respect to survival probabilities
barplot(
  S_el_plot2,
  beside = TRUE,
  ylim = c(0, .20),
  col = cols,
  border = "white",
  las = 2,
  ylab = expression("Normalized elasticity of " * R[0] * ""),
  main = "Elasticities with respect to survival probabilities"
)
mtext("(b)",
      side = 3,
      adj = 0,
      line = 0.2)
lims <-  par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x =  NULL,
  y = mydiff * .75,
  height =  .3 * mydiff * dim(img)[1] / 1024
)

par(xpd = NA)
legend(
  "topright",
  inset = c(-0.15, -0.05),
  legend = model_names,
  fill = cols,
  bty = "n",
  cex = 0.9
)
par(xpd = FALSE)

par(op)
```
**How to read Figure 3.** A larger elasticity (of \(R_0\) with respect to matrix 
entries) for a given vital rate means that small proportional changes to that vital 
rate would have a comparatively larger effect on net reproductive rate. For 
Asian elephants, elasticities are  concentrated in survival probabilities rather 
than fertility rates, reflecting the strong influence of survivorship on \(R_0\). 
Aggregation methods yield similar elasticities.  Late-life fertility rates and
survival probabilities have higher elasticities in the Nagarhole National Park 
population than in the Periyar Reserve population.The silhouette is from PhyloPic 
(https://www.phylopic.org; Keesey, 2023) and were added using the rphylopic R 
package (Gearty & Jones, 2023). The silhouette was contributed as follows: 
"Elephas maximus" by  T. Michael Keesey (2011; CC BY 3.0).

### Comparing elasticities of \(\lambda\) and \(R_0\)

We compare elasticities of \(\lambda\) and \(R_0\) primarily through visual 
inspection, focusing on the relative distribution of elasticity mass between fertility 
and survival components of the matrix.

Across models, elasticity patterns under the \(\lambda\) and \(R_0\) frameworks are 
similar. In both cases, elasticities are dominated by survival transitions, 
consistent with the life history of long-lived species such as Asian elephants. The 
allocation of elasticity mass to fertility terms differs only slightly between the 
two frameworks.

Specifically, the total elasticity mass associated with fertility rates is 0.17 under 
the \(\lambda\) framework and 0.14 under the \(R_0\) framework. This small difference 
indicates that elasticities of \(\lambda\) place slightly greater weight 
on fertility than do elasticities of \(R_0\), although survival probability remains 
the dominant contributor in both cases.

```{r lambda-vs.-R0-elasticities}
#compare elasticities with respect to fertility rates between frameworks 
#fertility rates in lambda framework 
  sum(F_el_plot)/5
#fertility rates in R0 framework
  sum(F_el_plot2)/5
```

## Summary

This vignette demonstrates how age-structured matrix population models (MPMs) defined with 
different projection intervals can be compared by aggregating the coarser model with
`leslie_aggregate()`. Aggregating the annual Leslie MPM for the Asian elephant 
Periyar Reserve population to a 5-year Leslie MPM allowed us to directly 
compare it to a 5-year Leslie MPM of the Nagarhole National Park population.

Estimates of \(\lambda\) and \(R_0\) for the Periyar Reserve population indicate faster 
population growth and shorter generation time relative to the Nagarhole National Park 
population (Table 1). The lower population growth rate of the Nagarhole National Park population 
is attributable to lower fertility rates, as demonstrated by Figure 1. 
Elasticities indicate the survival probability dominates fertility rates
as the most influential group of vital rates controlling \(\lambda\) and \(R_0\).


## References

Chelliah, K., Bukka, H., and Sukumar, R. (2013). Modeling harvest rates and numbers 
from age and sex ratios: a demonstration for elephant populations. *Biological Conservation*, 
165, 54–61. https://doi.org/10.1016/j.biocon.2013.05.008

Gearty, W., and Jones, L. A. (2023). rphylopic: An R package for fetching, transforming, 
and visualising PhyloPic silhouettes. *Methods in Ecology and Evolution*, 14, 2700–2708. 
https://doi.org/10.1111/2041-210X.14221

Goswami, V. R., Vasudev, D., and Oli, M. K. (2014). The importance of conflict-induced 
mortality for conservation planning in areas of human–elephant co-occurrence. 
*Biological Conservation*, 176, 191–198. https://doi.org/10.1016/j.biocon.2014.05.026

Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application to 
ten diverse animal species. *Population Ecology*, 65(3), 146–166. 
https://doi.org/10.1002/1438-390X.12149

Hinrichsen, R. A., Yokomizo, H., and Salguero-Gómez, R. (2026). From theory to 
application: Elasticity-consistent aggregation of Leslie matrix population models 
for comparative demography. *bioRxiv*, preprint. 
https://doi.org/10.64898/2026.02.04.703802

Jones, O. R., Barks, P., Stott, I., James, T. D., Levin, S., Petry, W. K., Capdevila, P., 
Che-Castaldo, J., Jackson, J., Römer, G., Schuette, C., Thomas, C. C., and Salguero-Gómez, R. 
(2022). Rcompadre and Rage—Two R packages to facilitate the use of the COMPADRE and COMADRE 
databases and calculation of life-history traits from matrix population models. 
*Methods in Ecology and Evolution*, 13(12), 2700–2708. 
https://doi.org/10.1111/2041-210X.13792

Salguero-Gómez, R., Jones, O. R., Archer, C. R., Bein, C., de Buhr, H., Farack, C., 
Gottschalk, F., Hartmann, A., Henning, A., Hoppe, G., Römer, G., Ruoff, T., Sommer, V., 
Wille, J., Voigt, J., Zeh, S., Vieregg, D., Buckley, Y. M., Che-Castaldo, J., Hodgson, D., 
Scheuerlein, A., Caswell, H., and Vaupel, J. W. (2016). COMADRE: a global database of animal 
demography. *Journal of Animal Ecology*, 85, 371–384. 
https://doi.org/10.1111/1365-2656.12482

---
title: "mpmaggregate: An Introduction to Aggregating Matrix Population Models"
author:
  - name: "Richard A. Hinrichsen"
    orcid: "0000-0003-0761-3005"
output:
  rmarkdown::html_vignette:
    number_sections: false
    mathjax: default
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{mpmaggregate: An Introduction to Aggregating Matrix Population Models}
  %\VignetteEngine{knitr::rmarkdown}
---
**ORCID:** <https://orcid.org/0000-0003-0761-3005>

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview
Matrix population models (MPMs) are widely used to describe population dynamics,
and large demographic databases such as COMPADRE and COMADRE now make it possible
to study life-history patterns across a wide range of taxa. However, these models
often differ substantially in their structure: they may be constructed with
different projection intervals, life-cycle stage definitions, and levels of
complexity. Such differences can make models difficult to compare directly,
even when they describe similar populations. Aggregation methods address this
problem by combining stages while preserving key demographic properties of the
original model. The `mpmaggregate` package provides tools for reducing the
dimensionality of MPMs under different demographic frameworks, allowing
finer-scale models to be compared with coarser models. For example, a 12-stage
life-cycle model can be reduced to a simpler 3-stage model that still captures
the essential demographic behavior of the population.

### Demographic frameworks: \(\lambda\) and \(R_0\)

Aggregation methods can be defined by the demographic quantities they are designed to 
preserve. In matrix population models, two commonly used measures of growth
are the population growth rate \(\lambda\) and the net reproductive rate \(R_0\).
These quantities describe different aspects of population dynamics, and therefore
preserving them leads to different frameworks for aggregation.

The quantity \(\lambda\) is the dominant eigenvalue of the population projection matrix and 
describes stable population growth per time step (Caswell, 2001). The quantity 
\(R_0\) measures expected lifetime reproductive output and is interpreted 
from a cohort-based perspective (Caswell, 2011). In MPMs, \(R_0\) is calculated from 
the next-generation matrix \(F(I-U)^{-1}\) (Diekmann et al., 1990; Caswell, 2001).

Although \(\lambda\) and \(R_0\) share the same population growth threshold, with 
\(\lambda > 1\) if and only if \(R_0 > 1\), they summarize different aspects 
of demography: \(\lambda\) emphasizes population trajectories through time, 
whereas \(R_0\) emphasizes lifetime reproductive pathways through the life cycle 
(Cushing & Yicang, 1994; de-Camino-Beck & Lewis, 2007). The \(\lambda\) framework 
describes population growth through repeated time-step projection of the entire 
population, so reproduction is incorporated into future population states and 
therefore compounds over time. By contrast, the \(R_0\) framework follows a 
single cohort through the life cycle and counts the offspring produced by that 
cohort, but those offspring are not themselves projected forward. In this sense, 
the \(\lambda\) framework includes compounding reproduction, whereas the 
\(R_0\) framework does not (see Figure 1).

This distinction matters for aggregation, because a method that preserves 
demographic properties in one framework will not generally preserve those same
properties in the other. Aggregation in the \(\lambda\) framework is appropriate 
when the goal is to preserve stable population growth and related population-based 
demographic quantities, whereas aggregation in the \(R_0\) framework is appropriate 
when the goal is to preserve lifetime reproductive output and related cohort-based 
demographic quantities.
<!--display of the actual population projection matrix (4x4) -->
```{r projection matrix, echo=FALSE, out.width="50%", fig.cap="  "}
knitr::include_graphics("figures/Slide2.PNG")
```
<!--figure showing life-cycle representation in different frameworks -->
```{r lambda-r0-figure, echo=FALSE, out.width="90%", fig.cap="Figure 1. Interpretation of λ and R0 frameworks"}
knitr::include_graphics("figures/Slide1.PNG")
```

### Aggregation criteria: standard and elasticity-consistent

Within each framework, users may choose between **standard aggregation**, which maintains 
consistency of the stable population growth rate and stable stage distribution 
(Hooley, 2000; Salguero-Gómez & Plotkin, 2010), and **elasticity-consistent aggregation**, 
which additionally maintains consistency in reproductive values and elasticities 
(Bienvenu et al., 2017). These aggregators are applied in two forms: general-to-general
MPM aggregation and Leslie-to-Leslie MPM aggregation (Hinrichsen, 2023; 
Hinrichsen et al., 2026). 

When applying general-to-general MPM aggregation, the projection interval of the aggregated
matrix does not change. But when applying Leslie-to-Leslie MPM aggregation, the
projection interval changes because, in a Leslie MPM, the projection interval is equal
to the age-class width, which increases with the level of aggregation
(Hinrichsen et al., 2026).

### Package functions
Two primary functions are provided. The function `mpm_aggregate()` aggregates general 
stage-structured MPMs (Caswell, 2001) to coarser stage classifications, whereas 
leslie_aggregate() is designed specifically for age-structured Leslie matrices 
(Leslie, 1945) and returns a lower-dimensional Leslie matrix.  In both cases, 
aggregation is accompanied by an effectiveness metric that quantifies the degree 
of agreement between the aggregated and original models. Values close to 1 indicate 
minimal aggregation error and strong agreement, whereas smaller values reflect 
increasing approximation.

Together, these tools make it possible to simplify complex matrix models, compare
models constructed at different resolutions, and explore how demographic conclusions 
depend on the level of aggregation.

## Loading packages

We begin by loading the `mpmaggregate` package, which provides the functions used 
throughout this vignette for aggregating matrix population models under the \(\lambda\) 
and \(R_0\) demographic frameworks. The package `expm` is used to raise a matrix
to a power, `knitr` is used for report generation, and `kableExtra` is used for
table formatting.

```{r}
library(mpmaggregate)
library(expm)
library(knitr)
library(kableExtra)
```

## Aggregating general-to-general MPM

```{r table-preserved, echo=FALSE, results='asis'}

preserved_tbl <- data.frame(
  Framework = c("lambda", "lambda", "R0", "R0"),
  Criterion = c("standard", "elasticity-consistent", "standard", "elasticity-consistent"),
  Preserved = c(
    "<em>λ</em> (stable growth rate)<br>
     Stable stage distribution<br>
     Interstage flow matrix",

    "<em>λ</em> (stable growth rate)<br>
     Stable stage distribution<br>
     Reproductive values<br>
     Elasticities of <em>λ</em><br>
     Generation time (<em>T</em><sub>a</sub>)",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable stage distribution<br>
     Cohort interstage flow matrix",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable stage distribution<br>
     Cohort reproductive values<br>
     Elasticities of <em>R</em><sub>0</sub><br>
     Cohort generation time (<em>T</em><sub>c</sub>)"
  ),
  check.names = FALSE
)

knitr::kable(
  preserved_tbl,
  format = "html",
  escape = FALSE,
  align = c("l", "l", "l"),
  col.names = c("Framework", "Criterion", "Consistent Demographic Quantities"),
caption = paste0(
  "<strong>Table 1.</strong> Demographic quantities that remain consistent under ",
  "<code>mpm_aggregate()</code> in the <em>λ</em> and <em>R</em><sub>0</sub> frameworks ",
  "for both the standard and elasticity-consistent aggregation criteria. ",
  "Under the <em>λ</em> framework, aggregation maintains consistency in stable ",
  "growth properties, whereas under the <em>R</em><sub>0</sub> framework it maintains ",
  "consistency in cohort-based reproductive quantities. ",
  "Interstage flow weights the columns of <strong>A</strong> by their respective ",
  "stable stage densities (Yokomizo et al. 2024). ",
  "Here, “cohort” denotes the <em>R</em><sub>0</sub>-framework analogues of the ",
  "corresponding <em>λ</em>-framework demographic quantities. ",
  "Notice the symmetry between the <em>λ</em> and <em>R</em><sub>0</sub> frameworks."
)
) |>
  kable_styling(
    full_width = FALSE,
    bootstrap_options = c("striped", "condensed")
  ) |>
  column_spec(3, width = "7cm") |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<div style='border-top: 1px solid black; padding-top: 6px;'>",
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> <code>mpm_aggregate()</code> retains the projection ",
      "interval of the original matrix population model.",
      "</span></div>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )
```

To illustrate general-to-general MPM aggregation, we begin with a simple 3x3 stage
structured \( \mathbf{A} \) COMPADRE MatrixID = 247940 (Salguero‐Gómez et al., 2015), for a
Marula (*Sclerocarya	birrea*) population, with a seedling, juvenile, and adult stage
(Emanuel et al., 2005). Here, the projection interval is 1 year, and is unchanged by
general-to-general MPM aggregation.

We define a grouping of stages using the object `groups`, which specifies how the 
original stages are combined into broader classes. In this example, the first stage 
is left unchanged, while the second and third stages are aggregated into a single 
group. Internally, this grouping is converted to a partition matrix using `mpm_partition()`, 
which is then used to construct the aggregated model.

```{r}
# Example matrices used in the general-to-general MPM aggregation examples below
#population projection matrix for Sclerocarya	birrea
#MatrixID = 247940

# Not run: requires Rcompadre and internet access

#these matrices can be retrieved with:
#library(Rcompadre)
#compadre <- cdb_fetch("compadre")
#mpm <- compadre[compadre$MatrixID == 247940, ]
#A <- matA(mpm)[[1]]
#U <- matU(mpm)[[1]]
#F <- matF(mpm)[[1]]


A <- matrix(
  c(0.0, 0.0000, 12.46,
    0.4, 0.9160,  0.00,
    0.0, 0.0371,  0.99),
  nrow = 3,
  byrow = TRUE
)
#survival/transitions matrix
U <- matrix(
  c(0.0, 0.0000,  0.00,
    0.4, 0.9160,  0.00,
    0.0, 0.0371,  0.99),
  nrow = 3,
  byrow = TRUE
)
#reproduction matrix
F <- matrix(
  c(0.0, 0.0000, 12.46,
    0.0, 0.0000,  0.00,
    0.0, 0.0000,  0.00),
  nrow = 3,
  byrow = TRUE
)

# Stage aggregation used in later examples:
# group vegetative + reproductive stages together,
# leaving the seedling stage separate
groups <- list(
  c(1),
  c(2,3)
)
```

The object `groups` defines how stages are aggregated.
Internally, this is converted to a partition matrix using
`mpm_partition()`.


### Lambda framework: standard aggregation

We first aggregate in the \(\lambda\) framework using **standard aggregation** 
(Hooley, 2000; Salguero-Gomez & Plotkin, 2010). This option is designed to maintain 
consistency in the dominant eigenvalue (\(\lambda\)) and the corresponding stable stage distribution, 
while producing a lower-dimensional matrix  consistent with the grouping defined 
in `groups`. Consistency of interstage flows is also maintained, and this condition 
can be used to derive the standard aggregator (Hinrichsen et al., 2026). 

The call below returns (i) the aggregated projection matrix and (ii) an effectiveness 
value \(\rho^2\), which summarizes how closely the aggregated model reproduces the 
dynamics of the original model. Values of \(\rho^2\) close to 1 indicate near-perfect 
agreement.

```{r}

# General-to-general MPM aggregation
# in the lambda framework, standard aggregation
res_lambda_std <- mpm_aggregate(
  matA = A,
  groups = groups,
  framework = "lambda",
  criterion = "standard"
)

# Aggregated projection matrix
res_lambda_std$matA_agg

# Effectiveness of aggregation
res_lambda_std$effectiveness

# ---- Check: lambda consistency ----

# Expected
spectral_radius(A)

# Lambda of aggregated model
spectral_radius(res_lambda_std$matA_agg)

# ---- Check: stable stage distribution consistency ----

# P is the partitioning matrix that groups stages
P <- mpm_partition(groups=groups,n=3)

# w is the stable stage distribution
w <- stable_stage(A=A)

# Expected
c(P%*%w)

# stable stage distribution of aggregated model
w_agg <-stable_stage(res_lambda_std$matA_agg)
w_agg

# ---- Check: interstage flows consistency ----

# Expected
P%*%A%*%diag(w)%*%t(P)

# Interstage flows of aggregated model
res_lambda_std$matA_agg%*%diag(w_agg)
```

### Lambda framework: elasticity-consistent aggregation

Elasticity-consistent aggregation additionally maintains consistency of reproductive 
values. This approach can produce survival probabilities greater than 1 in the aggregated 
matrix (Bienvenu et al. 2017), a behavior that does not occur under standard aggregation. 
Here, consistency of interstage flows is sacrificed in order to maintain consistency 
of the elasticities of (\(\lambda\)) with respect to matrix entries.

```{r}

# General-to-general MPM aggregation
# in the lambda framework, elasticity-consistent aggregation
res_lambda_el <- mpm_aggregate(
  matA = A,
  matF = F,
  matU = U,
  groups = groups,
  framework = "lambda",
  criterion = "elasticity"
)

# Aggregated matrix
res_lambda_el$matA_agg

# Effectiveness of aggregation
res_lambda_el$effectiveness

# ---- Check: lambda consistency ----

# Expected
spectral_radius(A)

# Lambda of aggregated model
spectral_radius(res_lambda_el$matA_agg)

# ---- Check: stable stage distribution consistency ----

# P is the partitioning matrix
P <- mpm_partition(groups=groups,n=3)

# w us the stable stage distribution 
w <- stable_stage(A=A)

# Expected
c(P%*%w)

# Stable stage distribution of aggregated model
stable_stage(res_lambda_el$matA_agg)

# ---- Check: reproductive values consistency ----

# v is reproductive values (dominant left eigenvector)
v = reproductive_values(A)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(w*v))

# Reproductive values of aggregated model
reproductive_values(res_lambda_el$matA_agg)

# ---- Check: elasticities consistency ----

# Elasticity matrix
EA <- mpm_elasticity(matA=A,framework="lambda")$elasticity

# Expected
P%*%EA%*%t(P)

# Elasticities of aggregated model
mpm_elasticity(matA=res_lambda_el$matA_agg,framework="lambda")$elasticity

# ---- Check: generation time consistency ----

# Expected
generation_time(matF=F,matU=U,framework="lambda")$generation_time

# Generation time from aggregated model
generation_time(matF=res_lambda_el$matF_agg,
                matU=res_lambda_el$matU_agg,
                framework="lambda")$generation_time
```
**Notice** that the [2,1] entry, which is survival probability of the  aggregated matrix 
`res_lambda_el$matA_agg` exceeds 1, which is a known difficulty with the elasticity-consistent 
aggregator (Bienvenu et al. 2017). 

### \(R_0\)framework: decomposition of \( \mathbf{A} \)

We now illustrate aggregation under the \(R_0\) framework, which is based on the
decomposition of the population projection matrix into survival and reproduction
components. Rather than working directly with the full projection matrix \( \mathbf{A} \),
aggregation in this framework is performed using the survival matrix \( \mathbf{U} \) and
the fertility matrix \( \mathbf{F} \).

For this example, we use the same life cycle as above, with \( \mathbf{U} \) describing
survival and transitions among stages and \( \mathbf{F} \) describing reproduction. Together,
these matrices satisfy \( \mathbf{A} = \mathbf{U} + \mathbf{F} \)
.


### \(R_0\) framework: standard aggregation

We first apply standard aggregation in the \(R_0\) framework. In this setting,
aggregation is carried out using the survival and reproduction components of the
model and is designed to ensure consistency of the net reproductive rate (\(R_0\)) and its
associated cohort stable stage distribution. Also consistent is the cohort
interstage flow matrix which weights the columns of \( \mathbf{A} \) by their associated
cohort stable stage densities.


As before, the function returns the aggregated projection matrix and an effectiveness
value that summarizes how closely the aggregated model reproduces the demographic
behavior of the original model.

```{r}

# General-to-general MPM aggregation
# in the R0 framework, standard aggregation
res_R0_std <- mpm_aggregate(
  matU = U,
  matF = F,
  groups = groups,
  framework = "R0",
  criterion = "standard"
)

# Aggregated projection matrix
res_R0_std$matA_agg

# Effectiveness of aggregation
res_R0_std$effectiveness

# ---- Check: R0 consistency ----

# Expected
spectral_radius(F%*%solve(diag(3)-U))

# R0 from aggregated matrix
spectral_radius(res_R0_std$matF_agg%*%solve(diag(2)-res_R0_std$matU_agg))

# ---- Check: cohort stable stage distribution consistency ----

# w is the cohort stable stage distribution from a next generation matrix
w <- stable_stage(solve(diag(3)-U)%*%F)

# Expected for consistency
c(P%*%stable_stage(solve(diag(3)-U)%*%F))

# Aggregated cohort stable stage distribution
w_agg <- stable_stage(solve(diag(2)-res_R0_std$matU_agg)%*%res_R0_std$matF_agg)
w_agg

# ---- Check: cohort interstage flows  consistency ----

# Expected
P%*%A%*%diag(w)%*%t(P)

# Aggregated interstage flows
res_R0_std$matA_agg%*%diag(w_agg)
```


### \(R_0\) framework: elasticity-consistent aggregation

Elasticity-consistent aggregation in the \(R_0\) framework additionally ensures
consistent cohort reproductive values and the elasticities of \(R_0\) with respect to matrix
entries. As in the \(\lambda\) framework, this approach can produce survival
probabilities greater than 1 in the aggregated matrix. This behavior reflects the
constraints imposed by elasticity preservation and does not occur under standard
aggregation or in Leslie-to-Leslie aggregation.

```{r}

# General-to-general MPM aggregation
# in the R0 framework, elasticity-consistent aggregation
res_R0_el <- mpm_aggregate(
  matU = U,
  matF = F,
  groups = groups,
  framework = "R0",
  criterion = "elasticity"
)

# Aggregated projection matrix
res_R0_el$matA_agg

# Effectiveness of aggregation
res_R0_el$effectiveness

# ---- Check: R0 consistency ----

# Expected R0
R0 <- spectral_radius(F%*%solve(diag(3)-U))
R0

# R0 from the aggregated model
spectral_radius(res_R0_el$matF_agg%*%solve(diag(2)-res_R0_el$matU_agg))

# ---- Check: cohort stable stage distribution consistency ----

# Get partitioning matrix
P <- mpm_partition(groups=groups,n=3)

# Construct cohort projection matrix
Aref = F + U*R0

# Cohort stable stage distribution
w <- stable_stage(A=Aref)

# Expected for consistency
c(P%*%w)

# Aggregated cohort projection matrix
Aref_agg = res_R0_el$matF_agg+ R0*res_R0_el$matU_agg

# Aggregated cohort stable stage distribution
stable_stage(Aref_agg)

# ---- Check: cohort reproductive values consistency ----
v = reproductive_values(Aref)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(w*v))

# Cohort reproductive values for aggregated matrix
reproductive_values(Aref_agg)

# ---- Check: elasticities of R0 consistency ----
# Here elasticities are normalized to sum to 1

# Original elasticities
EA <- mpm_elasticity(matF=F,matU=U,framework="R0")$elasticity

# Expected for consistency
P%*%EA%*%t(P)

# Elasticities of R0 from aggregated matrix
mpm_elasticity(matF=res_R0_el$matF_agg,matU=res_R0_el$matU_agg,framework="R0")$elasticity

# ---- Check: cohort generation time consistency ----

# Expected (from original model)
generation_time(matF=F,matU=U,framework="R0")$generation_time

# Cohort generation time from aggregated model
generation_time(matF = res_R0_el$matF_agg, 
                matU = res_R0_el$matU_agg,
                framework="R0")$generation_time
```

## Aggregating Leslie-to-Leslie MPM

```{r table-leslie-preserved, echo=FALSE, results='asis'}
preserved_leslie_tbl <- data.frame(
  Framework = c("lambda", "lambda", "R0", "R0"),
  Criterion = c("standard", "elasticity-consistent", "standard", "elasticity-consistent"),
  Preserved = c(
    "<em>λ</em> (stable growth rate)<br>
     Stable age distribution<br>
     Interstage flow matrix of <strong>A</strong><sup><em>k</em></sup>",

    "<em>λ</em> (stable growth rate)<br>
     Stable age distribution<br>
     Reproductive values<br>
     Elasticities of <em>λ</em><sup><em>k</em></sup> with respect to the entries of <strong>A</strong><sup><em>k</em></sup><br>
     Generation time (<em>T</em><sub>a</sub>) of <strong>A</strong><sup><em>k</em></sup>",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable age distribution<br>
     consistent of <strong>A</strong><sub><em>k</em></sub><sup></sup>",

    "<em>R</em><sub>0</sub> (net reproductive rate)<br>
     Cohort stable age distribution<br>
     Cohort reproductive values<br>
     Elasticities of <em>R</em><sub>0</sub> with respect to <strong>A</strong><sub><em>k</em></sub><sup></sup><br>
     Cohort generation time (<em>T</em><sub>c</sub>) for <strong>A</strong><sub><em>k</em></sub><sup></sup>"
  ),
  check.names = FALSE
)

knitr::kable(
  preserved_leslie_tbl,
  format = "html",
  escape = FALSE,
  align = c("l", "l", "l"),
  col.names = c("Framework", "Criterion", "Consistent Demographic Quantities"),
caption = paste0(
  "<strong>Table 2.</strong> Summary of the demographic properties that remain consistent under ",
  "<code>leslie_aggregate()</code> when reducing a Leslie matrix from dimensionality ",
  "<em>n</em> to <em>m</em>. The quantities the remain consistent depend on both the demographic framework ",
  "(long-term growth under <em>λ</em> or lifetime reproductive output under ",
  "<em>R</em><sub>0</sub>) and the aggregation criterion (standard versus ",
  "elasticity-consistent). The integer <em>k</em> = <em>n</em>/<em>m</em> defines the ",
  "temporal rescaling by aggregating a Leslie model. Quantities in the ",
  "<em>λ</em> framework are defined for <strong>A</strong><sup><em>k</em></sup>, ",
  "while cohort quantities in the <em>R</em><sub>0</sub> framework are defined for ",
  "<strong>A</strong><sub><em>k</em></sub><sup>&dagger;</sup>. Notice the structural symmetry ",
  "between the <em>λ</em> and <em>R</em><sub>0</sub> frameworks."
)
) |>
  kable_styling(
    full_width = FALSE,
    bootstrap_options = c("striped", "condensed")
  ) |>
  column_spec(3, width = "7cm") |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<div style='border-top: 1px solid black; padding-top: 6px;'>",
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> Aggregation with <code>leslie_aggregate()</code> ",
      "changes the projection interval from Δt to <em>k</em>Δt.<br>",
      "<sup>&dagger;</sup> <strong>A</strong><sub><em>k</em></sub><sup></sup> denotes the <em>k</em>-step-ahead projection matrix ",
      "in the <em>R</em><sub>0</sub> framework.</span></div>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )
```

In some applications, it is desirable to aggregate an age-structured Leslie matrix
while retaining its Leslie form (Hinrichsen 2023; Hinrichsen et al., 2026). The 
function `leslie_aggregate()` is designed for this purpose. It aggregates a Leslie 
matrix with fine-grained age classes into a coarser Leslie matrix of dimensionality
`ngroups`.

To illustrate Leslie-to-Leslie MPM aggregation, we begin with a simple 4x4 Leslie matrix 
\( \mathbf{A} \) COMADRE MatrixID = 248187 (Salguero‐Gómez et al., 2016), for a 
fathead minnow (*Pimephales	promelas*) population (Spromberg & Birge, 2005).

```{r}
# New example used for Leslie-to-Leslie aggregation
# These matrices replace the earlier example matrices (A, U, F)
# Leslie population projection matrix for Pimephales promelas (MatrixID = 248187)

# Not run: requires Rcompadre and internet access

# Matrices can be retrieved with:
# library(Rcompadre)
# comadre <- cdb_fetch("comadre",version = "4.23.3.1")
# mpm <- comadre[comadre$MatrixID == 248187, ]
# A <- matA(mpm)[[1]]
# U <- matU(mpm)[[1]]
# F <- matF(mpm)[[1]]



# Leslie population projection matrix
A <- rbind(
  c(0.00, 0.00,  6.00, 62.50),
  c(0.50, 0.00,  0.00,  0.00),
  c(0.00, 0.18,  0.00,  0.00),
  c(0.00, 0.00,  0.12,  0.00)
)

# Reproduction matrix
F <- rbind(
  c(0.00, 0.00,  6.00, 62.50),
  c(0.00, 0.00,  0.00,  0.00),
  c(0.00, 0.00,  0.00,  0.00),
  c(0.00, 0.00,  0.00,  0.00)
)

# Survival/transition matrix
U <- rbind(
  c(0.00, 0.00,  0.00,  0.00),
  c(0.50, 0.00,  0.00,  0.00),
  c(0.00, 0.18,  0.00,  0.00),
  c(0.00, 0.00,  0.12,  0.00)
)

# Partitioning matrix used for the aggregation examples below
# (first two age classes combined, last two combined)
# In practice this matrix is constructed automatically inside leslie_aggregate()
# It is reproduced here only to verify the aggregation results.
P <- rbind(
  c(1,1,0,0),
  c(0,0,1,1)
)

# Dimensions used in the aggregation examples
n <- 4     # dimensionality of the original matrix A
m <- 2     # dimensionality of the aggregated matrix
k <- n / m #(also constructed automatically)
```
  


### Lambda framework: standard aggregation

We first aggregate the Leslie matrix in the \(\lambda\) framework using standard
aggregation (Hinrichsen, 2023). As in the general-to-general case, this approach ensures
consistency of the stable population growth rate (\(\lambda\)) and the stable age distribution,
while producing a Leslie matrix of reduced dimensionality. 
The interstage flow of the aggregated matrix is consistent with that of
\( \mathbf{A}^{k} \).

```{r}

# Leslie-to-Leslie MPM aggregation
# in the lambda framework, standard aggregation
res_leslie_lambda_std <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "lambda",
  criterion = "standard"
)


# Aggregated Leslie matrix
res_leslie_lambda_std$matAk_agg

# Effectiveness of aggregation
res_leslie_lambda_std$effectiveness

# ---- Check: lambda consistency ----

# Expected lambda
spectral_radius(A)^k

# Lambda of aggregated model
spectral_radius(res_leslie_lambda_std$matAk_agg)

# ---- Check: stable age distribution consistency ----

# Stable age distribution
w <- leslie_stable_age(A=A)

# Expected 
c(P%*%w)

# Aggregated stable age distribution
w_agg <-leslie_stable_age(res_leslie_lambda_std$matAk_agg)
w_agg

# ---- Check: interstage flow consistency ----

# Expected
P%*%(A%^%k)%*%diag(w)%*%t(P)

# Interstage flows from aggregated model
res_leslie_lambda_std$matAk_agg%*%diag(w_agg)
```

**Note.** In this case, the dimensionality of the aggregated matrix (\(m = 2\))
divides the dimensionality of \( \mathbf{A} \) (\(n = 4\)) evenly. More generally, when these
dimensionalities are not compatible, a disaggregation step is used internally
to expand \( \mathbf{L} \) to a compatible dimensionality prior to aggregation 
(Hinrichsen, 2023).

### Lambda framework: elasticity-consistent aggregation

We next apply elasticity-consistent aggregation to the Leslie matrix in the
\(\lambda\) framework (Hinrichsen et al., 2026). In addition to yielding consistent stable population growth
rate and stable age distribution, this approach also yields consistent reproductive values and
the elasticities of \(\lambda^{k}\) with respect to the entries entries of \( \mathbf{A}^{k} \).
Unlike elasticity-consistent aggregation for general stage-structured MPMs, Leslie-to-Leslie
elasticity-consistent aggregation does not produce survival probabilities greater
than 1.

```{r}

# Leslie-to-Leslie MPM aggregation
# in the lambda framework, elasticity-consistent aggregation
res_leslie_lambda_el <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "lambda",
  criterion = "elasticity"
)

# Aggregated Leslie matrix
res_leslie_lambda_el$matAk_agg

# Effectiveness of aggregation
res_leslie_lambda_el$effectiveness

# get aggregated F and U
F_agg <-matrix(0,2,2)
F_agg[1,] <- res_leslie_lambda_el$matAk_agg[1,]
U_agg <- res_leslie_lambda_el$matAk_agg
U_agg[1,] <-0

# ---- Check: lambda consistency ----

# Expected
spectral_radius(A)^k

# Lambda for aggregated model
spectral_radius(res_leslie_lambda_el$matAk_agg)

# ---- Check: stable age distribution consistency ----

# Stable age distribution
w <- leslie_stable_age(A=A)

# Expected
c(P%*%w)

# Stable age distribution of aggregated odel
w_agg <-leslie_stable_age(res_leslie_lambda_el$matAk_agg)
w_agg

# ---- Check: reproductive values consistency ----

# Get reproductive values
v <- leslie_reproductive_values(A=A)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(v*w))

# Reproductive values of aggregated model
leslie_reproductive_values(A=res_leslie_lambda_el$matAk_agg)

# ---- Check: elasticities consistency ----

# Get elasticities of lambda^k
EA <- mpm_elasticity(A%^%k,framework="lambda")$elasticity

# Expected
P%*%EA%*%t(P)

# Elasticities of lambda^k for aggregated model
mpm_elasticity(res_leslie_lambda_el$matAk_agg,framework="lambda")$elasticity

# ---- Check: generation time consistency ----

# Survival/transition matrix
U_k <- U%^%k

# Reproduction matrix
F_k <- A%^%k - U_k

# Expected generation time
generation_time(matF=F_k,matU=U_k,framework="lambda")$generation_time

# Generation time of aggregated model
generation_time(matF=F_agg,matU=U_agg,framework="lambda")$generation_time
```
---
### \(R_0\) framework

Leslie-to-Leslie aggregation can also be carried out under the **\(R_0\) framework**,
emphasizing net reproductive rate rather than stable population
growth. As in the general-to-general case, aggregation in this framework is based
on the underlying survival and fertility components of the Leslie matrix,
but the result again retains Leslie form.

### \(R_0\) framework: standard aggregation

We first apply **standard aggregation** in the \(R_0\) framework. This approach yields
consistent net reproductive rate (\(R_0\)) and the cohort stable age distribution, while
producing a coarser Leslie matrix compatible with the specified number of age groups.

```{r}

# Leslie-to-Leslie MPM aggregation
# in the R0 framework, standard aggregation
res_leslie_R0_std <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "R0",
  criterion = "standard"
)

# Aggregated Leslie matrix
res_leslie_R0_std$matAk_agg

# Effectiveness of aggregation
res_leslie_R0_std$effectiveness

# get F and U for aggregated matrix
F_agg <- matrix(0,2,2)
F_agg[1,] <- res_leslie_R0_std$matAk_agg[1,]
U_agg <- res_leslie_R0_std$matAk_agg
U_agg[1,] <- 0

# ---- Check: generation time consistency ----

# Expected
R0 <- spectral_radius(F%*%solve(diag(4)-U))
R0 

# R0 for aggregated model
spectral_radius(F_agg%*%solve(diag(2)-U_agg))


# ---- Check: cohort stable age distribution consistency ----

# Standardized cohort projection matrix
A_ref <- (F + U*R0)/R0

# Standardized cohort projection matrix for aggregated model
A_ref_agg <- (F_agg + U_agg*R0)/R0

# w is the stable age distribution
w <- leslie_stable_age(A=A_ref)

# Expected
c(P%*%w)

# Stable age distribution for aggregated model
w_agg <-leslie_stable_age(A_ref_agg)
w_agg

# ---- Check: cohort interstage flows consistency ----

# Survival/transition matrix (k steps ahead)
U_k <- U%^%k

# Reproduction matrix (k steps ahead)
F_k <- A%^%k - U_k

# A projection matrix (k steps ahead)
A_k <- F_k + U_k

# Expected
P%*%A_k%*%diag(w)%*%t(P)

# Interstage flows for aggregated model
(F_agg + U_agg)%*%diag(w_agg)
```

### \(R_0\) framework: elasticity-consistent aggregation

We now apply **elasticity-consistent aggregation** in the \(R_0\) framework. In addition
to providing consistent net reproductive rate (\(R_0\)) and the cohort stable age distribution,
this approach provides consistent cohort reproductive values and the elasticities of \(R_0\)
with respect to the entries of \( \mathbf{A}_k^{*} \), the \(k\)-step–ahead version of \( \mathbf{A} \), where
\(k = n/m\), \(n\) is the dimensionality of the original model, and \(m\) is the
dimensionality of the aggregated model. As in the \(\lambda\) framework, the Leslie
structure is maintained and survival probabilities cannot exceed 1.

```{r}

# Leslie-to-Leslie MPM aggregation
# in the R0 framework, elasticity-consistent aggregation
res_leslie_R0_el <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "R0",
  criterion = "elasticity"
)

# Aggregated Leslie matrix
res_leslie_R0_el$matAk_agg

# Effectiveness of aggregation
res_leslie_R0_el$effectiveness

# Get F and U for aggregated matrix
F_agg <- matrix(0,2,2)
F_agg[1,] <- res_leslie_R0_el$matAk_agg[1,]
U_agg <- res_leslie_R0_el$matAk_agg
U_agg[1,] <- 0

# ---- Check: R0 consistency ----

# Expected
R0 <- spectral_radius(F%*%solve(diag(4)-U))
R0 

# R0 of aggregated model
spectral_radius(F_agg%*%solve(diag(2)-U_agg))


# ---- Check: cohort stage age distribution consistency ----

# Standardized cohort projection matrix
A_ref <- (F + U*R0)/R0

# Standardized cohort projection matrix for aggregated model
A_ref_agg <- (F_agg + U_agg*R0)/R0

# Cohort stable age distribution
w <- leslie_stable_age(A=A_ref)

# Expected
c(P%*%w)

# Cohort stage age distribution for aggregated model
w_agg <-leslie_stable_age(A_ref_agg)
w_agg

# ---- Check: cohort reproductive values consistency ----

# Cohort reproductive values
v <- leslie_reproductive_values(A_ref)

# Expected
c(solve(P%*%diag(w)%*%t(P))%*%P%*%(v*w))

# Cohort reproductive values of aggregated model
leslie_reproductive_values(A_ref_agg)

# ---- Check: elasticities of R0 consistency ----
# These elasticities are normalized to sum to 1

# Survival/transition matrix (k steps ahead)
U_k <- U%^%k

# Reproduction matrix (k steps ahead)
F_k <- A%^%k - U_k

# Cohort projection matrix (k steps ahead)
A_k <- F_k + U_k

# Elasticity of R0 (normalized to sum to 1)
EA <- mpm_elasticity(matF=F_k,matU=U_k,framework="R0")$elasticity

# Expected
P%*%EA%*%t(P)

# Elasticity of R0 for the aggregated model
mpm_elasticity(matF=F_agg,matU=U_agg,framework="R0")$elasticity

# ---- Check: cohort generation time consistency ----

# Expected
generation_time(matF=F_k, matU = U_k, framework="R0")$generation_time

# Generation time of aggregated model
generation_time(matF=F_agg,matU=U_agg, framework="R0")$generation_time
```



## Interpretation

- The **lambda framework** ensures consistency of stable growth and stable structure.
- The **R0 framework** ensures consistency of generation-based reproductive dynamics.
- **Standard aggregation** ensures consistency of the core demographic quantity associated
  with the chosen framework (either \(\lambda\) or \(R_0\)).
- **Elasticity-consistent aggregation** additionally ensures consistency of reproductive
  values, associated elasticities, and generation times within their
  respective frameworks.


## References

Bienvenu, F., Akçay, E., Legendre, S., & McCandlish, D. M. (2017). The genealogical
decomposition of a matrix population model with applications to the aggregation
of stages. *Theoretical Population Biology*, 115, 69–80.
https://doi.org/10.1016/j.tpb.2017.04.002

Caswell, H. (2001). *Matrix Population Models: Construction, Analysis, and Interpretation* 
(2nd ed.). Sinauer Associates, Sunderland, MA.

Caswell, H. (2011). Beyond R0: demographic models for variability of lifetime 
reproductive output. *PloS ONE*, 6(6), p.e20809. 
https://doi.org/10.1371/journal.pone.0020809

Cushing, J.M., & Yicang, Z.(1994). The net reproductive value and stability in 
matrix population models. *Natural Resource Modeling*, 8(4), 297-333. 
https://doi.org/10.1111/j.1939-7445.1994.tb00188.x

de-Camino-Beck, T., & Lewis, M.A. (2007). A new method for calculating net 
reproductive rate from graph reduction with applications to the control of 
invasive species. *Bulletin of Mathematical Biology*, 69(4), 1341-1354.
https://doi.org/10.1007/s11538-006-9162-0

Diekmann, O., Heesterbeek, J.A.P., & Metz, J.A.J. (1990). On the definition and 
the computation of the basic reproduction ratio R0 in models for infectious 
diseases in heterogeneous populations. *Journal of mathematical biology*, 28(4), 
365-382. https://doi.org/10.1007/BF00178324

Emanuel, P. L., Shackleton, C. M., & Baxter, J. S. (2005). Modelling the sustainable
harvest of *Sclerocarya birrea* subsp. *caffra* fruits in the South African lowveld.
*Forest Ecology and Management*, 214(1–3), 91–103.
https://doi.org/10.1016/j.foreco.2005.03.066

Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application to
ten diverse animal species. *Population Ecology*, 65(3), 146–166.
https://doi.org/10.1002/1438-390X.12149

Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
application: Elasticity-consistent aggregation of Leslie matrix population models
for comparative demography. *bioRxiv*, preprint.
https://doi.org/10.64898/2026.02.04.703802

Hooley, D. E. (2000). Collapsed matrices with (almost) the same eigenstuff.
*The College Mathematics Journal*, 31(4), 297–299.
https://doi.org/10.1080/07468342.2000.11974162

Leslie, P. H. (1945). On the use of matrices in certain population mathematics.
*Biometrika*, 33(3), 183–212.
https://doi.org/10.2307/2332297

Salguero-Gómez, R., Jones, O. R., Archer, C. R., Buckley, Y. M., Che-Castaldo, J.,
Caswell, H., Hodgson, D., Scheuerlein, A., Conde, D. A., Brinks, E., de Buhr, H.,
Farack, C., Gottschalk, F., Hartmann, A., Henning, A., Hoppe, G., Römer, G.,
Runge, J., Ruoff, T., Wille, J., Zeh, S., Davison, R., Vieregg, D., Baudisch, A.,
Altwegg, R., Colchero, F., Dong, M., de Kroon, H., Lebreton, J.-D., Metcalf, C. J. E.,
Neel, M. M., Parker, I. M., Takada, T., Valverde, T., Vélez-Espino, L. A., Wardle,
G. M., Franco, M., & Vaupel, J. W. (2015). The COMPADRE Plant Matrix Database:
an open online repository for plant demography. *Journal of Ecology*, 103, 202–218.
https://doi.org/10.1111/1365-2745.12334

Salguero-Gómez, R., Jones, O. R., Archer, C. R., Bein, C., de Buhr, H., Farack, C.,
Gottschalk, F., Hartmann, A., Henning, A., Hoppe, G., Römer, G., Ruoff, T., Sommer,
V., Wille, J., Voigt, J., Zeh, S., Vieregg, D., Buckley, Y. M., Che-Castaldo, J.,
Hodgson, D., Scheuerlein, A., Caswell, H., & Vaupel, J. W. (2016). COMADRE:
a global database of animal demography. *Journal of Animal Ecology*, 85, 371–384.
https://doi.org/10.1111/1365-2656.12482

Salguero-Gómez, R., & Plotkin, J. B. (2010). Matrix dimensions bias demographic
inferences: implications for comparative plant demography. *The American Naturalist*,
176, 710–722.
https://doi.org/10.1086/657044

Spromberg, J. A., & Birge, W. J. (2005). Modeling the effects of chronic toxicity
on fish populations: The influence of life-history strategies. 
*Environmental Toxicology and Chemistry*, 24(6), 1532–1540.
https://doi.org/10.1897/04-160.1

Yokomizo, H., Fukaya, K., Lambrinos, J. G., Kawai, Y., & Takada, T. (2024).
Interstage flow matrices: Population statistics derived from matrix population
models. *Journal of Ecology*, 112(6), 1326–1338.
https://doi.org/10.1111/1365-2745.14303

---
title: "Aggregating a general-to-general MPM"
author:
  - name: "Richard A. Hinrichsen"
    orcid: "0000-0003-0761-3005"
output:
  rmarkdown::html_vignette:
    number_sections: false
    mathjax: default
vignette: >
  %\VignetteIndexEntry{Aggregating a general-to-general MPM}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
**ORCID:** <https://orcid.org/0000-0003-0761-3005>

## Overview

Matrix population models (MPMs) often differ in the number and definition of life-cycle 
stages used to represent population structure, making comparisons difficult. This 
heterogeneity is prevalent in models structured by age, stage, or size, referred 
to here generically as stage-structured models. To facilitate comparison of MPMs 
with differing stage resolutions, we use aggregation, whereby a complex model is 
reduced to a simpler form by grouping stages. The idea is to choose these groupings 
in such a way that the aggregated model is directly comparable with another model 
that used those same groupings in its construction.

In this vignette, we demonstrate how to aggregate a general MPM 
to another general MPM using the `mpmaggregate` package. Unlike Leslie-to-Leslie MPM 
aggregation, which modifies the projection interval, general-to-general MPM aggregation 
leaves the projection interval unchanged.  

We demonstrate aggregation of a size-structured MPM for *Rourea induta* 
originally published by Hoffmann (1999) and obtained from the COMPADRE database 
(Salguero-Gómez et al., 2015) via `Rcompadre` (Jones et al., 2022). 
The model contains multiple fine-grained size stages and is well suited to illustrate 
how aggregation can be used to simplify stage structure while preserving 
key demographic properties (Salguero-Gómez & Plotkin, 2010).

We emphasize interpretation throughout: how aggregation alters the 
representation of the life cycle, what demographic information is preserved, and how 
the resulting aggregated model can be used for comparison or subsequent demographic 
analyses, such as estimating population growth rates, generation time, or elasticities. 
Although *Rourea induta* is used here as an example, the approach is broadly applicable 
to general stage-structured matrix population models across taxa.

We also emphasize how different aggregators influence the reduced MPM. 
The vignette compares all four alternative aggregation approaches of `mpm_aggregate()`, 
which differ in the demographic quantities they are designed to preserve and in the 
criteria used to fit aggregated transitions.

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Loading packages
We begin by loading the `mpmaggregate` package, which provides the functions used 
throughout this vignette for aggregating matrix population models under the \(\lambda\) 
and \(R_0\) demographic frameworks. The package `knitr` is used for report generation, 
and `kableExtra` is used for
table formatting.

```{r packages}
library(mpmaggregate)
library(knitr)
library(kableExtra)
```

## Data acquisition
To demonstrate general-to-general aggregation, we use a 14 × 14 matrix population 
model for *Rourea induta* (MatrixID 246781) from the COMPADRE database (Hoffmann 1999),
which has a projection interval of 1 year.

```{r fetch-comadre}
# Example matrices used in the general-to-general MPM aggregation examples below
# Population projection matrix for Rourea induta

# Not run: requires Rcompadre and internet access

# MatrixID = 246781
# Matrices can be retrieved with:
# library(Rcompadre)
# compadre <- cdb_fetch("compadre")
# mpm <- compadre[compadre$MatrixID == 246781, ]
# matA <- matA(mpm)[[1]]
# matU <- matU(mpm)[[1]]
# matF <- matF(mpm)[[1]]
# matC <- matC(mpm)[[1]]


# The matrices are defined locally so that Rcompadre and internet access
# are not required

# Population projection matrix
matA <- matrix(c(
  0.000,0.000,0.000,0.000,0.002,0.007,0.021,0.055,0.102,0.139,0.176,0.191,0.203,0.214,
  0.000,0.000,0.000,0.001,0.002,0.004,0.007,0.011,0.016,0.021,0.036,0.052,0.072,0.096,
  0.833,0.043,0.883,0.151,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.391,0.043,0.791,0.079,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.348,0.000,0.047,0.810,0.139,0.024,0.024,0.056,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.087,0.733,0.082,0.071,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.087,0.000,0.000,0.000,0.129,0.765,0.024,0.000,0.013,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.106,0.762,0.028,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.012,0.119,0.778,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.083,0.861,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.127,0.891,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.909,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.918,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.082,0.967
), nrow = 14, byrow = TRUE)

# Survival/transition matrix
matU <- matrix(c(
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.833,0.043,0.883,0.151,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.391,0.043,0.791,0.079,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.348,0.000,0.047,0.810,0.139,0.024,0.024,0.056,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.087,0.733,0.082,0.071,0.000,0.000,0.000,0.000,0.000,0.000,
  0.000,0.087,0.000,0.000,0.000,0.129,0.765,0.024,0.000,0.013,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.106,0.762,0.028,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.012,0.119,0.778,0.000,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.083,0.861,0.000,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.127,0.891,0.000,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.909,0.000,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.091,0.918,0.000,
  0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.082,0.967
), nrow = 14, byrow = TRUE)

# Sexual reproduction matrix
matF <- matrix(c(
  0,0,0,0,0.002,0.007,0.021,0.055,0.102,0.139,0.176,0.191,0.203,0.214,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000
),nrow=14,byrow=TRUE)

#Clonal reproduction matrix
matC <- matrix(c(
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.001,0.002,0.004,0.007,0.011,0.016,0.021,0.036,0.052,0.072,0.096,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
  0,0,0,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000
), nrow = 14, byrow = TRUE)

#redefined matF so it includes sexual + clonal reproduction
matF <- matF + matC

#sanity check
stopifnot(all.equal(unname(matA), unname(matU + matF)))

#1                active          Seedling
#2                active            Sucker
#3                active         1-1 .9 mm
#4                active         2-3 .9 mm
#5                active         4-5 .9 mm
#6                active         6-7 .9 mm
#7                active        8-.9 .9 mm
#8                active       10-14 .9 mm
#9                active        15-19.9 mm
#10               active          20-29 mm
#11               active          30-39 mm
#12               active          40-49 mm
#13               active          50-69 mm
#14               active            70+ mm

# Stage aggregation used in later examples:
# Form the aggregated groups, leaving Seedling and Sucker stages alone
groups <- list(
  c(1),                #Seedling
  c(2),                #Sucker
  c(3, 4, 5, 6),       #1-7.9 mm
  c(7, 8, 9, 10),      #8-29 mm
  c( 11, 12, 13, 14)   #30+ mm
)
```
## Helper functions for demographic quantities

We compute:

- \(\lambda\): spectral radius of \( \mathbf{A} \)
- \(R_0\): spectral radius of \( \mathbf{F} \left( \mathbf{I} - \mathbf{U} \right)^{-1} \)
- \(T_a\): generation time (years) under the \(\lambda\) framework
- \(T_c\): cohort generation time (years) under the \(R_0\) framework

```{r helpers}
`%||%` <- function(x, y) if (!is.null(x)) x else y

get_R0 <- function(U, F) {
  I <- diag(nrow(U))
  N <- solve(I - U)
  K <- F %*% N
  spectral_radius(K)
}

get_Ta <- function(U, F) {
  generation_time(F, U, framework = "lambda")$generation_time
}

get_Tc <- function(U, F) {
  generation_time(F, U, framework = "R0")$generation_time
}
```

## Aggregate in four ways

We aggregate the model using the following four combinations of framework and criterion:

1. `framework = "lambda"`, `criterion = "standard"`
2. `framework = "lambda"`, `criterion = "elasticity"`
3. `framework = "R0"`, `criterion = "standard"`
4. `framework = "R0"`, `criterion = "elasticity"`

In the \(\lambda\) framework, we consider both standard aggregation and elasticity-consistent 
aggregation. Standard aggregation follows the method of Hooley (2000), while elasticity-consistent 
aggregation uses the "genealogical collapse" method of Bienvenu et al. (2017). Both approaches 
maintain the consistency of the asymptotic population growth rate \(\lambda\) and 
stable stage distribution. Elasticity-consistent aggregation additionally maintains
consistency of the reproductive values and the elasticities of \(\lambda\) with respect 
to matrix entries.

We extend these approaches to the \(R_0\) framework, where standard aggregation 
maintains the consistency of the net reproductive rate \(R_0\) and the cohort stable 
stage distribution. Elasticity-consistent aggregation in this framework additionally 
maintains consistency of the cohort reproductive values and the elasticities of \(R_0\) with respect 
to matrix entries. 

```{r aggregate}
agg1 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "lambda",
  criterion = "standard"
)

agg2 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "lambda",
  criterion = "elasticity"
)

agg3 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "R0",
  criterion = "standard"
)

agg4 <- mpm_aggregate(
  matU = matU,
  matF = matF,
  groups = groups,
  framework = "R0",
  criterion = "elasticity"
)

# Extract aggregated U and F robustly (in case object field names differ by version)
extract_U <- function(x) x$matUk_agg %||% x$matU_agg %||% x$matUagg %||% x$U
extract_F <- function(x) x$matFk_agg %||% x$matF_agg %||% x$matFagg %||% x$F
extract_A <- function(x) x$matAk_agg %||% x$matA_agg %||% x$matAagg %||% x$A

as_ufA <- function(x) {
  U <- extract_U(x)
  F <- extract_F(x)
  A <- extract_A(x)
  if (is.null(A) && !is.null(U) && !is.null(F)) A <- U + F
  list(U = U, F = F, A = A)
}

m0 <- list(U = matU, F = matF, A = matA)
m1 <- as_ufA(agg1)
m2 <- as_ufA(agg2)
m3 <- as_ufA(agg3)
m4 <- as_ufA(agg4)


eff <- c(NA,agg1$effectiveness,agg2$effectiveness,agg3$effectiveness,agg4$effectiveness)

models <- list(
  "Original"                  = m0,
  "Agg lambda + standard"     = m1,
  "Agg lambda + elasticity"   = m2,
  "Agg R0 + standard"         = m3,
  "Agg R0 + elasticity"       = m4
)

model_labels <- c(
  Original         = "Original",
  Agg_lambda_std   = "Agg &lambda; + standard",
  Agg_lambda_elast = "Agg &lambda; + elasticity",
  Agg_R0_std       = "Agg R<sub>0</sub> + standard",
  Agg_R0_elast     = "Agg R<sub>0</sub> + elasticity"
)

names(models) <- c(
  "Original",
  "Agg_lambda_std",
  "Agg_lambda_elast",
  "Agg_R0_std",
  "Agg_R0_elast"
)

# Basic checks
stopifnot(all(sapply(models, function(m) !is.null(m$A))))
#stopifnot(all(sapply(models, function(m) nrow(m$A) == length(groups))))

#show aggregated matrices
print("Agg λ + standard")
m1$A
print("Agg λ + elasticity")
m2$A
print("Agg R0 + standard")
m3$A
print("Agg R0 + elasticity")
m4$A

```

**Important.** Elasticity-consistent aggregation can, in some cases, produce biologically 
infeasible parameter values (Bienvenu et al. 2017). As seen in the aggregated matrices above, 
aggregation under the \(\lambda\) framework yields a model with a survival probability 
greater than 1 (1.12). A similar issue arises under the \(R_0\) framework, where 
elasticity-consistent aggregation results in a survival probability of 1.22. This issue 
does not occur when using standard aggregation in the \(\lambda\) framework 
(Hooley, 2000) or in the \(R_0\) framework.

## Compare \(\lambda\), \(R_0\), \(T_a\), and \(T_c\) in a table

```{r results-table}

results <- data.frame(
  Model  = unname(model_labels[names(models)]),
  lambda = sapply(models, function(m) spectral_radius(m$A)),
  R0     = sapply(models, function(m) get_R0(m$U, m$F)),
  Ta     = sapply(models, function(m) get_Ta(m$U, m$F)),
  Tc     = sapply(models, function(m) get_Tc(m$U, m$F)),
  Effectiveness = eff,
  row.names = NULL,
  stringsAsFactors = FALSE
)

n_rows <- nrow(results)

knitr::kable(
  results,
  col.names = c(
    "Model",
    "<em>&lambda;</em>",
    "<em>R</em><sub>0</sub>",
    "<em>T</em><sub>a</sub> (years)",
    "<em>T</em><sub>c</sub> (years)",
    "<em>&rho;</em><sup>2</sup>"
  ),
  digits = 3,
  caption = paste0(
  "<strong>Table 1.</strong> Comparison of demographic properties for the original ",
  "stage-structured matrix population model of <em>Rourea induta</em> ",
  "(COMPADRE MatrixID 246781) and four aggregated versions. The original 14 stages ",
  "are collapsed to 5 while leaving the Seedling and Sucker stages unchanged. ",
  "&ldquo;Standard&rdquo; denotes use of a standard aggregator and ",
  "&ldquo;elasticity&rdquo; denotes elasticity-consistent aggregation. ",
  "<em>T</em><sub>a</sub> is generation time (years) in the <em>&lambda;</em> framework and ",
  "<em>T</em><sub>c</sub> is cohort generation time (years) in the ",
  "<em>R</em><sub>0</sub> framework. ",
  "<em>&rho;</em><sup>2</sup> quantifies aggregation effectiveness, with values closer to 1 ",
  "indicating closer agreement between the aggregated and reference models."
  ),
  format = "html",
  escape = FALSE
) |>
  kable_styling(full_width = TRUE) |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  row_spec(nrow(results), extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> The projection interval associated with the population ",
      "growth rate (<em>&lambda;</em>) is 1 year.</span>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )
```


**How to read Table 1.** The first row reports the true demographic parameters of 
the original matrix population model for *Rourea induta* (Hoffmann 1999), while the 
remaining rows report estimates obtained from aggregated versions of the MPM. 
Generation time is preserved under “Agg λ + elasticity,” and cohort generation time 
is preserved under “Agg R₀ + elasticity.” All aggregated models achieve near-perfect 
aggregation, with effectiveness values exceeding 0.99.

### Interpretation notes

- Aggregation under the \(\lambda\) framework prioritizes preservation of long-term population growth.
- Aggregation under the \(R_0\) framework prioritizes preservation net reproductive rate.

## Elasticities of \(\lambda\): making them comparable across models

To make the elasticities of the original model comparable with those of the
aggregated models, we transform the original elasticity matrix to the
aggregated stage structure as

\[
\mathbf{E}^{\mathrm{true}}_{\mathrm{agg}}
=
\mathbf{P}\, \mathbf{E}\!\bigl(\mathbf{A}\bigr)\, \mathbf{P}^{\top},
\]

where \( \mathbf{E}\!\bigl(\mathbf{A}\bigr) \) is the elasticity of
\( \lambda \) with respect to the entries of the original projection
matrix \( \mathbf{A} \), and \( \mathbf{P} \) is the partitioning matrix
induced by `groups`. Elasticities for each aggregated model are then
computed directly from its aggregated population projection matrix, and the results
for all models are plotted together.

```{r elasticities}

# Partitioning matrix induced by the stage groups
# This matrix is constructed internally by mpm_aggregate(),
# but we reproduce it here because it can be used to map quantities
# from the original stage space to the aggregated space.
P <- mpm_partition(groups=groups, n=nrow(matA))

# Elasticity matrices in aggregated space (k x k) for each model
E_A <- mpm_elasticity(matA=matA,framework="lambda")$elasticity
E_list <- list(
  "Original" = P %*% E_A %*% t(P),
  "Agg lambda + standard" = mpm_elasticity(matA=m1$A,framework="lambda")$elasticity,
  "Agg lambda + elasticity" = mpm_elasticity(matA=m2$A,framework="lambda")$elasticity,
  "Agg R0 + standard" = mpm_elasticity(matA=m3$A,framework="lambda")$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matA=m4$A,framework="lambda")$elasticity
)

# Build a long data.frame for all nonzero entries (row-major order)
to_long <- function(M, model_name) {
  idx <- which(M != 0, arr.ind = TRUE)
  data.frame(
    model = model_name,
    row = idx[, 1],
    col = idx[, 2],
    entry = paste(idx[, 1], idx[, 2], sep = ","),
    elasticity = M[idx],
    stringsAsFactors = FALSE
  )
}

elast_df <- do.call(rbind, Map(to_long, E_list, names(E_list)))

# Keep positive values for log scale
elast_df <- elast_df[elast_df$elasticity > 0, ]

# Row-major ordering: row 1 entries first, row 2 second, rows 3+ third, etc.
elast_df <- elast_df[order(elast_df$row, elast_df$col), ]

models_order <- names(E_list)
entries <- unique(elast_df$entry)

# Create matrix for barplot: rows = models (5), cols = entries
elast_mat <- sapply(models_order, function(m) {
  elast_df$elasticity[elast_df$model == m]
})
elast_mat <- t(elast_mat)
rownames(elast_mat) <- models_order
colnames(elast_mat) <- entries
```

### Plot elasticities of \(\lambda\) by matrix entry

Each x-axis category corresponds to a matrix entry \([i,j]\) that is nonzero in at least 
one of the five populationb projection matrices. Within each category, elasticities are shown for 
the original model and the four aggregated models.

**Figure 1. Elasticities of \(\lambda\) with respect to matrix entries.**  
Bars are grouped by matrix entry, with colors indicating the different five models. Because 
elasticities sum to 1 within each matrix, the figure illustrates how the influence on 
\(\lambda\) is distributed across life-history components. The models shown in the 
legend correspond to those described in Table 1.


```{r lambda-elasticity-plot, fig.width=12}
# Build biologically meaningful entry labels
# Build (row, col) vectors in the exact column order of elast_mat
rc <- do.call(rbind, strsplit(colnames(elast_mat), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

make_entry_label <- function(r, c) {
  if (r == 1) {
    bquote(F[1, .(c)])
  } else if (r == 2) {
    bquote(C[2, .(c)])
  } else {
    bquote(U[.(r), .(c)])
  }
}

entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)


# Colors by model
# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Leave room for left legend and long x labels
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat,
  beside = TRUE,
  log = "y",
  col = cols,
  border = NA,
  ylab = expression("Elasticity of " * lambda * " (log scale)"),
  xaxt = "n",
  space = c(0.2, 1)   # ← this is the key line
)

axis(
  side = 1,
  at = colMeans(bp),
  labels = entry_labels,
  las = 2,
  cex.axis = 0.9
)

legend(
  "topleft",
  legend = c(
    "Original",
    expression(paste("Agg ", lambda, " + standard")),
    expression(paste("Agg ", lambda, " + elasticity")),
    expression(paste("Agg ", R[0], " + standard")),
    expression(paste("Agg ", R[0], " + elasticity"))
  ),
  fill = cols,
  bty = "n",
  inset = 0.02
)

par(op)
```

**How to read Figure 1.** Each x-axis label corresponds to a vital rate represented by 
a matrix entry \([i,j]\). Fertility rates \(F[i,j]\) describe sexual reproduction, 
representing the contribution of individuals in stage \(i\) to seedling production 
over the projection interval. Clonal reproduction rates \(C[i,j]\) represent the 
contribution of individuals in stage \(i\) to the production of suckers over the 
projection interval. Survival and transition probabilities \(U[i,j]\) give the probability 
that an individual in stage \(i\) survives and transitions to stage \(j\) over the 
projection interval.


## Elasticities of \(R_0\): making them comparable across models

To make the elasticities of the original model comparable with those of the
aggregated models, we transform the original elasticity matrix to the
aggregated stage structure as

\[
\mathbf{E}^{\mathrm{true}}_{\mathrm{agg}}
=
\mathbf{P}\, \mathbf{E}\!\bigl(\mathbf{A}\bigr)\, \mathbf{P}^{\top},
\]

where \( \mathbf{E}\!\bigl(\mathbf{A}\bigr) \) is the elasticity of
\( R_0 \) with respect to the entries of the original projection
matrix \( \mathbf{A} \), and \( \mathbf{P} \) is the partitioning matrix
induced by `groups`. Elasticities for each aggregated model are then
computed directly from its aggregated population projection matrix, and the results
for all models are plotted together.

```{r elasticities of R0}
# Partitioning matrix induced by the stage group
# Reintroduced here so this section can be read independently
P <- mpm_partition(groups=groups, n=nrow(matA))

# Elasticity matrices in aggregated space (k x k) for each model
# The elasticity matrix of the original model is presented in its aggregated
# form using the partitioning matrix P. This is the true aggregated form of the elasticity matrix
# to which elasticities derived from aggregated models are compared.
E_A <- mpm_elasticity(matF=matF,matU=matU,framework="R0", normalize=TRUE)$elasticity
E_list <- list(
  "Original" = P %*% E_A %*% t(P),
  "Agg lambda + standard" = mpm_elasticity(matF=m1$F, matU=m1$U,framework="R0", normalize=TRUE)$elasticity,
  "Agg lambda + elasticity" = mpm_elasticity(matF=m2$F, matU=m2$U,framework="R0", normalize=TRUE)$elasticity,
  "Agg R0 + standard" = mpm_elasticity(matF=m3$F, matU=m3$U,framework="R0", normalize=TRUE)$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matF=m4$F, matU=m4$U,framework="R0", normalize=TRUE)$elasticity
)

# Build a long data.frame for all nonzero entries (row-major order)
to_long <- function(M, model_name) {
  idx <- which(M != 0, arr.ind = TRUE)
  data.frame(
    model = model_name,
    row = idx[, 1],
    col = idx[, 2],
    entry = paste(idx[, 1], idx[, 2], sep = ","),
    elasticity = M[idx],
    stringsAsFactors = FALSE
  )
}

elast_df <- do.call(rbind, Map(to_long, E_list, names(E_list)))

# Keep positive values for log scale
elast_df <- elast_df[elast_df$elasticity > 0, ]

# Row-major ordering: row 1 entries first, row 2 second, rows 3+ third, etc.
elast_df <- elast_df[order(elast_df$row, elast_df$col), ]

models_order <- names(E_list)
entries <- unique(elast_df$entry)

# Create matrix for barplot: rows = models (5), cols = entries
elast_mat2 <- sapply(models_order, function(m) {
  elast_df$elasticity[elast_df$model == m]
})
elast_mat2 <- t(elast_mat2)
rownames(elast_mat2) <- models_order
colnames(elast_mat2) <- entries
```

### Plot elasticities of \(R_0\) by matrix entry

Each x-axis category corresponds to a matrix entry \([i,j]\) that is nonzero in at least 
one of the five projection matrices. Within each category, elasticities are shown for 
the original model and the four aggregated models.

**Figure 2. Elasticities of \(R_0\) with respect to matrix entries.**  
Bars are grouped by matrix entry, with colors indicating the five different models. Elasticities 
are normalized to sum to 1 within each matrix, so the figure illustrates how the 
contribution to \(R_0\) is distributed across life-history components. The models shown 
in the legend correspond to those described in Table 1.


```{r elasticity-R0-plot, fig.width=12, fig.height=6}
# Build biologically meaningful entry labels
# Build (row, col) vectors in the exact column order of elast_mat
rc <- do.call(rbind, strsplit(colnames(elast_mat2), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

make_entry_label <- function(r, c) {
  if (r == 1) {
    bquote(F[1, .(c)])
  } else if (r == 2) {
    bquote(C[2, .(c)])
  } else {
    bquote(U[.(r), .(c)])
  }
}

entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)


# Colors by model
# Same colors as the vital-rate plot for consistency
cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")

# Leave room for left legend and long x labels
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat2,
  beside = TRUE,
  log = "y",
  col = cols,
  border = NA,
  ylab = expression("Normalized elasticity of " * R[0] * " (log scale)"),
  xaxt = "n",
  space = c(0.2, 1)   # ← this is the key line
)

axis(
  side = 1,
  at = colMeans(bp),
  labels = entry_labels,
  las = 2,
  cex.axis = 0.9
)

legend(
  "topleft",
  legend = c(
    "Original",
    expression(paste("Agg ", lambda, " + standard")),
    expression(paste("Agg ", lambda, " + elasticity")),
    expression(paste("Agg ", R[0], " + standard")),
    expression(paste("Agg ", R[0], " + elasticity"))
  ),
  fill = cols,
  bty = "n",
  inset = 0.02
)

par(op)
```
**How to read Figure 2.** Each x-axis label corresponds to a vital rate represented by 
a matrix entry \([i,j]\). Fertility rates \(F[i,j]\) describe sexual reproduction, 
representing the contribution of individuals in stage \(i\) to seedling production 
over the projection interval. Clonal reproduction rates \(C[i,j]\) represent the 
contribution of individuals in stage \(i\) to the production of suckers over the 
projection interval. Survival and transition probabilities \(U[i,j]\) give the probability 
that an individual in stage \(i\) survives and transitions to stage \(j\) over the 
projection interval.

Across both Figures 1 and 2, the majority of elasticity is concentrated in a small 
number of vital rates. In particular, three survival terms—\(U[3,3]\), \(U[4,4]\), 
and \(U[5,5]\)—account for most of the total elasticity. These entries 
correspond to stasis within the larger size classes and indicate that population 
performance, whether measured by \(\lambda\) or \(R_0\), is driven primarily by 
persistence in these stages. This pattern is consistent across the original and 
aggregated models, suggesting that aggregation preserves the dominant contributors 
to asymptotic population dynamics.

## Comparing elasticities of \(\lambda\) and \(R_0\)
There is remarkable consistency between the elasticities in the \(\lambda\) and \(R_0\) 
frameworks. In both cases, the same three matrix entries account for the majority of elasticity 
mass, indicating strong agreement in the demographic processes identified as most 
influential. These shared entries account for 96% of the total elasticity mass in the 
\(\lambda\) framework and 94% in the \(R_0\) framework. Elasticities in the \(R_0\) 
framework are normalized to sum to 1, whereas elasticities in the \(\lambda\) framework 
require no such normalization because they already sum to 1 by definition.

```{r lambda-vs.-R0-elasticities}
#compare elasticities of top 3 vital rates
#top three elasticities in lambda framework
sum(elast_mat[,c(9,13,15)])/5
#top three elasticities in R0 framework
sum(elast_mat2[,c(9,13,15)])/5
```
## Summary

Across aggregation methods, estimates of \(\lambda\) and \(R_0\) in Table 1 are broadly 
similar, indicating that aggregation preserves overall conclusions about population 
growth. Differences among methods are small and largely reflect whether aggregation 
prioritizes long-term growth (\(\lambda\)) or lifetime reproduction (\(R_0\)). In 
contrast, generation time estimates clearly differ between frameworks: generation 
time in the \(\lambda\) framework (\(T_a\)) and cohort generation time in the \(R_0\) 
framework (\(T_c\)) reflect fundamentally different perspectives on the pace of life.

Particularly striking is that \(\lambda\) is close to 1 (within 1%), yet 
\(R_0\) is not. If \(\lambda\) were exactly equal to 1, then \(R_0\) would also necessarily 
equal 1, and all corresponding demographic quantities would coincide across the two frameworks. 
That the demographic quantities differ greatly here, reveals how sensitive demographic 
parameters can sometimes be to even small deviations of the stable growth rate around 
unity.

Despite these differences in scalar demographic quantities, elasticity patterns are 
remarkably consistent across frameworks. Elasticities of \(\lambda\) and \(R_0\) identify 
the same matrix entries as dominant contributors, with most of the elasticity 
mass shared between them. This consistency suggests that, in this example, the 
demographic processes driving stable population growth and net reproductive rate 
are similar, even when different aggregation objectives are used.

## References

Bienvenu, F., Akçay, E., Legendre, S., and McCandlish, D. M. (2017). The genealogical
decomposition of a matrix population model with applications to the aggregation
of stages. *Theoretical Population Biology*, 115, 69–80. 
https://doi.org/10.1016/j.tpb.2017.04.002

Hoffmann, W. A. (1999). Fire and population dynamics of woody plants in a neotropical
savanna: matrix model projections. *Ecology*, 80(4), 1354–1369.
https://doi.org/10.1890/0012-9658(1999)080[1354:FAPDOW]2.0.CO;2

Hooley, D. E. (2000). Collapsed matrices with (almost) the same eigenstuff.
*The College Mathematics Journal*, 31(4), 297–299.
https://doi.org/10.1080/07468342.2000.11974162

Jones, O. R., Barks, P., Stott, I., James, T. D., Levin, S., Petry, W. K., Capdevila, P.,
Che-Castaldo, J., Jackson, J., Römer, G., Schuette, C., Thomas, C. C., and
Salguero-Gómez, R. (2022). Rcompadre and Rage—Two R packages to facilitate the
use of the COMPADRE and COMADRE databases and calculation of life-history traits
from matrix population models. *Methods in Ecology and Evolution*, 13(4), 770–781.
https://doi.org/10.1111/2041-210X.13792

Salguero-Gómez, R., and Plotkin, J. B. (2010). Matrix dimensions bias demographic
inferences: implications for comparative plant demography. *The American Naturalist*,
176(6), 710–722.
https://doi.org/10.1086/657044

Salguero-Gómez, R., Jones, O. R., Archer, C. R., Buckley, Y. M., Che-Castaldo, J.,
Caswell, H., Hodgson, D., Scheuerlein, A., Conde, D. A., Brinks, E., de Buhr, H.,
Farack, C., Gottschalk, F., Hartmann, A., Henning, A., Hoppe, G., Römer, G.,
Runge, J., Ruoff, T., Wille, J., Zeh, S., Davison, R., Vieregg, D., Baudisch, A.,
Altwegg, R., Colchero, F., Dong, M., de Kroon, H., Lebreton, J.-D., Metcalf, C. J. E.,
Neel, M. M., Parker, I. M., Takada, T., Valverde, T., Vélez-Espino, L. A., Wardle,
G. M., Franco, M., and Vaupel, J. W. (2015). The COMPADRE Plant Matrix Database:
an open online repository for plant demography. *Journal of Ecology*, 103, 202–218.
https://doi.org/10.1111/1365-2745.12334

---
title: "Aggregating a Leslie-to-Leslie MPM"
author:
  - name: "Richard A. Hinrichsen"
    orcid: "0000-0003-0761-3005"
output:
  rmarkdown::html_vignette:
    number_sections: false
    mathjax: default
vignette: >
  %\VignetteIndexEntry{Aggregating a Leslie-to-Leslie MPM}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

**ORCID:** <https://orcid.org/0000-0003-0761-3005>

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Overview

Matrix population models (MPMs) are constructed at different age or stage resolutions, 
which complicates demographic comparisons across populations. For age-structured populations, 
such complications arise when a Leslie matrix model (MPM1) with a small age-class 
width is compared with a coarser Leslie matrix model (MPM2). In such a case, aggregation 
can be used to group fine age classes of MPM1 into a smaller number of wider age classes 
that are compatible with MPM2, thus allowing direct comparison of the demographic 
features. Aggregation must proceed carefully because certain demographic parameters 
are known to change with aggregation.

In this vignette, we demonstrate Leslie-to-Leslie MPM aggregation using the 
`mpmaggregate` package. Leslie-to-Leslie MPM aggregation proceeds by reducing the number of
age classes of the Leslie MPM while retaining its Leslie form. Unlike general-to-general 
MPM aggregation, Leslie-to-Leslie MPM aggregation changes the projection interval to ensure that 
it is equal to the age-class width.

As a worked example, we use an age-structured matrix population model for the
yellow-bellied marmot *Marmota flaviventris* obtained from the COMADRE database
(Jones et al. 2022), which archives demographic data originally published by
Ozgul et al. (2009). We focus on aggregation using `leslie_aggregate()`, and we compare
four combinations of demographic framework and aggregation criterion, highlighting what
each approach preserves and how the aggregated models differ in interpretation.

## Loading packages

We begin by loading the `mpmaggregate` package, which provides the functions used 
throughout this vignette for aggregating matrix population models under the \(\lambda\) 
and \(R_0\) demographic frameworks. The package `expm` is used to raise a matrix
to a power, `knitr` is used for report generation, and `kableExtra` is used for
table formatting.

```{r packages}
library(mpmaggregate)
library(kableExtra)
library(rphylopic)
library(expm)
```

## Data acquisition

To demonstrate Leslie-to-Leslie MPM aggregation, we use an age-structured MPM for
*Marmota flaviventris* (MatrixID **249455**) from the COMADRE database
(Ozgul et al. 2009). We aggregate this Leslie MPM with 10 age classes to a Leslie MPM
with 5 age classes. Age classes 1-2 are combined to form a new age class 1, age classes 3-4 
are combined to form a new age class 2, etc..

The yellow-bellied marmot *Marmota flaviventris* is a large, stout-bodied ground squirrel
(rodent) native to the mountainous regions of western North America.

```{r}
# Leslie population projection matrix for Marmota flaviventris
# MatrixID = 249455

# Not run: requires Rcompadre and internet access

# Matrices can be retrieved with:
# library(Rcompadre)
# comadre <- cdb_fetch("comadre",version = "4.23.3.1")
# mpm <- comadre[comadre$MatrixID == 249455, ]
# matA <- matA(mpm)[[1]]
# matU <- matU(mpm)[[1]]
# matF <- matF(mpm)[[1]]


# Matrices are defined locally to avoid requiring the use of the internet and Rcompadre

# Leslie population projection matrix
matA <- matrix(c(
  0.000000, 0.258040, 0.627254, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037,
  0.545125, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.498625, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.592875, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000, 0.000000,
  0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.643500, 0.000000
), nrow = 10, byrow = TRUE)


# Reproduction matrix
matF <- matrix(c(
  0, 0.25804, 0.627254, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037, 0.700037,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0,
  0, 0,       0,        0,        0,        0,        0,        0,        0,        0
), nrow = 10, byrow = TRUE)

# Survival/transition matrix
matU <- matrix(c(
  0,       0,       0,       0,      0,      0,      0,      0,      0,      0,
  0.545125,0,       0,       0,      0,      0,      0,      0,      0,      0,
  0,       0.498625,0,       0,      0,      0,      0,      0,      0,      0,
  0,       0,       0.592875,0,      0,      0,      0,      0,      0,      0,
  0,       0,       0,       0.6435, 0,      0,      0,      0,      0,      0,
  0,       0,       0,       0,      0.6435, 0,      0,      0,      0,      0,
  0,       0,       0,       0,      0,      0.6435, 0,      0,      0,      0,
  0,       0,       0,       0,      0,      0,      0.6435, 0,      0,      0,
  0,       0,       0,       0,      0,      0,      0,      0.6435, 0,      0,
  0,       0,       0,       0,      0,      0,      0,      0,      0.6435, 0
), nrow = 10, byrow = TRUE)

# Sanity check: A = U + F (after folding any C into F)
stopifnot(all.equal(unname(matA), unname(matU + matF)))
   
```

## Helper functions for demographic quantities

We compute:

- \( \lambda \): spectral radius of \( \mathbf{A} \)
- \( R_0 \): spectral radius of \( \mathbf{F} \left( \mathbf{I} - \mathbf{U} \right)^{-1} \)
- \( T_a \): generation time (years) under the \( \lambda \) framework
- \( T_c \): cohort generation time (years) under the \( R_0 \) framework

```{r helpers}
`%||%` <- function(x, y) if (!is.null(x)) x else y

# Return R0, the net reproductive rate
get_R0 <- function(U, F) {
  I <- diag(nrow(U))
  N <- solve(I - U)
  K <- F %*% N
  spectral_radius(K)
}

# Return generation time
get_Ta <- function(U, F) {
  generation_time(F, U, framework = "lambda")$generation_time
}

# Return cohort generation time
get_Tc <- function(U, F) {
  generation_time(F, U, framework = "R0")$generation_time
}
```

## Aggregate in four ways

We aggregate the model using the following four combinations of framework and criterion:

1. `framework = "lambda"`, `criterion = "standard"`
2. `framework = "lambda"`, `criterion = "elasticity"`
3. `framework = "R0"`, `criterion = "standard"`
4. `framework = "R0"`, `criterion = "elasticity"`

In the \(\lambda\) framework, standard aggregation follows the method of Hooley (2000),
while elasticity-consistent aggregation follows Bienvenu et al. (2017).
These methods are extended to Leslie-to-Leslie aggregation (Hinrichsen, 2023; 
Hinrichsen et al., 2026). Both approaches maintain  consistency of the asymptotic 
growth rate \(\lambda\) and the stable age distribution. Elasticity-consistent aggregation 
additionally maintains consistency of the reproductive values and the elasticities 
of \(\lambda^k\) with respect to entries of \( \mathbf{A}^k \), where \(k = n/m\).

We extend these Leslie-to-Leslie approaches to the \(R_0\) framework, where standard 
aggregation maintains the consistency of the net reproductive rate \(R_0\) and cohort 
stable age distribution. Elasticity-consistent aggregation in this framework 
additionally maintains the consistency of the cohort reproductive values and 
elasticities of \(R_0\) with respect to matrix entries of \( \mathbf{A}_k \),
which is a k-step-ahead version of \( \mathbf{A}\) in the \(R_0\) framework.

```{r aggregate}
# Aggregate into 5 x 5 matrices (m = 5)
ngroups <- 5

agg1 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "lambda", criterion = "standard"
)

agg2 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "lambda", criterion = "elasticity"
)

agg3 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "R0", criterion = "standard"
)

agg4 <- leslie_aggregate(
  matA = matA, ngroups = ngroups,
  framework = "R0", criterion = "elasticity"
)

# Split a Leslie matrix into U and F components (Leslie form: fertility in row 1)
split_matA <- function(A) {
  n <- nrow(A)
  U <- A
  U[1, ] <- 0
  F <- matrix(0, n, n)
  F[1, ] <- A[1, ]
  list(U = U, F = F, A = A)
}

m0 <- list(U = matU, F = matF, A = matA)
m1 <- split_matA(agg1$matAk_agg)
m2 <- split_matA(agg2$matAk_agg)
m3 <- split_matA(agg3$matAk_agg)
m4 <- split_matA(agg4$matAk_agg)

eff <- c(
  NA,
  agg1$effectiveness,
  agg2$effectiveness,
  agg3$effectiveness,
  agg4$effectiveness
)

models <- list(
  Original            = m0,
  Agg_lambda_std      = m1,
  Agg_lambda_elast    = m2,
  Agg_R0_std          = m3,
  Agg_R0_elast        = m4
)

model_labels <- c(
  Original         = "Original",
  Agg_lambda_std   = "Agg &lambda; + standard",
  Agg_lambda_elast = "Agg &lambda; + elasticity",
  Agg_R0_std       = "Agg R<sub>0</sub> + standard",
  Agg_R0_elast     = "Agg R<sub>0</sub> + elasticity"
)

# Basic checks
stopifnot(all(sapply(models, function(m) !is.null(m$A))))

# Show aggregated matrices
print("Agg λ + standard");   m1$A
print("Agg λ + elasticity"); m2$A
print("Agg R0 + standard");  m3$A
print("Agg R0 + elasticity");m4$A
```

**Important.** Unlike general-to-general MPM aggregation, Leslie-to-Leslie MPM aggregation 
cannot produce survival probabilities greater than 1. Furthermore, within each framework, 
the survival probabilities are the same under standard and elasticity-consistent 
aggregation, while the fertility schedules can differ between the standard and 
elasticity-consistent aggregated Leslie matrices (Hinrichsen et al., 2026).

## Compare \(\lambda\), \(R_0\), \(T_a\), and \(T_c\) in a table

```{r results-table alt}
# Adjust lambdas so that they are all per capita growth rate over a year
# Adjust generation times so they all use units of 1 year
adj = c(1,2,2,2,2)

lambda = sapply(models, function(m) spectral_radius(m$A))
lambda = lambda^(1/adj)
R0     = sapply(models, function(m) get_R0(m$U, m$F))
Ta     = sapply(models, function(m) get_Ta(m$U, m$F))*adj
Tc     = sapply(models, function(m) get_Tc(m$U, m$F))*adj

results <- data.frame(
  Model  = unname(model_labels[names(models)]),
  lambda = lambda,
  R0     = R0,
  Ta     = Ta,
  Tc     = Tc,
  Effectiveness = eff,
  row.names = NULL,
  stringsAsFactors = FALSE
)

n_rows <- nrow(results)

knitr::kable(
  results,
  col.names = c(
    "Model",
    "<em>&lambda;</em>",
    "<em>R</em><sub>0</sub>",
    "<em>T</em><sub>a</sub> (years)",
    "<em>T</em><sub>c</sub> (years)",
    "<em>&rho;</em><sup>2</sup>"
  ),
  digits = 3,
  caption = paste0(
    "<strong>Table 1.</strong> Comparison of demographic properties for the ",
    "yellow-bellied marmot <em>Marmota flaviventris</em> (COMADRE MatrixID 249455) ",
    "using the original 10 × 10 Leslie matrix and four aggregated Leslie matrices ",
    "collapsed to 5 age groups. Metrics include population growth rate ",
    "(<em>&lambda;</em>), net reproductive rate (<em>R</em><sub>0</sub>), ",
    "generation time (years; <em>T</em><sub>a</sub>), cohort generation time ",
    "(years; <em>T</em><sub>c</sub>), and aggregation effectiveness ",
    "(<em>&rho;</em><sup>2</sup>). &ldquo;Standard&rdquo; denotes use of a standard aggregator ",
    "and &ldquo;elasticity&rdquo; denotes elasticity-consistent aggregation."
  ),
  format = "html",
  escape = FALSE
) |>
  kable_styling(full_width = TRUE) |>
  row_spec(0, extra_css = "border-bottom: 2px solid black;") |>
  row_spec(nrow(results), extra_css = "border-bottom: 2px solid black;") |>
  footnote(
    general_title = "",
    general = paste0(
      "<span style='font-size: 90%;'>",
      "<strong>Note.</strong> Population growth rates (<em>&lambda;</em>) are all transformed to ",
      "represent growth over 1 year. Generation times have all been transformed to ",
      "represent generation time measured in years.</span>"
    ),
    footnote_as_chunk = TRUE,
    escape = FALSE
  )
```

**How to read Table 1.** The first row reports demographic quantities computed from the
original Leslie matrix for *Marmota flaviventris* (Ozgul et al. 2009), while the remaining
rows report estimates obtained from aggregated Leslie matrices.

### Interpretation notes

- Aggregation under the **\(\lambda\) framework** prioritizes consistency of stable population growth rate.
- Aggregation under the **\(R_0\) framework** prioritizes consistency of net reproductive rate.


## Elasticities of \(\lambda\): making them comparable across models

To make the elasticities of the original model comparable with those of the
aggregated models, we map the original elasticity matrix to the
aggregated age structure as

\[
\mathbf{E}^{\mathrm{true}}_{\mathrm{agg}}
=
\mathbf{P}\, \mathbf{E}\!\bigl(\mathbf{A}^{k}\bigr)\, \mathbf{P}^{\top},
\]

where \( \mathbf{E}\!\bigl(\mathbf{A}^{k}\bigr) \) is the elasticity of
\( \lambda^{k} \) with respect to the entries of the original
\( \mathbf{A}^{k} \), and \( \mathbf{P} \) is the partitioning matrix
induced by `groups`. Elasticities for each aggregated model are then
computed directly from its aggregated population projection matrix, and the results
for all models are plotted together.

```{r elasticities-lambda^k}

# Age groups used for aggregation in example
groups <- list(c(1,2), c(3,4), c(5,6), c(7,8), c(9,10))

# Partitioning matrix induced by the age groups.
# This matrix is constructed internally by leslie_aggregate(),
# but we construct it explicitly here so it can be used to
# map quantities from the original age-class space to the
# aggregated space.
P <- mpm_partition(groups = groups, n = nrow(matA))

# Dimensions of the original and aggregated matrices
m <- ngroups
n <- nrow(matA)
k <- n / m

matAk <- matA %^% k

# Elasticity of lambda^k with respect to entries of A^k
E_A <- mpm_elasticity(matA = matAk, framework = "lambda")$elasticity

E_list <- list(
  "Original"            = P %*% E_A %*% t(P),
  "Agg λ + standard"    = mpm_elasticity(matA = m1$A, framework = "lambda")$elasticity,
  "Agg λ + elasticity"  = mpm_elasticity(matA = m2$A, framework = "lambda")$elasticity,
  "Agg R0 + standard"   = mpm_elasticity(matA = m3$A, framework = "lambda")$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matA = m4$A, framework = "lambda")$elasticity
)

to_long <- function(M, model_name) {
  idx <- which(M != 0, arr.ind = TRUE)
  data.frame(
    model = model_name,
    row = idx[, 1],
    col = idx[, 2],
    entry = paste(idx[, 1], idx[, 2], sep = ","),
    elasticity = M[idx],
    stringsAsFactors = FALSE
  )
}

elast_df <- do.call(rbind, Map(to_long, E_list, names(E_list)))
elast_df <- elast_df[elast_df$elasticity > 0, ]
elast_df <- elast_df[order(elast_df$row, elast_df$col), ]

models_order <- names(E_list)
entries <- unique(elast_df$entry)

elast_mat <- sapply(models_order, function(m) elast_df$elasticity[elast_df$model == m])
elast_mat <- t(elast_mat)
rownames(elast_mat) <- models_order
colnames(elast_mat) <- entries
```

### Plot: elasticities of \( \lambda^k \) by matrix entry

Each x-axis category corresponds to a matrix entry \([i,j]\) that is nonzero in at least one of
the five \(A\) matrices. Within each category, we plot elasticities for the original
(“true aggregated”) model and the four aggregated models.

**Figure 1. Elasticities of \( \lambda^k \) with respect to matrix entries.**  
Elasticities quantify the proportional change in \( \lambda^k \) resulting from proportional
changes in entries of the entries of \( \mathbf{A}^{k} \) . Bars are grouped by matrix entry, 
with colors indicating the five models. Because elasticities sum to 1 within each matrix, 
the figure illustrates how the influence on \( \lambda^k \) is distributed across 
life-history components. The models shown in the legend correspond to those described in Table 1.

```{r elasticity-plot, fig.width=12, fig.height=6}
# Not run: requires internet access

# Marmot 2012 Feb 2 by T. Michael Keesey, CC0 1.0, Marmota monax (Linnaeus 1758)
# uuid <- "eee50efb-40dc-47d0-b2cb-52a14a5e0e51"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 869)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/marmot.png")

# Load image locally
img_file <- system.file("extdata", "marmot.png", package = "mpmaggregate")

if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/marmot.png"
}

img <- png::readPNG(img_file)

rc <- do.call(rbind, strsplit(colnames(elast_mat), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

make_entry_label <- function(r, c) {
  if (r == 1) {
    bquote(F[1, .(c)])
  } else {
    bquote(U[.(r), .(c)])
  }
}
entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)

cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat,
  beside = TRUE,
  ylim = c(0, .35),
  col = cols,
  border = NA,
  ylab = expression("Elasticity of " * lambda^k),
  xaxt = "n",
  space = c(0.2, 1)
)

axis(side = 1, at = colMeans(bp), labels = entry_labels, las = 2, cex.axis = 0.9)
legend("topleft", legend = rownames(elast_mat), fill = cols, bty = "n", inset = 0.02)

lims <- par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x = NULL,
  y = mydiff * .9,
  height = .2 * mydiff * dim(img)[1] / 1024
)

par(op)
```

**How to read Figure 1.** Each x-axis label corresponds to a vital rate represented 
by a matrix entry in the aggregated space. Fertility rate, \(F[1,i]\) denotes 
reproduction of age class \(i\) over the projection interval. Survival probability, 
\(U[i+1,i]\) gives  the probability of surviving and transitioning from age class 
\(i\) to age class  \(i+1\) over the projection interval. There is exact agreement 
between the elasticities  of models “Original” and “Agg \(\lambda\) + elasticity,” 
which is by design. The silhouette is from PhyloPic (https://www.phylopic.org; 
Keesey, 2023) and were added  using the rphylopic R package (Gearty & Jones, 2023). 
The silhouette was contributed as follows:  *Marmota monax* by  T. Michael Keesey 
(2012; CC0 1.0).

## Elasticities of \(R_0\): making them comparable across models

To make the elasticities of the original model comparable with those of the
aggregated models, we map the original elasticity matrix to the
aggregated age structure as

\[
\mathbf{E}^{\mathrm{true}}_{\mathrm{agg}}
=
\mathbf{P}\, \mathbf{E}\!\left(\mathbf{A}_{k}\right)\, \mathbf{P}^{\top},
\]

where \( \mathbf{E}\!\left(\mathbf{A}_{k}\right) \) is the normalized elasticity of
\( R_0 \) with respect to the entries of the \( k \)-step-ahead population projection matrix
\( \mathbf{A}_{k} \) in the \( R_0 \) framework, and \( \mathbf{P} \) is the partitioning
matrix induced by `groups`. Elasticities for each aggregated model are then
computed directly from its corresponding \( \mathbf{U}_{k} \) and
\( \mathbf{F}_{k} \), and the results for all models are plotted together.

```{r elasticities-r0}
# Age groupings used for aggregation (reintroduced here so this section
# can be read independently)
groups <- list(c(1,2), c(3,4), c(5,6), c(7,8), c(9,10))

# Partitioning matrix induced by the age groups.
# Constructed internally by leslie_aggregate(), but reproduced here
# because it is used to map quantities to aggregated space.
P <- mpm_partition(groups = groups, n = nrow(matA))

# Dimensions of the original and aggregated matrices
# Reintroduced here for clarity
m <- ngroups
n <- nrow(matA)
k <- n / m

R0 <- spectral_radius(matF %*% solve(diag(n) - matU))

# Reference operator for aggregation eigenstructure in the R0 framework
# Divide by R0 so that it has Leslie matrix form
A_R0ref <- (matF + R0 * matU) / R0

Uk <- expm::`%^%`(matU, k)
Rk <- (expm::`%^%`(A_R0ref, k) - Uk) * R0
Ak <- Uk + Rk

# Elasticity of R0 with respect to entries of Ak (closest to what aggregated matrices yield)
E_A <- mpm_elasticity(matF = Rk, matU = Uk, framework = "R0", normalize = TRUE)$elasticity

E_list2 <- list(
  "Original"            = P %*% E_A %*% t(P),
  "Agg λ + standard"    = mpm_elasticity(matF = m1$F, matU = m1$U, framework = "R0", normalize = TRUE)$elasticity,
  "Agg λ + elasticity"  = mpm_elasticity(matF = m2$F, matU = m2$U, framework = "R0", normalize = TRUE)$elasticity,
  "Agg R0 + standard"   = mpm_elasticity(matF = m3$F, matU = m3$U, framework = "R0", normalize = TRUE)$elasticity,
  "Agg R0 + elasticity" = mpm_elasticity(matF = m4$F, matU = m4$U, framework = "R0", normalize = TRUE)$elasticity
)

elast_df2 <- do.call(rbind, Map(to_long, E_list2, names(E_list2)))
elast_df2 <- elast_df2[elast_df2$elasticity > 0, ]
elast_df2 <- elast_df2[order(elast_df2$row, elast_df2$col), ]

models_order2 <- names(E_list2)
entries2 <- unique(elast_df2$entry)

elast_mat2 <- sapply(models_order2, function(m) elast_df2$elasticity[elast_df2$model == m])
elast_mat2 <- t(elast_mat2)
rownames(elast_mat2) <- models_order2
colnames(elast_mat2) <- entries2
```

### Plot: elasticities of \(R_0\) by matrix entry

Each x-axis category corresponds to a matrix entry \([i,j]\) that is nonzero in at least one of
the five matrices. Within each category, we plot elasticities for the original
(“true aggregated”) model and the four aggregated models.

**Figure 2. Elasticities of \(R_0\) with respect to matrix entries.**  
Elasticities quantify the proportional change in \(R_0\) resulting from proportional changes in
individual vital rates. Bars are grouped by matrix entry, with colors indicating the five models.
The elasticities of \(R_0\) are normalized to sum to 1, so each elasticity can be interpreted as
the fraction of the total contribution to \(R_0\) attributable to that matrix entry.

```{r elasticity-r0-plot, fig.width=12, fig.height=6}

# Not run: requires internet access

# Marmot 2012 Feb 2 by T. Michael Keesey, CC0 1.0, Marmota monax (Linnaeus 1758)
# uuid <- "eee50efb-40dc-47d0-b2cb-52a14a5e0e51"
# img <- get_phylopic(uuid = uuid, format = "raster", height = 869)
# Write image 
# png::writePNG(unclass(img), "inst/extdata/marmot.png")

# Load the image file locally
img_file <- system.file("extdata", "marmot.png", package = "mpmaggregate")

#For development
if (!nzchar(img_file)) {
  img_file <- "../inst/extdata/marmot.png"
}

img <- png::readPNG(img_file)

rc <- do.call(rbind, strsplit(colnames(elast_mat2), ","))
r_vec <- as.integer(rc[, 1])
c_vec <- as.integer(rc[, 2])

entry_labels <- mapply(make_entry_label, r_vec, c_vec, SIMPLIFY = FALSE)

cols <- c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e")
op <- par(mar = c(7, 6, 4, 2))

bp <- barplot(
  elast_mat2,
  beside = TRUE,
  ylim = c(0, .35),
  col = cols,
  border = NA,
  ylab = expression("Normalized elasticity of " * R[0]),
  xaxt = "n",
  space = c(0.2, 1)
)

axis(side = 1, at = colMeans(bp), labels = entry_labels, las = 2, cex.axis = 0.9)
legend("topleft", legend = rownames(elast_mat2), fill = cols, bty = "n", inset = 0.02)

lims <- par("usr")
mydiff <- lims[4] - lims[3]
add_phylopic_base(
  img = img,
  x = NULL,
  y = mydiff * .9,
  height = .2 * mydiff * dim(img)[1] / 1024
)

par(op)
```

**How to read Figure 2.** Each x-axis label corresponds to a vital rate represented 
by a matrix entry in the aggregated space. Fertility rate, \(F[1,i]\) 
denotes reproduction of age class \(i\) over the projection interval.  Survival 
probability, \(U[i,i+1]\)  gives the probability of surviving and transitioning from 
age class \(i\) to age  class \(i+1\) over the projection interval. There is exact 
agreement between the elasticities of models “Original” and “Agg \(R_0\) + elasticity,” 
which is by design. The silhouette is from PhyloPic (https://www.phylopic.org; 
Keesey, 2023) and wereadded  using the rphylopic R package (Gearty & Jones, 2023). 
The silhouette was contributed as follows: *Marmota monax* by  T. Michael Keesey 
(2012; CC0 1.0).

### Comparing elasticities of \(\lambda^k\) and \(R_0\)

Within each framework, true elasticities and those derived from aggregated matrices agree
qualitatively, regardless of aggregation method. Both frameworks identify early survival
and early fertility as dominant contributors \(\lambda^k\) and \(R_0\).

Overall, fertility rates play a larger role in determining \(R_0\) than \(\lambda^k\), accounting for
approximately 51\% versus 45\% of total elasticity mass, respectively. Across analyses,
early survival probability (the transition \(U[2,1]\)) is the single most influential vital rate.

```{r fertility-elasticity-mass}
# Mass for elasticity of R0 w.r.t. fertilities
sum(elast_mat2[, 1:5]) / 5

# Mass for elasticity of lambda^k w.r.t. fertilities
sum(elast_mat[, 1:5]) / 5
```

## References

Bienvenu, F., Akçay, E., Legendre, S., and McCandlish, D. M. (2017). The genealogical
decomposition of a matrix population model with applications to the aggregation
of stages. *Theoretical Population Biology*, 115, 69–80.
https://doi.org/10.1016/j.tpb.2017.04.002

Gearty, W., and Jones, L. A. (2023). rphylopic: An R package for fetching,
transforming, and visualising PhyloPic silhouettes. 
*Methods in Ecology and Evolution*, 14, 2700–2708.
https://doi.org/10.1111/2041-210X.14221

Hooley, D. E. (2000). Collapsed matrices with (almost) the same eigenstuff.
*The College Mathematics Journal*, 31(4), 297–299.
https://doi.org/10.1080/07468342.2000.11974162

Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application
to ten diverse animal species. *Population Ecology*, 65(3), 146–166.
https://doi.org/10.1002/1438-390X.12149

Hinrichsen, R. A., Yokomizo, H., and Salguero-Gómez, R. (2026). From theory to
application: Elasticity-consistent aggregation of Leslie matrix population
models for comparative demography. *bioRxiv*, preprint.
https://doi.org/10.64898/2026.02.04.703802

Jones, O. R., Barks, P., Stott, I., James, T. D., Levin, S., Petry, W. K.,
Capdevila, P., Che-Castaldo, J., Jackson, J., Römer, G., Schuette, C.,
Thomas, C. C., and Salguero-Gómez, R. (2022). Rcompadre and Rage—Two R
packages to facilitate the use of the COMPADRE and COMADRE databases and
calculation of life-history traits from matrix population models.
*Methods in Ecology and Evolution*, 13(4), 770–781.
https://doi.org/10.1111/2041-210X.13792

Ozgul, A., Oli, M. K., Armitage, K. B., Blumstein, D. T., and Van Vuren, D. H.
(2009). Influence of local demography on asymptotic and transient dynamics
of a yellow-bellied marmot metapopulation. *The American Naturalist*,
173(4), 517–530.
https://doi.org/10.1086/597225

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_leslie.R
\name{is_leslie}
\alias{is_leslie}
\title{Test whether a matrix is a Leslie matrix}
\usage{
is_leslie(A, tol = sqrt(.Machine$double.eps))
}
\arguments{
\item{A}{A numeric square matrix. A length-1 positive numeric will be coerced to a 1x1 matrix.}

\item{tol}{Numeric tolerance used for structural comparisons and bounds checks.}
}
\value{
Logical; \code{TRUE} if \code{A} is a Leslie matrix, otherwise \code{FALSE}.
}
\description{
Checks whether a matrix \code{A} has Leslie form: nonzero entries are allowed only in the first row
(fertility rates) and on the subdiagonal (survival probabilities). Survival probabilities
must be in \eqn{(0, 1]} (within tolerance), fertility rates must be nonnegative, and
at least one fertility rate must be positive.
}
\details{
A 1x1 matrix with a positive entry is treated as a valid (degenerate) Leslie matrix.
}
\examples{
L <- matrix(c(
  0, 2, 1,
  0.6, 0, 0,
  0, 0.7, 0
), nrow = 3, byrow = TRUE)
is_leslie(L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_aggregate.R
\name{mpm_partition}
\alias{mpm_partition}
\title{Create a partitioning matrix for MPM aggregation}
\usage{
mpm_partition(groups, n = NULL)
}
\arguments{
\item{groups}{A list of integer vectors giving stage indices for each aggregated group.}

\item{n}{Optional integer. Number of original stages. If \code{NULL}, inferred as
\code{max(unlist(groups))}. Supplying \code{n} can help detect errors when
some stage indices are missing from \code{groups}, since all indices
\code{1:n} must appear exactly once.}
}
\value{
A numeric matrix \eqn{P} of dimensionality \code{m x n} with entries 0 or 1.
}
\description{
Builds a 0/1 partitioning matrix \eqn{P} that maps an \eqn{n}-stage model to an \eqn{m}-stage
aggregated model by summing specified original stages within each aggregated stage.
}
\details{
The grouping is provided as a list of integer vectors. For example,
\code{groups = list(c(1), c(2:3))} defines \eqn{m = 2} aggregated stages:
the first contains original stage 1, and the second contains original stages 2 and 3.

Each original stage index \code{1:n} must appear exactly once across \code{groups}.
}
\examples{
g <- list(c(1), c(2:3))
P <- mpm_partition(g, n = 3)
P

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigen.R
\name{spectral_radius}
\alias{spectral_radius}
\title{Compute the spectral radius of a matrix}
\usage{
spectral_radius(A)
}
\arguments{
\item{A}{A square numeric matrix.}
}
\value{
A single numeric value giving the spectral radius of \code{A}.
}
\description{
Returns the spectral radius of a square matrix, defined as the maximum
modulus of its eigenvalues:
\deqn{\rho(A) = \max |\lambda_i|}
}
\details{
This function is primarily intended for use with matrix population models,
where the spectral radius corresponds to the stable growth rate
(i.e., \eqn{\lambda}) when the governing matrix is square and nonnegative.
It is also used to calculate net reproductive rate \eqn{R_0} from a
next generation matrix.
}
\examples{
A <- matrix(c(
  0, 1,
  0.5, 0
), nrow = 2, byrow = TRUE)

spectral_radius(A)

}
\references{
Horn, R. A. and Johnson, C. R. (2013). \emph{Matrix analysis}. Cambridge University Press.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigen.R
\name{reproductive_values}
\alias{reproductive_values}
\title{Compute reproductive values}
\usage{
reproductive_values(A, tol = 1e-12, ensure_positive = TRUE)
}
\arguments{
\item{A}{A square numeric nonnegative matrix.}

\item{tol}{Numeric tolerance used for normalization/positivity checks.}

\item{ensure_positive}{Logical. If \code{TRUE}, attempts to orient the eigenvector so its
entries are mostly positive by flipping its sign if necessary.}
}
\value{
A numeric vector of length \code{nrow(A)} giving reproductive values, scaled so
  that \code{sum(v * w) = 1}.
}
\description{
Returns the reproductive value vector \code{v} for a square nonegative matrix \code{A}, defined as the
left eigenvector associated with the dominant eigenvalue (spectral radius). The vector is
normalized in the usual demographic way so that \code{sum(v * w) = 1}, where \code{w} is the
stable stage distribution returned by \code{\link{stable_stage}}.
}
\details{
For matrix population models with nonegative irreducible matrix, reproductive values are strictly
positive (up to scaling).
}
\examples{
#irreducible example
A <- matrix(c(
  0, 1,
  0.5, 0
), nrow = 2, byrow = TRUE)

w <- stable_stage(A)
v <- reproductive_values(A)
v
sum(v * w)  # should be 1

}
\references{
Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
interpretation} (2nd ed.). Sinauer.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigen.R
\name{dominant_eigen}
\alias{dominant_eigen}
\title{Dominant eigen-elements of a population projection matrix}
\usage{
dominant_eigen(A, tol = 1e-12, ensure_positive = TRUE)
}
\arguments{
\item{A}{A square numeric matrix.}

\item{tol}{Numeric tolerance passed to \code{\link{stable_stage}} and
\code{\link{reproductive_values}}.}

\item{ensure_positive}{Logical. Passed to \code{\link{stable_stage}} and
\code{\link{reproductive_values}}. If \code{TRUE}, attempts to orient eigenvectors
so entries are mostly positive by flipping sign if needed.}
}
\value{
A named list with elements:
\itemize{
  \item \code{lambda}: spectral radius of \code{A}
  \item \code{w}: stable stage distribution (right eigenvector), \code{sum(w)=1}
  \item \code{v}: reproductive values (left eigenvector), \code{sum(v*w)=1}
}
}
\description{
Returns the dominant eigenvalue and its associated right/left eigenvectors
(stable stage distribution \code{w} and reproductive values \code{v}) for a square
population projection matrix \code{A}.
}
\details{
The output is normalized using the conventions in \code{\link{stable_stage}} and
\code{\link{reproductive_values}}:
\itemize{
  \item \code{w} is scaled so that \code{sum(w) = 1}
  \item \code{v} is scaled so that \code{sum(v * w) = 1}
}
}
\examples{
A <- matrix(c(
  0,   1,   2,
  0.5, 0,   0,
  0,   0.8, 0.9
), nrow = 3, byrow = TRUE)

dom <- dominant_eigen(A)
dom$lambda
sum(dom$w)
sum(dom$v * dom$w)

}
\references{
Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
interpretation} (2nd ed.). Sinauer.
}
\seealso{
\code{\link{spectral_radius}}, \code{\link{stable_stage}},
\code{\link{reproductive_values}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_elasticity.R
\name{mpm_elasticity}
\alias{mpm_elasticity}
\title{Elasticity of \eqn{\lambda} or \ifelse{latex}{\eqn{R_0}}{R0} with respect to entries of \code{matA}}
\usage{
mpm_elasticity(
  matA = NULL,
  matF = NULL,
  matU = NULL,
  matC = NULL,
  framework = c("lambda", "R0"),
  normalize = TRUE,
  tol = 1e-12
)
}
\arguments{
\item{matA}{A square projection matrix (numeric matrix). Required when
\code{framework = "lambda"}. Must be \code{NULL} when \code{framework = "R0"}.}

\item{matF}{A square fecundity/sexual reproduction matrix (numeric matrix).
Required when \code{framework = "R0"}. Must be \code{NULL} when
\code{framework = "lambda"}.}

\item{matU}{A square survival/growth/transition matrix (numeric matrix).
Required when \code{framework = "R0"}. Must be \code{NULL} when
\code{framework = "lambda"}.}

\item{matC}{Optional square clonal reproduction matrix (numeric matrix).
Only allowed when \code{framework = "R0"}.}

\item{framework}{Character scalar. Must be either \code{"lambda"} or \code{"R0"}.}

\item{normalize}{Logical. Must be either \code{TRUE} or \code{FALSE}. This is
relevant only when \code{framework = "R0"}. When true, the elasticity of
\code{"R0"} with respect to the entries of \code{"matA"} are scaled to
sum to 1.}

\item{tol}{Numeric tolerance used for nonnegativity checks. Default \code{1e-12}.}
}
\value{
A list with elements:
\itemize{
  \item \code{framework}: the chosen framework.
  \item \code{elasticity}: a matrix of elasticities with the same dimensionality as
    \code{matA}.
}
}
\description{
Compute elasticities of either the dominant eigenvalue \eqn{\lambda} (\eqn{\lambda}
framework) or the net reproductive rate \eqn{R_0} (\eqn{R_0} framework) with respect
to the entries of the population projection matrix \code{matA}.
}
\details{
The function enforces a strict separation of inputs by framework:
\itemize{
  \item \code{framework = "lambda"}: \code{matA} must be provided and
    \code{matF}, \code{matU}, and \code{matC} must be \code{NULL}.
  \item \code{framework = "R0"}: \code{matF} and \code{matU} must be provided,
    \code{matC} is optional, and \code{matA} must be \code{NULL}. The projection
    matrix is constructed internally as \code{matA = matR + matU}, where
    \code{matR = matF (+ matC)}.
}

Irreducibility of the resulting \code{matA} is enforced using the internal helper
\code{.check_irreducible_hj} (defined elsewhere in the package).

Eigen-elements (e.g., dominant eigenvalue, left and right eigenvectors) are
obtained via the function \code{\link{dominant_eigen}}.


Lambda framework: Elasticity is the elasticity of \eqn{\lambda} with respect
to the entries of \code{matA}.

R0 framework: Elasticity is the elasticity of \eqn{R_0} with respect
to the entries of the internally constructed \code{matA}.
}
\examples{
## Lambda framework: matA provided directly
matA <- matrix(
  c(0.2, 1.2,
    0.3, 0.4),
  nrow = 2, byrow = TRUE
)
out_lambda <- mpm_elasticity(matA = matA, framework = "lambda")
str(out_lambda)

## R0 framework: matA constructed from matF and matU
matU <- matrix(
  c(0.2, 0.0,
    0.3, 0.4),
  nrow = 2, byrow = TRUE
)
matF <- matrix(
  c(0.0, 1.2,
    0.0, 0.0),
  nrow = 2, byrow = TRUE
)
out_R0 <- mpm_elasticity(matF = matF, matU = matU, framework = "R0")
str(out_R0)

}
\references{
Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
interpretation} (2nd ed.). Sinauer.
}
\seealso{
\code{\link{dominant_eigen}}, \code{\link{spectral_radius}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpm_aggregate.R
\name{mpm_aggregate}
\alias{mpm_aggregate}
\title{Aggregate a general-to-general matrix population model}
\usage{
mpm_aggregate(
  matA = NULL,
  matU = NULL,
  matF = NULL,
  matC = NULL,
  groups,
  framework = c("lambda", "R0"),
  criterion = c("standard", "elasticity"),
  tol = 1e-12,
  ...
)
}
\arguments{
\item{matA}{Optional projection matrix \eqn{A} (square, finite, nonnegative).
If \code{NULL}, \eqn{A} is constructed as \eqn{U + (F + C)}.}

\item{matU}{Optional survival-transition matrix \eqn{U} (square, finite, nonnegative).
Required when \code{framework = "R0"} and/or when \code{matA} is \code{NULL}.}

\item{matF}{Optional fecundity matrix \eqn{F} (square, finite, nonnegative).
Interpreted as part of reproduction \eqn{R = F + C}.}

\item{matC}{Optional clonal reproduction matrix \eqn{C} (square, finite, nonnegative).
Treated as reproduction and combined with \code{matF} as \eqn{R = F + C}.}

\item{groups}{A non-empty list of integer vectors specifying aggregation
groups. Each stage in \code{1:n} must appear exactly once across the list.}

\item{framework}{Character scalar; either \code{"lambda"} or \code{"R0"}.
Determines whether aggregation is based on the projection matrix \eqn{A}
or an \eqn{R_0}-based reference matrix.}

\item{criterion}{Character scalar; either \code{"standard"} for standard aggregation
or \code{"elasticity"} for elasticity-consistent aggregation.}

\item{tol}{Numeric tolerance used in positivity/zero-mass checks and balancing
calculations for the elasticity-consistent case.}

\item{...}{Reserved for future use.}
}
\value{
A named list with elements:
\describe{
  \item{\code{framework}}{The matched framework used (\code{"lambda"} or \code{"R0"}).}
  \item{\code{criterion}}{The matched criterion used (\code{"standard"} or \code{"elasticity"}).}
  \item{\code{matA_agg}}{Aggregated projection matrix \eqn{A_{\mathrm{agg}}}.}
  \item{\code{matU_agg}}{Aggregated survival-transition matrix \eqn{U_{\mathrm{agg}}}, or \code{NULL} if \code{matU} was not supplied.}
  \item{\code{matF_agg}}{Aggregated fecundity matrix \eqn{F_{\mathrm{agg}}}, or \code{NULL} if \code{matF} was not supplied.}
  \item{\code{matC_agg}}{Aggregated clonal reproduction matrix \eqn{C_{\mathrm{agg}}}, or \code{NULL} if \code{matC} was not supplied.}
  \item{\code{effectiveness}}{Numeric effectiveness measure for the aggregation (definition depends on \code{criterion}).}
}
}
\description{
Aggregates one or more matrix population model (MPM) components from an
\eqn{n \times n} model to an \eqn{m \times m} model by combining stages into
user-defined groups.
}
\details{
Groupings are supplied by \code{groups}, a list of integer vectors specifying
how original stages (1,\dots,n) are combined; see \code{\link{mpm_partition}}.
Each original stage must appear exactly once.

Aggregation can be performed under two frameworks (\code{"lambda"} or \code{"R0"})
and two criteria (\code{"standard"} or \code{"elasticity"}), which determine how
stages are weighted during aggregation.

Clonal reproduction \code{matC} is treated as reproductive output. When both
\code{matF} and \code{matC} are supplied, the effective reproductive matrix
\eqn{R = F + C} is used internally (e.g., in the \code{"R0"} framework) but is
not returned. The aggregated effective reproduction can be recovered as
\code{matF_agg + matC_agg}.

Irreducibility requirement: The effective projection matrix \eqn{A}
must be nonnegative, square, and irreducible. This function enforces
irreducibility of \eqn{A} as either \code{matA} (if supplied) or
\eqn{A = U + (F + C)} otherwise, and will fail if reducible.


When \code{framework = "R0"}, this function requires \code{matU} and at least one
of \code{matF} or \code{matC} in order to form the effective reproductive matrix
\eqn{R = F + C} and the next generation matrix
\eqn{K = R (I - U)^{-1}}. The matrix \eqn{R} is used internally for computing
reference quantities in the \code{"R0"} framework but is not returned.
The aggregated effective reproduction can be obtained as
\code{matF_agg + matC_agg} when both components are supplied.

All returned matrices are aggregated using the same weighting rules implied by
the selected \code{framework} and \code{criterion}.

Effectiveness measures how closely the aggregated model reproduces the behavior
of the original model under the chosen \code{framework} and \code{criterion}. When
\code{effectiveness} is high (close to 1), applying the original model and then
aggregating gives nearly the same result as aggregating first and then applying the
aggregated model.

The returned element \code{effectiveness} is computed for both criteria:
\itemize{
  \item For \code{criterion = "standard"}, effectiveness is computed using weights
    based on the reference stable stage distribution \code{w}.
  \item For \code{criterion = "elasticity"}, effectiveness is computed after
    balancing transformations (with weights based on \code{w * v}).
}
}
\examples{
# Example aggregation of a 3x3 projection matrix to 2x2 using groups:
# group 1 = stage 1; group 2 = stages 2 and 3.
A <- matrix(c(
  0.0, 1.0, 2.0,
  0.5, 0.0, 0.0,
  0.0, 0.8, 0.9
), nrow = 3, byrow = TRUE)

res_std <- mpm_aggregate(
  matA = A,
  groups = list(c(1), c(2, 3)),
  framework = "lambda",
  criterion = "standard"
)
res_std$matA_agg
res_std$effectiveness

res_el <- mpm_aggregate(
  matA = A,
  groups = list(c(1), c(2, 3)),
  framework = "lambda",
  criterion = "elasticity"
)
res_el$matA_agg
res_el$effectiveness

}
\references{
Bienvenu, F., Akcay, E., Legendre, S. and McCandlish, D.M. (2017). The genealogical
decomposition of a matrix population model with applications to the aggregation
of stages. \emph{Theoretical Population Biology}, 115, 69-80.
\doi{10.1016/j.tpb.2017.04.002}

Hooley, D. E. (2000). Collapsed matrices with (almost) the same eigenstuff.
\emph{The College Mathematics Journal}, 31(4), 297-299.
\doi{10.1080/07468342.2000.11974162}

Salguero-Gomez, R. & Plotkin, J. B. (2010). Matrix dimensions bias
demographic inferences: implications for comparative plant demography.
\emph{The American Naturalist}, 176, 710-722. \doi{10.1086/657044}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generation_time.R
\name{generation_time}
\alias{generation_time}
\title{Generation time from a matrix population model}
\usage{
generation_time(
  matF,
  matU,
  matC = NULL,
  framework = c("lambda", "R0"),
  tol = 1e-12
)
}
\arguments{
\item{matF}{A square fecundity/sexual reproduction matrix (numeric matrix).
Must not be \code{NULL}.}

\item{matU}{A square survival/growth/transition matrix (numeric matrix).
Must not be \code{NULL}.}

\item{matC}{Optional square clonal reproduction matrix (numeric matrix),
same dimensions as \code{matF} and \code{matU}. Default \code{NULL}.}

\item{framework}{Character scalar. Must be either \code{"lambda"} or \code{"R0"}.}

\item{tol}{Numeric tolerance used for nonnegativity checks and near-zero screening.}
}
\value{
A list with elements:
\itemize{
  \item \code{framework}: the chosen framework (\code{"lambda"} or \code{"R0"}).
  \item \code{generation_time}: generation time (numeric scalar).
}
}
\description{
Returns a generation time for either the \eqn{\lambda} or \eqn{R_0} framework.
}
\details{
The full projection matrix \code{matA} is constructed internally as
\code{matA = matU + matF (+ matC)} and is not accepted as an input because
\code{matA} alone is not sufficient to define the required decomposition.

Eigen-elements (e.g., \eqn{\lambda}, \eqn{R_0}, stable stage distribution,
and reproductive values) are obtained via the function \code{\link{dominant_eigen}}.

If \code{matC} is supplied, the reproductive matrix is defined as
\code{matR = matF + matC}. Otherwise, \code{matR = matF}.

This function performs input validation, constructs \code{matA} and \code{matR},
and sets up the framework-specific branch for computing generation time.
}
\examples{
matU <- matrix(c(0.2, 0.0,
                0.3, 0.4), nrow = 2, byrow = TRUE)
matF <- matrix(c(0.0, 1.2,
                0.0, 0.0), nrow = 2, byrow = TRUE)
generation_time(matF = matF, matU = matU, framework = "lambda")
generation_time(matF = matF, matU = matU, framework = "R0")

}
\references{
Bienvenu, F. & Legendre, S. (2015). A new approach to the generation time in
matrix population models. \emph{The American Naturalist}, 185(6), 834-843.
\doi{10.1086/681104}

Ellner, S. P. (2018). Generation time in structured populations.
\emph{The American Naturalist}, 192(1), 105-110.
\doi{10.1086/697539}
}
\seealso{
\code{\link{dominant_eigen}}, \code{\link{spectral_radius}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigen.R
\name{stable_stage}
\alias{stable_stage}
\title{Compute the stable stage distribution}
\usage{
stable_stage(A, tol = 1e-12, ensure_positive = TRUE)
}
\arguments{
\item{A}{A square numeric matrix.}

\item{tol}{Numeric tolerance used for positivity/normalization checks.}

\item{ensure_positive}{Logical. If \code{TRUE}, attempts to orient the eigenvector so its
entries are mostly positive by flipping its sign if necessary.}
}
\value{
A numeric vector of length \code{nrow(A)} giving the stable stage distribution,
  normalized to sum to 1.
}
\description{
Returns the stable stage distribution for a square nonnegative matrix \code{A}, defined as the
right eigenvector associated with the dominant eigenvalue. The returned vector \code{w}
is normalized so that \code{sum(w) = 1}.
}
\details{
For matrix population models with nonegative irreducible matrix, stable stage densities are strictly
positive.
}
\examples{
#irreducible example
A <- matrix(c(
  0, 1,
  0.5, 0
), nrow = 2, byrow = TRUE)

w <- stable_stage(A)
w
sum(w)

}
\references{
Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
interpretation} (2nd ed.). Sinauer.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leslie_aggregate.R
\name{leslie_aggregate}
\alias{leslie_aggregate}
\title{Aggregate Leslie-to-Leslie matrix population model}
\usage{
leslie_aggregate(
  matA,
  ngroups,
  framework = c("lambda", "R0"),
  criterion = c("standard", "elasticity"),
  tol = 1e-12,
  ...
)
}
\arguments{
\item{matA}{A square numeric matrix. Must be finite, nonnegative, and satisfy
the structural constraints of a Leslie matrix (checked internally).}

\item{ngroups}{Positive integer giving the number of aggregated age groups
(the dimension \eqn{m} of the aggregated model). Must satisfy \code{ngroups <= n}.}

\item{framework}{Character scalar; either \code{"lambda"} or \code{"R0"}.
Determines whether aggregation is based on the projection matrix \eqn{A}
or an \eqn{R_0}-based reference matrix built from the implied \eqn{U} and
reproductive component of the Leslie matrix.}

\item{criterion}{Character scalar; either \code{"standard"} for standard aggregation
or \code{"elasticity"} for elasticity-consistent aggregation.}

\item{tol}{Numeric tolerance used in positivity/zero-mass checks and balancing
calculations for the elasticity-consistent case.}

\item{...}{Reserved for future use.}
}
\value{
A named list with elements:
\describe{
  \item{\code{framework}}{The matched framework used (\code{"lambda"} or \code{"R0"}).}
  \item{\code{criterion}}{The matched criterion used (\code{"standard"} or \code{"elasticity"}).}
  \item{\code{matAk_agg}}{Aggregated \eqn{k}-step Leslie matrix (\eqn{m \times m}).}
  \item{\code{effectiveness}}{A numeric effectiveness measure for the aggregation (definition depends on \code{criterion}).}
}
}
\description{
Aggregates a Leslie matrix into a smaller Leslie matrix with \code{ngroups}
contiguous age classes.
}
\details{
This function assumes the input is a valid Leslie matrix and checks the Leslie
structure internally. If the original dimensionality \eqn{n} is not an integer
multiple of \eqn{m}, a disaggregation step is applied internally so that the
expanded dimensionality is divisible by \eqn{m} before aggregation proceeds.

Aggregation can be performed under two frameworks (\code{"lambda"} or \code{"R0"})
and two criteria (\code{"standard"} or \code{"elasticity"}), which determine how
age classes are weighted during aggregation.

Irreducibility requirement: The input Leslie matrix \code{matA} must be
irreducible. This function checks irreducibility of \code{matA} and will fail
if it is reducible. (Any internal disaggregation preserves irreducibility.)


The Leslie matrix is internally decomposed into a reproductive component \eqn{R}
(the first row) and a survival component \eqn{U} (the subdiagonal and any survival
terms implied by the Leslie structure). For \code{framework = "R0"}, the
next-generation matrix \eqn{K = R (I - U)^{-1}} is formed and its dominant
eigenvalue is used as \eqn{R_0}.

Aggregation proceeds on a \eqn{k}-step matrix where \eqn{k = n/m},
with \eqn{n} the dimension of the input Leslie matrix and
\eqn{m =} \code{ngroups}. The function returns an
\eqn{m \times m} \eqn{k}-step Leslie matrix (stored as
\code{matAk_agg}) describing transitions among the aggregated age classes.

Effectiveness measures how closely the aggregated model reproduces the behavior
of the original model under the chosen \code{framework} and \code{criterion}. When
\code{effectiveness} is high (close to 1), applying the original model and then
aggregating gives nearly the same result as aggregating first and then applying the
aggregated model.

The returned element \code{effectiveness} is computed for both criteria:
\itemize{
  \item For \code{criterion = "standard"}, effectiveness is computed using weights
    based on the reference stable age distribution \code{w}.
  \item For \code{criterion = "elasticity"}, effectiveness is computed after
    balancing transformations (with weights based on \code{w * v}).
}
}
\examples{
# A simple 3x3 Leslie matrix (fertility rates in first row; survival probabilities on subdiagonal)
A <- matrix(c(
  0.0, 1.2, 1.8,
  0.5, 0.0, 0.0,
  0.0, 0.7, 0.0
), nrow = 3, byrow = TRUE)

# Aggregate to 2 age groups
res_std <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "lambda",
  criterion = "standard"
)
res_std$matAk_agg
res_std$effectiveness

res_el <- leslie_aggregate(
  matA = A,
  ngroups = 2,
  framework = "lambda",
  criterion = "elasticity"
)
res_el$matAk_agg
res_el$effectiveness

}
\references{
Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application to
ten diverse animal species. \emph{Population Ecology}, 65(3), 146-166.
\doi{10.1002/1438-390X.12149}

Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
application: Elasticity-consistent aggregation of Leslie matrix population
models for comparative demography. \emph{bioRxiv}, preprint.
\doi{10.64898/2026.02.04.703802}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matA_elephant1.R
\docType{data}
\name{matA_elephant1}
\alias{matA_elephant1}
\title{Coarse-scale Asian elephant projection matrix}
\format{
A numeric matrix with 12 rows and 12 columns.
}
\source{
COMADRE Animal Matrix Database, MatrixID 249273.
}
\usage{
matA_elephant1
}
\description{
A female-only projection matrix for an Asian elephant population,
included with the package for examples and vignettes. The matrix
has dimension 12 × 12 and a projection interval of 5 years.
}
\details{
Population/model ID: 249273 — a coarser model with a 5-year projection
interval for the Nagarhole National Park elephant population in India
(Chelliah et al., 2013).

The matrix was originally retrieved from the COMADRE Animal Matrix
Database using the `Rcompadre` package and bundled with this package
so examples and vignettes can run without requiring internet access.
}
\references{
Chelliah, K., Bukka, H., & Sukumar, R. (2013).
Modeling harvest rates and numbers from age and sex ratios:
a demonstration for elephant populations.
\emph{Biological Conservation}, 165, 54–61.
\doi{10.1016/j.biocon.2013.05.008}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpmaggregate-package.R
\docType{package}
\name{mpmaggregate-package}
\alias{mpmaggregate}
\alias{mpmaggregate-package}
\title{mpmaggregate: Aggregation of matrix population models}
\description{
\pkg{mpmaggregate} aggregates matrix population models (MPMs) under both the
\eqn{\lambda} (stable population growth rate) and \eqn{R_0} (net reproductive rate)
frameworks, using either standard or elasticity-consistent aggregation.
}
\details{
The package provides methods for aggregating matrix population models while
preserving key demographic quantities under either the \eqn{\lambda} or
\eqn{R_0} framework. Aggregation can be performed using either standard or
elasticity-consistent criteria, depending on whether reproductive values are
to be preserved in addition to growth rates.

\itemize{
  \item \strong{framework = "lambda"}: aggregation targets the dominant eigenvalue
    \eqn{\lambda} and its associated eigenvectors.
  \item \strong{framework = "R0"}: aggregation targets the dominant eigenvalue
    \eqn{R_0} of a next-generation matrix derived from reproductive and
    survival components.
  \item \strong{criterion = "standard"}: preserves \eqn{\lambda} and the stable
    stage distribution (\eqn{\lambda} framework) or \eqn{R_0} and the cohort
    stable stage distribution (\eqn{R_0} framework).
  \item \strong{criterion = "elasticity"}: additionally preserves reproductive
    values (\eqn{\lambda}) or cohort reproductive values (\eqn{R_0} framework).
}

Aggregation is implemented using effective left and right operators,
\eqn{P_{\mathrm{eff}}} and \eqn{Q_{\mathrm{eff}}}, which map an original matrix
\eqn{M} (dimension \eqn{n}) to an aggregated matrix
\eqn{M_{\mathrm{agg}}} (dimension \eqn{m}):
\deqn{M_{\mathrm{agg}} = P_{\mathrm{eff}} \, M \, Q_{\mathrm{eff}}.}

These operators are constructed from the partition matrix \eqn{P} and weights
that depend on the selected framework and criterion.

Aggregation can be performed in two primary settings:
\itemize{
  \item General-to-general matrix population model aggregation via \code{\link{mpm_aggregate}}.
    Groupings are supplied by the user as a list of stage indices (see
    \code{\link{mpm_partition}}).
  \item Leslie-to-Leslie matrix population model aggregation via
    \code{\link{leslie_aggregate}}, which aggregates age classes into cohort
    blocks and may apply a disaggregation step when required.
}

Effectiveness of aggregation measures the quality of an aggregation by how closely the
aggregated model reproduces the behavior of the original model under the chosen
framework and criterion.

The package assumes that inputs are nonnegative matrices and (when required for
computing Perron eigenvectors used in aggregation) that the population projection
matrix is irreducible.
}
\section{Main functions}{

\itemize{
  \item \code{\link{mpm_aggregate}}: general-to-general MPM aggregation using
    user-defined groups of stages.
  \item \code{\link{leslie_aggregate}}: Leslie-to-Leslie MPM aggregation into
    \code{ngroups} contiguous age classes.
  \item \code{\link{generation_time}}: return the generation time in either the
    lambda framework or R0 (cohort-based) framework.
  \item \code{\link{mpm_elasticity}}: compute elasticities of either the dominant
    eigenvalue \eqn{\lambda} (\eqn{\lambda} framework) or the net reproductive rate \eqn{R_0}
    (\eqn{R_0} framework) with respect to the entries of the population projection
    matrix \code{matA}.
}
}

\section{Utility functions}{

\itemize{
  \item \code{\link{mpm_partition}}: convert a user-supplied grouping list into a
    partitioning matrix.
  \item \code{\link{spectral_radius}}: compute the spectral radius (dominant
    eigenvalue modulus) of a square matrix.
  \item \code{\link{dominant_eigen}}: return the dominant eigenvalue and
    its associated stable stage distribution and reproductive values for a
    population projection matrix.
  \item \code{\link{leslie_dominant_eigen}}: return the dominant eigenvalue
    and its associated stable age distribution and reproductive values for a Leslie
    population projection matrix.
  \item \code{\link{is_leslie}}: check whether the input matrix is a
    Leslie population projection matrix.
  \item \code{\link{leslie_disaggregate}}: disaggregate a Leslie population projection
    model so that its dimensionality is compatible with that of the aggregated model.
  \item \code{\link{stable_stage}}: return the stable stage distribution (w) associated
    with the dominant eigenvalue, scaled so that \code{sum(w) = 1}.
  \item \code{\link{reproductive_values}}: return the reproductive values (v) associated
    with the dominant eigenvalue, scaled so that \code{sum(v * w) = 1}.
  \item \code{\link{leslie_stable_age}}: return the stable age distribution (w) associated
    with the dominant eigenvalue of a Leslie matrix, scaled so that \code{sum(w) = 1}.
  \item \code{\link{leslie_reproductive_values}}: return the reproductive values (v) associated
    with the dominant eigenvalue of a Leslie matrix, scaled so that \code{sum(v * w) = 1}.
}
}

\references{
Bienvenu, F., Akçay, E., Legendre, S. & McCandlish, D.M. (2017). The genealogical
 decomposition of a matrix population model with applications to the aggregation
 of stages. \emph{Theoretical Population Biology}, 115, 69-80. \doi{10.1016/j.tpb.2017.04.002}

 Bienvenu, F. & Legendre, S. (2015). A new approach to the generation time in matrix
 population models. \emph{The American Naturalist}, 185(6), 834-843. \doi{10.1086/681104}

 Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
 interpretation} (2nd ed.). Sinauer.

 Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models
 with application to ten diverse animal species. \emph{Population Ecology}, 65(3),
 146-166. \doi{10.1002/1438-390X.12149}

 Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
 application: Elasticity-consistent aggregation of Leslie matrix population
 models for comparative demography. \emph{bioRxiv}, preprint. \doi{10.64898/2026.02.04.703802}

 Salguero-Gómez, R., & Gamelon, M., eds. (2021). \emph{Demographic methods across the
 tree of life}. Oxford University Press.

 Salguero-Gómez, R. & Plotkin, J. B. (2010). Matrix dimensions bias
 demographic inferences: implications for comparative plant demography. \emph{The
 American Naturalist}, 176, 710-722. \doi{10.1086/657044}
}
\seealso{
\code{\link{mpm_aggregate}}, \code{\link{leslie_aggregate}}
}
\author{
\strong{Maintainer}: Richard A. Hinrichsen \email{r.hinrichsen.ecology@gmail.com} (\href{https://orcid.org/0000-0003-0761-3005}{ORCID})

Authors:
\itemize{
  \item Roberto Salguero-Gomez \email{rob.salguero@zoo.ox.ac.uk} (\href{https://orcid.org/0000-0002-6085-4433}{ORCID})
  \item Hiroyuki Yokomizo \email{hiroyuki.yokomizo@nies.go.jp} (\href{https://orcid.org/0000-0001-9814-8465}{ORCID})
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leslie_disaggregate.R
\name{leslie_disaggregate}
\alias{leslie_disaggregate}
\title{Disaggregate a Leslie matrix population model to a compatible dimensionality}
\usage{
leslie_disaggregate(A, m)
}
\arguments{
\item{A}{A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
is treated as a 1x1 Leslie matrix.}

\item{m}{Target aggregated dimensionality (positive integer).}
}
\value{
A Leslie matrix of dimensionality \eqn{n_{new} \times n_{new}},
  where \eqn{n_{new} = \mathrm{LCM}(n, m)}.
}
\description{
Expands an \eqn{n \times n} Leslie matrix \code{A} to an
\eqn{n_{new} \times n_{new}} Leslie matrix, where
\eqn{n_{new} = \mathrm{LCM}(n, m)}.
}
\details{
Disaggregation is required when aggregating a Leslie model to dimensionality \code{m}
whenever \code{m} does not divide \code{n} evenly. The expanded matrix introduces a finer
age structure that is compatible with both dimensionalities.

Each original age class is subdivided into
\eqn{n_{new} / n} sub-classes. Fertility rates from original age class \eqn{i}
are placed at the end of the corresponding block, deterministic aging within blocks
is represented by ones on the subdiagonal, and original survival probabilities are
inserted at block boundaries.
}
\examples{
L <- matrix(c(
  0, 2, 1,
  0.6, 0, 0,
  0, 0.7, 0
), nrow = 3, byrow = TRUE)
leslie_disaggregate(L, m = 2)

}
\references{
Hinrichsen, R. A. (2023). Aggregation of Leslie matrix models with application to
ten diverse animal species. \emph{Population Ecology}, 65(3), 146-166.
\doi{10.1002/1438-390X.12149}

Hinrichsen, R. A., Yokomizo, H., & Salguero-Gómez, R. (2026). From theory to
application: Elasticity-consistent aggregation of Leslie matrix population
models for comparative demography. \emph{bioRxiv}, preprint.
\doi{10.64898/2026.02.04.703802}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigen.R
\name{leslie_reproductive_values}
\alias{leslie_reproductive_values}
\title{Reproductive values for a Leslie matrix}
\usage{
leslie_reproductive_values(A)
}
\arguments{
\item{A}{A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
will be coerced to a 1x1 matrix.}
}
\value{
A numeric vector of length \code{nrow(A)} giving reproductive values, normalized so that
  \code{sum(v * w) = 1}.
}
\description{
Computes the reproductive value vector \code{v} for a Leslie matrix \code{A}, scaled so that
\code{sum(v * w) = 1}, where \code{w} is the stable age distribution returned by
\code{\link{leslie_stable_age}}.
}
\details{
This implementation follows Demetrius (1974) equation expressed in terms of
the stable age distribution and fertility rates.

A length-1 positive numeric is treated as a 1x1 Leslie matrix and returns \code{1}.
}
\examples{
leslie_reproductive_values(1)

L <- matrix(c(
  0, 2, 1,
  0.6, 0, 0,
  0, 0.7, 0
), nrow = 3, byrow = TRUE)
v <- leslie_reproductive_values(L)
w <- leslie_stable_age(L)
sum(v * w)  # should be 1

}
\references{
Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
interpretation} (2nd ed.). Sinauer.

Demetrius, L. (1974). Demographic parameters and natural selection.
\emph{Proceedings of the National Academy of Sciences}, 71(12), 4645-4647.
\doi{10.1073/pnas.71.12.4645}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matA_elephant2.R
\docType{data}
\name{matA_elephant2}
\alias{matA_elephant2}
\title{Fine-scale Asian elephant projection matrix}
\format{
A numeric matrix with 60 rows and 60 columns.
}
\source{
COMADRE Animal Matrix Database, MatrixID 249274.
}
\usage{
matA_elephant2
}
\description{
A female-only projection matrix for an Asian elephant population,
included with the package for examples and vignettes. The matrix
has dimension 60 × 60 and a projection interval of 1 year.
}
\details{
Population/model ID: 249274 — a fine-grained model with a 1-year
projection interval for the Periyar Reserve elephant population
in India (Goswami et al., 2014).

The matrix was originally retrieved from the COMADRE Animal Matrix
Database using the `Rcompadre` package and bundled with this package
so examples and vignettes can run without requiring internet access.
The final entry was zeroed out so that the matrix is a true Leslie
matrix.
}
\references{
Goswami, V. R., Vasudev, D., & Oli, M. K. (2014).
The importance of conflict-induced mortality for conservation planning
in areas of human–elephant co-occurrence.
\emph{Biological Conservation}, 176, 191–198.
\doi{10.1016/j.biocon.2014.05.026}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eigen.R
\name{leslie_dominant_eigen}
\alias{leslie_dominant_eigen}
\title{Dominant eigen-elements of a Leslie matrix}
\usage{
leslie_dominant_eigen(A)
}
\arguments{
\item{A}{A Leslie matrix (checked with \code{\link{is_leslie}}). A length-1 positive numeric
will be coerced to a 1x1 matrix.}
}
\value{
A named list with elements:
\itemize{
  \item \code{lambda}: spectral radius of \code{A}
  \item \code{w}: stable age distribution, \code{sum(w)=1}
  \item \code{v}: reproductive values, \code{sum(v*w)=1}
}
}
\description{
Returns the dominant eigenvalue (\eqn{\lambda}) and the stable age distribution
\code{w} and reproductive values \code{v} for a Leslie matrix \code{A}.
}
\details{
Normalization:
The output is normalized using the conventions in \code{\link{leslie_stable_age}} and
\code{\link{leslie_reproductive_values}}
\itemize{
  \item \code{w} is scaled so that \code{sum(w) = 1}
  \item \code{v} is scaled so that \code{sum(v * w) = 1}
}
}
\examples{
leslie_dominant_eigen(1)

L <- matrix(c(
  0, 2, 1,
  0.6, 0, 0,
  0, 0.7, 0
), nrow = 3, byrow = TRUE)
dom <- leslie_dominant_eigen(L)
dom$lambda
sum(dom$w)
sum(dom$v * dom$w)

}
\references{
Caswell, H. (2001). \emph{Matrix population models: construction, analysis and
interpretation} (2nd ed.). Sinauer.

Demetrius, L. (1974). Demographic parameters and natural selection.
\emph{Proceedings of the National Academy of Sciences}, 71(12), 4645-4647.
\doi{10.1073/pnas.71.12.4645}
}
\seealso{
\code{\link{spectral_radius}}, \code{\link{leslie_stable_age}},
\code{\link{leslie_reproductive_values}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leslie_disaggregate.R
\name{LCM}
\alias{LCM}
\title{Least common multiple of two integers}
\usage{
LCM(a, b)
}
\arguments{
\item{a}{A positive integer.}

\item{b}{A positive integer.}
}
\value{
A positive integer giving the LCM of \code{a} and \code{b}.
}
\description{
Computes the least common multiple (LCM) of two positive integers.
Used by \code{\link{leslie_disaggregate}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leslie_disaggregate.R
\name{GCD}
\alias{GCD}
\title{Greatest common divisor of a set of integers}
\usage{
GCD(x)
}
\arguments{
\item{x}{A numeric vector of positive integers.}
}
\value{
A positive integer giving the GCD of \code{x}.
}
\description{
Computes the greatest common divisor (GCD) of a numeric vector \code{x} of positive integers.
Used by \code{\link{LCM}}.
}
\keyword{internal}
